package jelite;

import java.io.*;
import liaquay.collection.*;
import jelite.hulls.*;

public class GalaxySeed
{
  private static final int[] SATELLITES_FOR_SIZE     = { 0, 1, 2, 4, 8, 8 };

  private static final int[] SATELLITE_SIZE_FOR_SIZE = { 0, 0, 1, 2, 2, 4 };

  private static final long[] PERIOD_FOR_ORBIT = { 800000,
                                                   1600000,
                                                   8000000,
                                                   16000000,
                                                   80000000,
                                                   80000000,
                                                   80000000,
                                                   800000000,
                                                   800000000 };

  private static final Renderable[] PLANET_FOR_SIZE = { Renderables.PLANET1,
                                                        Renderables.PLANET2,
                                                        Renderables.PLANET3,
                                                        Renderables.PLANET4,
                                                        Renderables.PLANET5,
                                                        Renderables.PLANET6 };

  private static final Renderable[] STAR_FOR_SIZE = { Renderables.STAR1,
                                                      Renderables.STAR1,
                                                      Renderables.STAR1,
                                                      Renderables.STAR1,
                                                      Renderables.STAR1,
                                                      Renderables.STAR1 };

  private static final double ORBITAL_RADIUS = 120000;

  private static String _digrams ="ABOUSEITILETSTONLONUTHNOALLEXEGEZACEBISOUSESARMAINDIREA?ERATENBERALAVETIEDORQUANTEISRION";

  int _a = 0x4a;
  int _b = 0x5a;
  int _c = 0x48;
  int _d = 0x02;
  int _e = 0x53;
  int _f = 0xb7;
  int _identifier = 0;

  boolean _carryFlag = false;

  public GalaxySeed()
  {
  }

  public GalaxySeed( GalaxySeed gs )
  {
    _a = gs._a;
    _b = gs._b;
    _c = gs._c;
    _d = gs._d;
    _e = gs._e;
    _f = gs._f;
    _carryFlag = gs._carryFlag;
  }

  public void initGalaxy1()
  {
    _a = 0x4a;
    _b = 0x5a;
    _c = 0x48;
    _d = 0x02;
    _e = 0x53;
    _f = 0xb7;
  }

  public void restore( GalaxySeed gs )
  {
    _a = gs._a;
    _b = gs._b;
    _c = gs._c;
    _d = gs._d;
    _e = gs._e;
    _f = gs._f;
    _carryFlag = gs._carryFlag;
  }

  private void waggle()
  {
    int x = _a + _c;
    int y = _b + _d;

    if (x > 0xFF)
    {
      y++;
    }

    x = x & 0xFF;
    y = y & 0xFF;

    _a = _c;
    _b = _d;
    _c = _e;
    _d = _f;

    x += _c;
    y += _d;

    if (x > 0xFF)
    {
      y++;
    }

    if (y > 0xFF)
    {
      _carryFlag = true;
    }
    else
    {
      _carryFlag = false;
    }

    x = x & 0xFF;
    y = y & 0xFF;

    _e = x;
    _f = y;
  }

  public String toString()
  {
    return "" + _a + " " + _b + " " + _c + " " + _d + " " + _e + " " + _f + " " + _carryFlag;
  }

  public Star generateStar( int x, int y, int z ) // throws NoSuchStarException
  {
    for( int i = 0; i < 256; ++i )
    {
      int sx = getStarX();
      int sy = getStarY();
      int sz = getStarZ();

      if( ( sx == x ) && ( sy == y ) && ( sz == z ) )
      {
        return generateStar();
      }
      else
      {
        next();
      }
    }
    System.out.println( "Could not find star x=" + x + " y=" + y + " z=" + z );
    return null;
  }

  public Star generateStar()
  {
    GalaxySeed save = new GalaxySeed( this );

    int x = getStarX();
    int y = getStarY();
    int z = getStarZ();
    String name = nameStar();
    int size = getStarSize();
    Star star = new Star( name, x, y, z, size, _identifier++, STAR_FOR_SIZE[ size ] );

    next();
    int planets = 1 + _e % ( SATELLITES_FOR_SIZE[ size + 1 ] + 1 );
    for( int i = 0; i < planets; ++i )
    {
      Satellite s = generateSatellite( size, i, ORBITAL_RADIUS * ( i + 1 ) );
      star.addSatellite( s );
    }

    addSpaceStation( star );

    restore( save );
    return star;
  }

  public Satellite generateSatellite( int sizeOfParent, int orbit, double orbitalRadius )
  {
    next();
    int size = _d % ( SATELLITE_SIZE_FOR_SIZE[ sizeOfParent ] + 1 );

    Satellite satellite = new Satellite( PERIOD_FOR_ORBIT[ orbit ],
                                         orbitalRadius,
                                         _identifier++,
                                         PLANET_FOR_SIZE[ size ] );

    int moons = _e % ( SATELLITES_FOR_SIZE[ size ] + 1 );
    for( int i = 0; i < moons; ++i )
    {
      Satellite s = generateSatellite( size, i, orbitalRadius * ( i + 1 ) / 8 );
      satellite.addSatellite( s );
    }
    return satellite;
  }

  public void addSpaceStation( Star star )
  {
    ArrayList satellites = star.getSatellites();
    Satellite satellite = null;
    int orbit = -1;
    while( satellites.size() > 0 )
    {
      ++orbit;
      satellite = (Satellite)satellites.get( ( ( (int)_c) & 0xff ) % satellites.size() );
      next();
      satellites = satellite.getSatellites();
    }
    if( satellite != null )
    {
      Renderable renderable;
      if( ( _d & 1 ) != 0 )
      {
        renderable = Renderables.CORIOLIS;
      }
      else
      {
        renderable = Renderables.DODO;
      }

      SpaceStationSatellite spaceStation =
        new SpaceStationSatellite( PERIOD_FOR_ORBIT[ orbit ],
                                   satellite.getOrbitalRadius() / 8,
                                   _identifier++,
                                   renderable );

      satellite.addSatellite( spaceStation );
    }
  }

  private String nameStar()
  {

    int size;
    StringBuffer gp = new StringBuffer();
    int x;

    if( (_a & 0x40 ) == 0 )
    {
      size = 3;
    }
    else
    {
      size = 4;
    }

    for( int i = 0; i < size; i++ )
    {
      x = _f & 0x1F;
      if (x != 0)
      {
        x += 12;
        x *= 2;
        gp.append( _digrams.charAt( x ) );

        if( _digrams.charAt( x + 1 ) != '?' )
        {
          gp.append( _digrams.charAt( x + 1 ) );
        }
      }

      waggle();
    }

    for( int i = 1; i < gp.length(); ++i )
    {
      gp.setCharAt( i, Character.toLowerCase( gp.charAt( i ) ) );
    }

    return gp.toString();
  }

  private int getStarX()
  {
    return _d - 128;
  }

  private int getStarY()
  {
    return _b -128;
  }

  private int getStarZ()
  {
    return _e - 128;
  }

  private int getStarSize()
  {
    return ( _f & 1 ) + 2 + ( _carryFlag ? 1 : 0 );
  }

  public void next()
  {
    waggle();
    waggle();
    waggle();
    waggle();
  }
}