package jelite;

import java.io.*;
import java.awt.*;
import liaquay.collection.*;
import jelite.hulls.*;

public class Satellite
{
  private ArrayList _satellites = new ArrayList();
  private long _period;
  private long _offset = 0;
  private double _orbitalRadius;
  private int _identifier;
  private Renderable _renderable;
  private double _x;
  private double _y;
  private double _z;

  public Satellite( long period, double orbitalRadius, int identifier, Renderable renderable )
  {
    _period = period;
    _orbitalRadius = orbitalRadius;
    _renderable = renderable;
    _identifier = identifier;

  }

  public String toString()
  {
    return "period = " + _period + " " +
           "orbitalRadius = " + _orbitalRadius;
  }

  public final void addSatellite( Satellite satellite )
  {
    _satellites.add( satellite );
  }

  public double getRadius()
  {
    return _renderable.getRadius();
  }

  public long getPeriod()
  {
    return _period;
  }

  public double getOrbitalRadius()
  {
    return _orbitalRadius;
  }

  public Renderable getRenderable()
  {
    return _renderable;
  }

  public ArrayList getSatellites()
  {
    return _satellites;
  }

  public final void calculatePosition( long ms, double parentX, double parentY, double parentZ )
  {
    double angle =  ( 2 * Math.PI * ( ( ms + _offset ) % _period ) / _period );
    double ca = Math.cos( angle );
    double sa = Math.sin( angle );

    _x = parentX + ( ca * _orbitalRadius );
    _y = parentY;
    _z = parentZ + ( sa * _orbitalRadius );

    for( int i = 0; i < _satellites.size(); ++i )
    {
      Satellite satellite = (Satellite)_satellites.get( i );
      satellite.calculatePosition( ms, _x, _y, _z );
    }
  }

  public final double getX()
  {
    return _x;
  }

  public final double getY()
  {
    return _y;
  }

  public final double getZ()
  {
    return _z;
  }

  public int getIdentifier()
  {
    return _identifier;
  }

}
