package jelite.client;

import java.applet.AudioClip;
import java.applet.Applet;
import java.net.*;

import liaquay.collection.*;
import jelite.Parameters;

class AudioClipMap extends IdentifierMap
{
  private int _playing = -1;

  public AudioClipMap()
  {
    try
    {
      put( Parameters.SOUND_EXPLOSION,   Applet.newAudioClip( new URL( "file:audio/explosion.au"   ) ) );
      put( Parameters.SOUND_ENGINE,      Applet.newAudioClip( new URL( "file:audio/engine.au"      ) ) );
      put( Parameters.SOUND_NEW_MESSAGE, Applet.newAudioClip( new URL( "file:audio/newmessage.au"  ) ) );
      put( Parameters.SOUND_HYPERSPACE,  Applet.newAudioClip( new URL( "file:audio/hyperspace.au"  ) ) );
      put( Parameters.SOUND_MISSILE,      Applet.newAudioClip( new URL( "file:audio/missile.au"      ) ) );
      put( Parameters.SOUND_FIRE,        Applet.newAudioClip( new URL( "file:audio/fire.au"        ) ) );
    }
    catch( MalformedURLException e )
    {
      e.printStackTrace();
    }
  }

  public AudioClipMap( Applet applet )
  {
    put( Parameters.SOUND_EXPLOSION,   applet.getAudioClip( applet.getCodeBase(), "audio/explosion.au"   ) );
    put( Parameters.SOUND_ENGINE,      applet.getAudioClip( applet.getCodeBase(), "audio/engine.au"      ) );
    put( Parameters.SOUND_NEW_MESSAGE, applet.getAudioClip( applet.getCodeBase(), "audio/newmessage.au"  ) );
    put( Parameters.SOUND_HYPERSPACE,  applet.getAudioClip( applet.getCodeBase(), "audio/hyperspace.au"  ) );
    put( Parameters.SOUND_MISSILE,      applet.getAudioClip( applet.getCodeBase(), "audio/missile.au"      ) );
    put( Parameters.SOUND_FIRE,        applet.getAudioClip( applet.getCodeBase(), "audio/fire.au"        ) );
  }

  public void play( int identifier )
  {
    if( identifier != _playing )
    {
      stop();
      _playing = identifier;
      AudioClip audioClip = (AudioClip)get( identifier );
      audioClip.play();
      _playing = -1;
    }
  }

  public void stop()
  {
    if( _playing >= 0 )
    {
      AudioClip audioClip = (AudioClip)get( _playing );
      audioClip.stop();
      _playing = -1;
    }
  }

  public void loop( int identifier )
  {
    if( identifier != _playing )
    {
      stop();
      _playing = identifier;
      AudioClip audioClip = (AudioClip)get( identifier );
      audioClip.loop();
    }
  }

  public boolean getPlaying()
  {
    return _playing != -1;
  }
}

public class ClientSoundMap
{
  private AudioClipMap[] _soundChannels = new AudioClipMap[ Parameters.NUMBER_OF_SOUND_CHANNELS ];

  private static ClientSoundMap _soundMap = null;


  public static ClientSoundMap getClientSoundMap()
  {
    return _soundMap;
  }

  public static void initialize( Applet applet )
  {
    _soundMap = new ClientSoundMap( applet );
  }

  public ClientSoundMap( Applet applet )
  {
    for( int i = 0; i < _soundChannels.length; ++i )
    {
      _soundChannels[ i ] = new AudioClipMap( applet );
    }
  }

  public ClientSoundMap()
  {
    for( int i = 0; i < _soundChannels.length; ++i )
    {
      _soundChannels[ i ] = new AudioClipMap();
    }
  }



  public void play( int identifier )
  {
    for( int i = 0; i < _soundChannels.length; ++i )
    {
      if( _soundChannels[ i ].getPlaying() == false )
      {
        _soundChannels[ i ].play( identifier );
        return;
      }
    }
  }

  public void stop( int channel )
  {
     _soundChannels[ channel ].stop();
  }

  public void loop( int identifier, int channel )
  {
    _soundChannels[ channel ].loop( identifier );
  }

  public void stopAll()
  {
    for( int i = 0; i < _soundChannels.length; ++i )
    {
      _soundChannels[ i ].stop();
    }
  }
}