package jelite.client;

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.EventObject.*;
import java.awt.event.*;

import jelite.*;
import jelite.hulls.*;
import jelite.engine.galaxy.Galaxy;
import jelite.engine.galaxy.*;
import jelite.engine.player.Player;
import liaquay.collection.*;

public class JEliteApplet extends Applet implements Runnable
{
  static final int STATE_INITIAL      = 0;
  static final int STATE_SPLASH       = 1;
  static final int STATE_CONNECTED    = 2;
  static final int STATE_DOCKED       = 3;
  static final int STATE_FLYING       = 4;

  private int _state = STATE_INITIAL;
  private ConnectedPanel _connectedPanel = null;
  private DockedPanel _dockedPanel = null;
  private FlyingPanel _flyingPanel = null;
  //private ClientSoundMap _clientSoundMap = null;

  private StarField _starField = new StarField();
  private Galaxy _galaxy = null;
  private Player _player = null;
  private EliteCraft _craft = null;

  Thread _thread = null;
  boolean _running = false;

  public Galaxy getGalaxy()
  {
    return _galaxy;
  }

  public StarField getStarField()
  {
    return _starField;
  }

  public Player getPlayer()
  {
    return _player;
  }

  public void setPlayer( Player player )
  {
    _player = player;
  }

  public EliteCraft getCraft()
  {
    return _craft;
  }

  public void setCraft( EliteCraft craft )
  {
    _craft = craft;
  }

  public String getAppletInfo()
  {
    return "JEliteApplet V1.0 by Philip M. Scull";
  }

  private final void enterState( int state )
  {
    switch( state )
    {
      case STATE_INITIAL:
        break;
      case STATE_SPLASH:
        System.out.println( "Entering state splash" );
        //_splashPanel.setVisible( true );
        break;
      case STATE_CONNECTED:
        System.out.println( "Entering state login" );
        _galaxy = new Galaxy( _starField.getStarMap() );
        _connectedPanel.setVisible( true );
        break;
      case STATE_DOCKED:
        System.out.println( "Entering state docked" );
        _dockedPanel.setVisible( true );
        break;
      case STATE_FLYING:
        System.out.println( "Entering state flying" );
        _running = true;
        _thread = new Thread( this );
        _thread.start();
        _flyingPanel.setVisible( true );
        break;
    }
    validate();
    repaint();
  }

  private final void exitState( int state )
  {
    switch( state )
    {
      case STATE_INITIAL:
        break;
      case STATE_SPLASH:
        System.out.println( "Exiting state splash" );
        //_splashPanel.setVisible( false );
        break;
      case STATE_CONNECTED:
        System.out.println( "Exiting state login" );
        _connectedPanel.setVisible( false );
        break;
      case STATE_DOCKED:
        System.out.println( "Exiting state docked" );
        _dockedPanel.setVisible( false );
        break;
      case STATE_FLYING:
        System.out.println( "Exiting state flying" );
        _running = false;
        /*try
        {
          _thread.join();
        }
        catch( InterruptedException e )
        {
        }*/
        _flyingPanel.setVisible( false );
        break;
    }
  }

  final void changeState( int state )
  {
    if( _state != state )
    {
      exitState( _state );
      _state = state;
      enterState( _state );
    }
  }

  public void init()
  {
    LoadOnDemandHull.setDocumentBase( getDocumentBase() );


    setLayout( new NiceBorderLayout(0,0) );
    //_splashPanel = new SplashPanel( this );
    _dockedPanel = new DockedPanel( this );
    _flyingPanel = new FlyingPanel( this );
    _connectedPanel = new ConnectedPanel( this );

    //setLayout( null );
    add( _connectedPanel, BorderLayout.CENTER );
    //add( _splashPanel, BorderLayout.CENTER );
    add( _dockedPanel, BorderLayout.CENTER );
    add( _flyingPanel, BorderLayout.CENTER );
    _connectedPanel.setVisible( false );
    //_splashPanel.setVisible( false );
    _dockedPanel.setVisible( false );
    _flyingPanel.setVisible( false );
    changeState( STATE_CONNECTED );
  }

  public void start()
  {
  }

  public void stop()
  {
    System.out.println( "STOP!" );

    changeState( STATE_INITIAL );
  }

  public void run()
  {
    ClientSoundMap.initialize( this );

    long lastTimeMillis = System.currentTimeMillis();
    int averagePeriod = 160;

    while( _running )
    {
      try
      {
        Thread.sleep( 20 );
      }
      catch( InterruptedException e )
      {
      }

      long millis = System.currentTimeMillis();
      long period = millis - lastTimeMillis;
      lastTimeMillis = millis;
      averagePeriod -= averagePeriod / 16;
      averagePeriod += period;

      if( millis > 0 )
      {
        _galaxy.update( millis, averagePeriod / 16 );
        _flyingPanel.paintNow();

        if( !_player.isAlive() )
        {
          changeState( STATE_CONNECTED );
        }
      }
    }
  }
}