package jelite.client;

import java.awt.*;
import liaquay.collection.*;

public class NiceBorderLayout implements LayoutManager2
{
    int hgap;                 //horizontal gap
    int vgap;                 //vertical gap
    Component _north = null;
    Component _south = null;
    /**
     * Creates a graph paper layout with a default of a 1 x 1 graph, with no
     * vertical or horizontal padding.
     */
    public NiceBorderLayout()
    {
        this(5,5);
    }

    /**
     * Creates a graph paper layout with the given grid size and padding.
     * @param gridSize size of the graph paper in logical units (n x m)
     * @param hgap horizontal padding
     * @param vgap vertical padding
     */
    public NiceBorderLayout(int hgap, int vgap)
    {
        this.hgap = hgap;
        this.vgap = vgap;
    }

    /**
     * Adds the specified component with the specified name to
     * the layout.  This does nothing in GraphPaperLayout, since constraints
     * are required.
     */
    public void addLayoutComponent(String name, Component comp) {
    }

    /**
     * Removes the specified component from the layout.
     * @param comp the component to be removed
     */
    public void removeLayoutComponent(Component comp) {
    }

    /**
     * Calculates the preferred size dimensions for the specified
     * panel given the components in the specified parent container.
     * @param parent the component to be laid out
     *
     * @see #minimumLayoutSize
     */
    public Dimension preferredLayoutSize(Container parent) {
        return getLayoutSize(parent, true);
    }

    /**
     * Calculates the minimum size dimensions for the specified
     * panel given the components in the specified parent container.
     * @param parent the component to be laid out
     * @see #preferredLayoutSize
     */
    public Dimension minimumLayoutSize(Container parent) {
        return getLayoutSize(parent, false);
    }

    /**
     * Algorithm for calculating layout size (minimum or preferred).
     * <p>
     * The width of a graph paper layout is the largest cell width
     * (calculated in <code>getLargestCellSize()</code> times the number of
     * columns, plus the horizontal padding times the number of columns
     * plus one, plus the left and right insets of the target container.
     * <p>
     * The height of a graph paper layout is the largest cell height
     * (calculated in <code>getLargestCellSize()</code> times the number of
     * rows, plus the vertical padding times the number of rows
     * plus one, plus the top and bottom insets of the target container.
     *
     * @param parent the container in which to do the layout.
     * @param isPreferred true for calculating preferred size, false for
     *                    calculating minimum size.
     * @return the dimensions to lay out the subcomponents of the specified
     *         container.
     * @see java.awt.GraphPaperLayout#getLargestCellSize
     */
    protected Dimension getLayoutSize(Container parent, boolean isPreferred) {
 /*       Dimension largestSize = new Dimension( 10,10 ); // HACK

        Insets insets = parent.getInsets();
        largestSize.width = ( largestSize.width * gridSize.width ) +
            ( hgap * ( gridSize.width + 1 ) ) + insets.left + insets.right;
        largestSize.height = ( largestSize.height * gridSize.height ) +
            ( vgap * ( gridSize.height + 1 ) ) + insets.top + insets.bottom; */
        return parent.getSize();
    }

    /**
     * Lays out the container in the specified container.
     * @param parent the component which needs to be laid out
     */
    public void layoutContainer(Container parent)
    {
        synchronized (parent.getTreeLock())
        {
            Insets insets = parent.getInsets();
            int ncomponents = parent.getComponentCount();

            if (ncomponents == 0)
            {
                return;
            }

            // Total parent dimensions
            Dimension size = parent.getSize();
System.out.println( "Parent size " + size );

            int totalW = size.width - (insets.left + insets.right);
            int totalH = size.height - (insets.top + insets.bottom);

            // The heights of the top and bottom components
            int northHeight = 0;
            int southHeight = 0;

            if( _north != null )
            {
              northHeight = _north.getPreferredSize().height;
              _north.setBounds( insets.left, insets.top, totalW, northHeight );
              System.out.println( "North left=" + insets.left + " top=" + insets.top + " width=" +  totalW + " height=" + northHeight );
            }

            if( _south != null )
            {
              southHeight = _south.getPreferredSize().height;
              _south.setBounds( insets.left, size.height - insets.bottom - southHeight, totalW, southHeight );
              System.out.println( "South left=" + insets.left + " top=" + (size.height - insets.bottom - southHeight) + " width=" +  totalW + " height=" + southHeight );
            }

            int x = insets.left;
            int y = insets.top + northHeight + vgap;
            int w = totalW;
            int h = totalH - northHeight - southHeight - vgap - vgap
            ;
            for ( int i = 0; i < ncomponents; i++ )
            {
                Component c = parent.getComponent(i);
                if( ( c != _north ) && ( c != _south ) )
                {
                   c.setBounds(x, y, w, h);
              System.out.println( "Center left=" + x + " top=" + y + " width=" + w + " height=" + h );
                }
            }
        }
    }

    // LayoutManager2 /////////////////////////////////////////////////////////

    /**
     * Adds the specified component to the layout, using the specified
     * constraint object.
     * @param comp the component to be added
     * @param constraints  where/how the component is added to the layout.
     */
    public void addLayoutComponent(Component comp, Object constraints)
    {
        System.out.println( "Adding with constraint " + constraints );
        if( constraints.equals( BorderLayout.NORTH ) )
        {
          System.out.println( "Adding north" );
          _north = comp;
        }
        else if( constraints.equals( BorderLayout.SOUTH ) )
        {System.out.println( "Adding south" );
          _south = comp;
        }
    }

    /**
     * Returns the maximum size of this component.
     * @see java.awt.Component#getMinimumSize()
     * @see java.awt.Component#getPreferredSize()
     * @see LayoutManager
     */
    public Dimension maximumLayoutSize(Container target) {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }

    /**
     * Returns the alignment along the x axis.  This specifies how
     * the component would like to be aligned relative to other
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public float getLayoutAlignmentX(Container target) {
        return 0.5f;
    }

    /**
     * Returns the alignment along the y axis.  This specifies how
     * the component would like to be aligned relative to other
     * components.  The value should be a number between 0 and 1
     * where 0 represents alignment along the origin, 1 is aligned
     * the furthest away from the origin, 0.5 is centered, etc.
     */
    public float getLayoutAlignmentY(Container target) {
        return 0.5f;
    }

    /**
     * Invalidates the layout, indicating that if the layout manager
     * has cached information it should be discarded.
     */
    public void invalidateLayout(Container target) {
        // Do nothing
    }
}
