package jelite.client;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import liaquay.collection.*;
import liaquay.graphics.*;
import jelite.*;
import jelite.engine.galaxy.Craft;

public class StarMapCanvas extends Canvas
{
  private JEliteApplet _applet = null;
  private float _offsetX = 0;
  private float _offsetY = 0;
  private float _scale = 2;
  private Image _image = null;
  private Selection _selection;
  private float craftX = 0;
  private float craftY = 0;

  public StarMapCanvas( Selection selection, JEliteApplet applet )
  {
    _image = createImage( 20, 20 ); // This will get destroyed by resize
    _selection = selection;
    _applet = applet;

    addMouseListener( new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
        Dimension d = getSize();
        int width = d.width;
        int height = d.height;
        boolean leftButton = ( e.getModifiers() & MouseEvent.BUTTON1_MASK ) !=  0;
        boolean rightButton = ( e.getModifiers() & MouseEvent.BUTTON3_MASK ) !=  0;
        int pickX = e.getX() - (width / 2);
        int pickY = e.getY() - (height / 2);

        if( leftButton )
        {
          int starX = (int)( ( (float)pickX / _scale ) - _offsetX );
          int starY = (int)( ( (float)pickY / _scale ) - _offsetY );
          StarField starField = _applet.getStarField();
          ClientStar clientStar = starField.nearest( starX, starY );
          if( clientStar != null )
          {
            _selection.select( clientStar );
          }
        }
        if( rightButton )
        {
          _offsetX -= (float)pickX / _scale;
          _offsetY -= (float)pickY / _scale;
        }
        repaint();
      }
    });
  }

  public final void zoomIn()
  {
    _scale *= 1.5;
    repaint();
  }

  public final void zoomOut()
  {
    _scale /= 1.5;
    repaint();
  }

  public final void panToSelection()
  {
    Iterator it = _selection.iterator();
    
    if( it.hasNext() )
    {
      Object o = it.next();
      
      if( o instanceof ClientStar )
      {
        ClientStar clientStar = (ClientStar)o;

        Star star = clientStar.getStar();
        _offsetX = -(float)star.getX();
        _offsetY = -(float)star.getY();
      }
    }
  }

  public final void update( Graphics g )
  {
    paint( g );
  }

  public final void paint( Graphics g )
  {
    Craft craft = _applet.getCraft();
    int sectorX = craft.getSectorX();
    int sectorY = craft.getSectorY();
    int sectorZ = craft.getSectorZ();
    setCraftPosition( sectorX, sectorY, sectorZ );
  }

  public final void setCraftPosition( int sectorX, int sectorY, int sectorZ )
  {
    Graphics g = _image.getGraphics();
    Dimension d = getSize();
    int width = d.width;
    int height = d.height;
    g.setColor( Color.black );
    g.fillRect( 0, 0, width, height );

    g.translate( width / 2, height / 2 );
    StarField starField = _applet.getStarField();
    ArrayList stars = starField.getStars();
    for( int i = 0; i < stars.size(); ++i )
    {
      ClientStar clientStar = (ClientStar)stars.get( i );
      Star star = clientStar.getStar();
      int w = (int)( (float)star.getClassification() * _scale / 4 );
      int x = (int)( ( _offsetX + (float)star.getX() ) * _scale );
      int y = (int)( ( _offsetY + (float)star.getY() ) * _scale );

      if( ( x > - ( width  / 2 ) ) && ( x < ( width  / 2 ) ) &&
          ( y > - ( height / 2 ) ) && ( y < ( height / 2 ) ) )
      {
        g.setColor( Color.white );
        if( w > 1 ) // Size of 1 does not render on NT!
        {
          g.fillOval( x - (w/2), y - (w/2), w, w );
        }
        else
        {
          g.fillRect( x, y, 1, 1 );
        }
        if( _scale > 5 )
        {
          g.setColor( Color.green );
          g.drawString( star.getName(), x + (w/2) + 9, y - (w/2) );

          if( _scale > 10 )
          {
            g.drawString( "[" + star.getX() + "," + star.getY() + "," + star.getZ() + "]", x + (w/2) + 9, y - (w/2) + 12 );
          }
        }

        if( _selection.isSelected( clientStar ) )
        {
          g.setColor( Color.green );
          int r = (w/2) + 4;
          g.drawLine( x + r, y + r, x + r, y - r );
          g.drawLine( x + r, y - r, x - r, y - r );
          g.drawLine( x - r, y - r, x - r, y + r );
          g.drawLine( x - r, y + r, x + r, y + r );
        }
      }

      float sx = (float)sectorX / (float)Parameters.SOLAR_SYSTEM_SIZE_IN_SECTORS;
      float sy = (float)sectorY / (float)Parameters.SOLAR_SYSTEM_SIZE_IN_SECTORS;
      float sz = (float)sectorZ / (float)Parameters.SOLAR_SYSTEM_SIZE_IN_SECTORS;

      int cx = (int)( ( _offsetX + sx ) * _scale );
      int cy = (int)( ( _offsetY + sy ) * _scale );

      if( ( cx > - ( width  / 2 ) ) && ( cx < ( width  / 2 ) ) &&
          ( cy > - ( height / 2 ) ) && ( cy < ( height / 2 ) ) )
      {
        g.setColor( Color.green );
        g.drawLine( cx - 20, cy, cx + 20, cy );
        g.drawLine( cx, cy - 20, cx, cy + 20 );
        if( _scale > 10 )
        {
          g.drawString( "[" + (int)sx + "," + (int)sy + "," + (int)sz + "]", cx + 9, cy + 15 );
        }
      }
    }
    g.translate( - width / 2, - height / 2 );
    getGraphics().drawImage( _image, 0, 0, null );
  }

  public void setSize( int width, int height )
  {
    super.setSize( width, height );
    _image = createImage( width, height );
  }

  public void setSize( Dimension d )
  {
    int width = d.width;
    int height = d.height;
    _image = createImage( width, height );
    super.setSize( width, height );
  }

  public void setBounds( int x, int y, int w, int h )
  {
    _image = createImage( w, h );
    super.setBounds( x,y,w,h);
  }
}