package jelite.client;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
//import jelite.*;
import jelite.engine.galaxy.*;
import jelite.Star;
import jelite.Satellite;
import jelite.hulls.*;
import jelite.graphics.*;
import liaquay.graphics.*;
import liaquay.math.vector.*;
import liaquay.collection.*;

public class ViewCanvas extends Panel
{
  private Image _image = null;
  private Graphics3D _graphics3D = null;
  private Camera3D _camera = new Camera3D( new Transform(), 1.0 );
  private Transform _transform = new Transform();
  private Selection _selection = new Selection();
  private boolean _pick = false;
  private int _pickX;
  private int _pickY;
  private float _enginePower = 1f;
  private JEliteApplet _applet;

  public final void rotateTranslate( double a, double b, double c, double x, double y, double z )
  {
    _transform.rotateTranslate( a, b, c, x, y, z );
    requestFocus();
  }

  public ViewCanvas( Selection selection, JEliteApplet applet )
  {
    _selection = selection;
    _applet = applet;

    rotateTranslate( 0, 0, 0, 0, 0, 0 );

    addKeyListener( new KeyAdapter()
    {
      int lastKeyCode = KeyEvent.CHAR_UNDEFINED;

      public void keyPressed( KeyEvent e )
      {
        if( lastKeyCode != e.getKeyCode() )
        {
          switch( e.getKeyChar() )
          {
            case 'p': _pick = true; _pickX = 0; _pickY = 0; break;
            case 'a': _applet.getCraft().fire( 0, 1 ); break;
            case ' ': _applet.getCraft().setAccelerate( true ); break;
            case '/': _applet.getCraft().setDecelerate( true ); break;
            case ',': _applet.getCraft().setThrusterPower( 2, _enginePower ); break;
            case '.': _applet.getCraft().setThrusterPower( 3, _enginePower ); break;
            case 's': _applet.getCraft().setThrusterPower( 6, _enginePower ); break;
            case 'x': _applet.getCraft().setThrusterPower( 7, _enginePower ); break;
            case 'l': lockSelection();break;
            case 'm': fireMissile();break;
          }

          char c = e.getKeyChar();
          if( ( c >= '1' ) && ( c <= '8' ) )
          {
            _applet.getCraft().setThrusterPower( (int)(c - '1'), _enginePower );
          }

          lastKeyCode = e.getKeyCode();
        }
      }

      public void keyReleased( KeyEvent e )
      {
        lastKeyCode = KeyEvent.CHAR_UNDEFINED;

        switch( e.getKeyChar() )
        {
          case ' ': _applet.getCraft().setAccelerate( false ); break;
          case '/': _applet.getCraft().setDecelerate( false ); break;
          case ',': _applet.getCraft().setThrusterPower( 2, 0 ); break;
          case '.': _applet.getCraft().setThrusterPower( 3, 0 ); break;
          case 's': _applet.getCraft().setThrusterPower( 6, 0 ); break;
          case 'x': _applet.getCraft().setThrusterPower( 7, 0 ); break;
        }

        char c = e.getKeyChar();
        if( ( c >= '1' ) && ( c <= '8' ) )
        {
          _applet.getCraft().setThrusterPower( (int)(c - '1'), 0 );
        }
      }
    });

    addMouseListener( new MouseAdapter()
    {
       public void mouseClicked(MouseEvent e)
       {
          Dimension d = getSize();
         _pickX = e.getX();
         _pickY = e.getY();
         _pick = true;
       }
    });
  }

  public final void update( Graphics g )
  {
  }

  public final void paint( Graphics g )
  {
  }

  public void paintNow()
  {

    if( _graphics3D != null )
    {
      _image = _graphics3D.getImage();

      if( _pick )
      {
        _graphics3D.pick( _pickX, _pickY );
        _pick = false;
      }

      EliteCraft craft = _applet.getCraft();
      StarField starField = _applet.getStarField();

      _transform.mul( craft.getTransform(), _camera.getTransform() );

      _graphics3D.beginScene( _camera );

      prepare( _applet.getStarField(), craft );

      _graphics3D.endScene();
/*
      Graphics graphics = _image.getGraphics();
      graphics.setColor( Color.white );
      graphics.drawString( "Shields " + craft.getShieldsPercentage(), 10, 20 );
      graphics.drawString( "Engine " + (int)(craft.getThrusterPower(0) * 100) , 10, 40 );
*/
      getGraphics().drawImage( _image, 0, 0, null );

    }
  }

  private void rebuild( int width, int height )
  {

    _graphics3D = new EliteGraphics3D( width, height, _selection );
  }

  public void setSize( int width, int height )
  {
    rebuild( width, height );
    super.setSize( width, height );
  }

  public void setSize( Dimension d )
  {
    int width = d.width;
    int height = d.height;
    rebuild( width, height );
    super.setSize( width, height );
  }

  public void setBounds( int x, int y, int width, int height )
  {
    super.setBounds( x,y,width,height);
    rebuild( width, height );
  }

  private final void prepare( StarField starField,
                              Craft craft )
  {
    Sector sector = craft.getSector();

    SolarSystem solarSystem = sector.getSolarSystem();

    int originX = sector.getX();
    int originY = sector.getY();
    int originZ = sector.getZ();

    starField.setOrigin( originX, originY, originZ );

    ArrayList stars = starField.getStars();

    for( int i = 0; i < stars.size(); ++i )
    {
      ClientStar clientStar = (ClientStar)stars.get( i );

      Star star = clientStar.getStar();

      if( ( solarSystem.getX() !=  star.getX() ) ||
          ( solarSystem.getY() !=  star.getY() ) ||
          ( solarSystem.getZ() !=  star.getZ() ) )
      {
        clientStar.render( _graphics3D );
      }
    }

    addSolarSystemToView( sector, solarSystem );

    addSectorsToView( sector );
  }

  private final void addSolarSystemToView( Sector origin, SolarSystem solarSystem )
  {
    int originX = origin.getX();
    int originY = origin.getY();
    int originZ = origin.getZ();

    ArrayList satellites = solarSystem.getSatellites();

    for( int i = 0; i < satellites.size(); ++i )
    {

      Item item = (Item)satellites.get( i );

      Sector sector = item.getSector();

      double sectorOffsetX = Sector.SIZE * ( sector.getX() - originX );
      double sectorOffsetY = Sector.SIZE * ( sector.getY() - originY );
      double sectorOffsetZ = Sector.SIZE * ( sector.getZ() - originZ );

      Transform transform = item.getTransform();
      Transform localTransform = new Transform( transform );

      localTransform.setDx( localTransform.getDx() + sectorOffsetX );
      localTransform.setDy( localTransform.getDy() + sectorOffsetY );
      localTransform.setDz( localTransform.getDz() + sectorOffsetZ );

      item.getRenderable().render(
        _graphics3D,
        localTransform,
        item );
    }
  }

  private final void addSectorsToView( Sector sector )
  {
    Sector[][][] neighbours = sector.getNeighbours();

    int originX = sector.getX();
    int originY = sector.getY();
    int originZ = sector.getZ();

    addSectorToView( sector, originX, originY, originZ );
    for( int i = 0; i < 3; ++ i )
    {
      for( int j = 0; j < 3; ++j )
      {
        for( int k = 0; k < 3; ++k )
        {
          Sector neighbour = neighbours[ i ][ j ][ k ];

          if( neighbour != null )
          {
            addSectorToView( neighbour, originX, originY, originZ );
          }
        }
      }
    }
  }

  private final void addSectorToView( Sector sector,
                                      int originX,
                                      int originY,
                                      int originZ )
  {
    double sectorOffsetX = Sector.SIZE * ( sector.getX() - originX );
    double sectorOffsetY = Sector.SIZE * ( sector.getY() - originY );
    double sectorOffsetZ = Sector.SIZE * ( sector.getZ() - originZ );

    for( int i = 0; i < sector.getNumberOfItems(); ++i )
    {
      Item item = sector.getItem( i );

      Transform transform = item.getTransform();
      Transform localTransform = new Transform( transform );

      localTransform.setDx( localTransform.getDx() + sectorOffsetX );
      localTransform.setDy( localTransform.getDy() + sectorOffsetY );
      localTransform.setDz( localTransform.getDz() + sectorOffsetZ );

      item.getRenderable().render(
        _graphics3D,
        localTransform,
        item );
    }
  }

  private void lockSelection()
  {
    Iterator i = _selection.iterator();

    if( i.hasNext() )
    {
      Object o = i.next();

      if( o instanceof Craft )
      {
        _applet.getCraft().setLock( (Craft)o );
        return;
      }
    }
     _applet.getCraft().setLock( null );

  }

  private void fireMissile()
  {
    _applet.getCraft().fireMissile();

  }
}
