package jelite.engine.galaxy;

import java.awt.*;
import liaquay.math.vector.*;
import jelite.hulls.*;
import liaquay.collection.*;
import jelite.engine.*;
import jelite.Parameters;
import jelite.engine.player.Player;

public class Craft extends Item
{
  protected Thruster[] _thrusters = null;
  private Player _player = null;
  protected Transform _velocity = new Transform();

  public Craft( Renderable renderable, Transform transform )
  {
    super( renderable, transform );
  }

  // Used to decide when to mop the transforms.
  private int _mopcount = 0;

  public Transform getVelocity()
  {
    return _velocity;
  }

  public void update( long ms, long period, Transform parentTransform )
  {
    if( _player != null )
    {
      _player.update( ms, period );
    }

    if( _thrusters != null )
    {
      _velocity = new Transform();
      boolean thrusting = false;

      for( int i = 0; i < _thrusters.length; ++i )
      {
        Thruster thruster = _thrusters[ i ];

        if( thruster.burning() )
        {
          _velocity = _velocity.inertiaMul( thruster.getTransform( _transform, period ) );
          thrusting = true;
        }
      }

      if( thrusting )
      {
        //_inertia = thrust.inertiaMul( _inertia );

        setTransform( getTransform().inertiaMul( _velocity ) );
        if( _mopcount++ > 100 )
        {
          _transform.mop();
          _mopcount = 0;
        }
      }
    }
    super.update( ms, period, parentTransform );
  }

  public final void setThrusterPower( int index, double power )
  {
    if( index < _thrusters.length )
    {
      _thrusters[ index ].setPower( power );
    }
  }

  public final double getThrusterPower( int index )
  {
    if( index < _thrusters.length )
    {
      return _thrusters[ index ].getPower();
    }
    else
    {
      return 0;
    }
  }

  public void collision( Item item, long period )
  {
    if( item instanceof SpaceStationItem )  // Not in C++ !
    {
      System.out.println( "Docking" + this );
      // Dock with the space station.
      SpaceStationItem spaceStation = (SpaceStationItem)item;
      spaceStation.dock( this );
      setRemove( true );
    }
    else
    {
      craftDestroyed();
    }
  }

  public void fire( int weapon, double power )
  {
  }


  /**
   * Called to handle all events when this craft is destoyed
   */
  public final void craftDestroyed()
  {

    genocide(); // Kill all the crew
    setDeceased(); // Destroy the ship

    // Add an explosion
    _sector.addNewItem( new Nova( _transform, _renderable.getRadius() ) );
  }

  /**
   * Kill all crew members. This happens if the craft is destroyed.
   */
  public final void genocide()
  {
    if( _player != null )
    {
      _player.killed();
    }
  }

  /**
   * Switch all the thrusters off
   */
  public final void allThrustersOff()
  {
    for( int i = 0; i < _thrusters.length; ++i )
    {
      Thruster thruster = _thrusters[ i ];
      thruster.setPower( 0 );
    }
  }

  /**
   * Launch the craft.
   */
  public final void launch()
  {
  }

  /**
   * Adds a crew member to a craft.
   * @returns false if there is no more room aboard.
   */
  public final void addCrewMember( Player player )
  {
    _player = player;
  }

  public Player getPlayer()
  {
    return _player;
  }

  /**
   * Engage hyperspace engine.
   * Coords are absolute sector.
   */
  public final void hyperspace( int x, int y, int z )
  {
    addSound( Parameters.SOUND_HYPERSPACE, 3000, false );

    double dx = (x - _sector.getX() ) * Sector.SIZE;
    double dy = (y - _sector.getY() ) * Sector.SIZE;
    double dz = (z - _sector.getZ() ) * Sector.SIZE;
    _transform.setDx( _transform.getDx() + dx );
    _transform.setDy( _transform.getDy() + dy );
    _transform.setDz( _transform.getDz() + dz );
  }
}
