package jelite.engine.galaxy;

import liaquay.math.vector.*;
import jelite.hulls.*;
import jelite.Parameters;

public class EliteCraft extends Craft
{
  private static final int MAX_SHIELDS = 2000;
  private GunMount _gunMount;
  private MissileLauncherMount _missileMount;
  private Craft _lock = null;
  private int _shields = MAX_SHIELDS;
  private boolean _accelerate = false;
  private boolean _decelerate = false;
  private int _missiles = 3;

  public int getShieldsPercentage()
  {
    return (_shields * 100) / MAX_SHIELDS;
  }

  public EliteCraft( Renderable renderable, Transform transform )
  {
    super( renderable, transform );

    _thrusters = new Thruster[ 8 ];

    _thrusters[ 0 ] = new Thruster( 0,   0,   1,   800,      0    );
    _thrusters[ 1 ] = new Thruster( 0,   0,   1,  -800,      0    );
    _thrusters[ 2 ] = new Thruster( 0,   0,   1,     0,      0.6f );
    _thrusters[ 3 ] = new Thruster( 0,   0,   1,     0,     -0.6f );
    _thrusters[ 4 ] = new Thruster( 0,   1,   0,     0,      0.6f );
    _thrusters[ 5 ] = new Thruster( 0,   1,   0,     0,     -0.6f );
    _thrusters[ 6 ] = new Thruster( 1,   0,   0,     0,      0.6f );
    _thrusters[ 7 ] = new Thruster( 1,   0,   0,     0,     -0.6f );

    Transform t = new Transform();
    t.translate( 0, 0, 320 );
    _gunMount = new GunMount( t );
    _missileMount = new MissileLauncherMount( t );

    _gunMount.mount( new PlasmaGun() );
    _missileMount.mount( new MissileLauncher() );
  }

  public void fire()
  {
    addSound( Parameters.SOUND_FIRE, 1000, false );
    _gunMount.fire( getSector(), getTransform(), _velocity );
  }

  public void fireMissile()
  {
    if( _lock != null )
    {
      if( !_lock.getDeceased() )
      {
        addSound( Parameters.SOUND_MISSILE, 1000, false );
        _missileMount.fire( getSector(), getTransform(), _velocity, _lock );
        _missiles--;
      }
    }
  }

  public int getNumberOfMissiles()
  {
    return _missiles;
  }

  public void fire( int weapon, double power )
  {
    fire();
    //debug();
  }

  public final void debug()
  {
    System.out.println( "\nPosition" );
    System.out.println( _transform );
    System.out.println( "\nVelocity" );
    System.out.println( _velocity );
    System.out.println( "Sector " + _sector );
  }

  public void setLock( Craft target )
  {
    _lock = target;
  }

  public void collision( Item item, long period )
  {

    if( item instanceof SpaceStationItem )  // Not in C++ !
    {
      System.out.println( "Docking" + this );
      // Dock with the space station.
      SpaceStationItem spaceStation = (SpaceStationItem)item;
      spaceStation.dock( this );
      setRemove( true );
    }
    else
    {
      _shields -= period;

      if( _shields < 0 )
      {
        craftDestroyed();
      }
    }
  }

  public void update( long ms, long period, Transform parentTransform )
  {
    if( _accelerate )
    {
      long time = System.currentTimeMillis();

      double p = getThrusterPower( 0 );

      p += (double)( period ) / 4000;

      if( p > 1.0 ) p = 1.0;

      setThrusterPower( 0, p );
    }
    else if( _decelerate )
    {
      double p = getThrusterPower( 0 );

      p -= (double)( period ) / 4000;

      if( p < -1.0 ) p = -1.0;

      setThrusterPower( 0, p );
    }

    super.update(ms,period,parentTransform);
  }

  public void setAccelerate( boolean accelerate )
  {
    _accelerate = accelerate;
  }

  public void setDecelerate( boolean decelerate )
  {
    _decelerate = decelerate;
  }

}