package jelite.engine.galaxy;

import java.io.*;
import java.awt.*;

import jelite.*;
import liaquay.collection.*;
import jelite.hulls.*;

public class Galaxy extends ArrayList
{
  private ArrayList _starList = new ArrayList();
  private StarMap _starMap = null;

  public Galaxy( StarMap starMap )
  {
    _starMap = starMap;
  }

  private final SolarSystem findSolarSystem( int x, int y, int z )
  {
    for( int i = 0; i < length(); ++i )
    {
      SolarSystem s = (SolarSystem)get( i );
      if( s.isSolarSystem( x, y, z ) )
      {
        return s;
      }
    }
    return null;
  }

  private final SolarSystem createSolarSystem( int x, int y, int z )
  {
    Star star = _starMap.getStar( x, y, z );

    SolarSystem s = new SolarSystem( this, x, y, z, star );

    add( s );

    return s;
  }

  public final SolarSystem getSolarSystem( int x, int y, int z )
  {
    SolarSystem s = findSolarSystem( x, y, z );

    if( s == null )
    {
      s = createSolarSystem( x, y, z );
    }

    return s;
  }

  public Sector getSector( SolarSystem solarSystem, int sectorX, int sectorY, int sectorZ )
  {
    int solarSystemX = SolarSystem.sectorToGalaxyCoordinate( sectorX );
    int solarSystemY = SolarSystem.sectorToGalaxyCoordinate( sectorY );
    int solarSystemZ = SolarSystem.sectorToGalaxyCoordinate( sectorZ );

    if( solarSystem == null )
    {
      solarSystem = getSolarSystem( solarSystemX, solarSystemY, solarSystemZ );
    }
    else
    {
      if( ( solarSystem.getX() != solarSystemX ) ||
          ( solarSystem.getY() != solarSystemY ) ||
          ( solarSystem.getZ() != solarSystemZ ) )
      {
        solarSystem = getSolarSystem( solarSystemX, solarSystemY, solarSystemZ );
      }
    }

    return solarSystem.getSector( sectorX, sectorY, sectorZ );
  }

  public Sector findSector( SolarSystem solarSystem, int sectorX, int sectorY, int sectorZ )
  {
    int solarSystemX = SolarSystem.sectorToGalaxyCoordinate( sectorX );
    int solarSystemY = SolarSystem.sectorToGalaxyCoordinate( sectorY );
    int solarSystemZ = SolarSystem.sectorToGalaxyCoordinate( sectorZ );

    if( solarSystem == null )
    {
      solarSystem = getSolarSystem( solarSystemX, solarSystemY, solarSystemZ );
    }
    else
    {
      if( ( solarSystem.getX() != solarSystemX ) ||
          ( solarSystem.getY() != solarSystemY ) ||
          ( solarSystem.getZ() != solarSystemZ ) )
      {
        solarSystem = findSolarSystem( solarSystemX, solarSystemY, solarSystemZ );
      }
    }

    if( solarSystem == null )
    {
      return null;
    }
    else
    {
      return solarSystem.findSector( sectorX, sectorY, sectorZ );
    }
  }

  public final void update( long ms, long period )
  {
    for( int i = 0; i < length(); ++i )
    {
      SolarSystem s = (SolarSystem)get( i );
      s.update( ms, period );
    }

    for( int i = 0; i < length(); ++i )
    {
      SolarSystem s = (SolarSystem)get( i );
      s.moveItems();
    }

    for( int i = 0; i < length(); ++i )
    {
      SolarSystem s = (SolarSystem)get( i );
      s.interact( ms, period );
    }

    for( int i = 0; i < length(); ++i )
    {
      SolarSystem s = (SolarSystem)get( i );

      if( s.getItemCount() == 0 )
      {
        System.out.println( "Removing solar system " + s.getName() );
        remove( s );
        System.out.println( "Number of Solar Systems " + length() );
      }
    }
  }
}