package jelite.engine.galaxy;

import liaquay.math.vector.*;
import jelite.hulls.*;

public class InertialItem extends Item
{
  protected Transform _inertia = new Transform();

// Tidy constructors
  public InertialItem(
    Renderable renderable,
    float a, float b, float c, float x, float y, float z )
  {
    super( renderable, new Transform() );
    _transform.rotateTranslate( a, b, c, x, y, z  );
  }

// Tidy constructors
  public InertialItem(
    Renderable renderable,
    float a, float b, float c, float x, float y, float z,
    float da, float db, float dc, float dx, float dy, float dz )
  {
    super( renderable, new Transform() );
    _transform.rotateTranslate( a, b, c, x, y, z  );
    _inertia.rotateTranslate( da, db, dc, dx, dy, dz );
  }


// These should be the only constructors.
  public InertialItem( Renderable renderable, Transform transform )
  {
    super( renderable, transform );
  }

  public InertialItem(
    Renderable renderable,
    Transform transform,
    Transform inertia )
  {
    super( renderable, transform );
    _inertia = inertia;
  }

  public final void stop()
  {
    _inertia.identity();
  }

private int _mopcount = 0;
  public void update( long ms, long period, Transform parentTransform )
  {
    Transform scaledInertia = _inertia.scale( (float)period / 1000 );
//    _transform = scaledInertia.inertiaMul( _transform );
    _transform = _transform.inertiaMul( scaledInertia );
    if( _mopcount++ > 100 )
    {
      _transform.mop();
      _mopcount = 0;
    }
    super.update( ms, period, parentTransform );
  }

  public void setInertia( Transform u )
  {
    _inertia = u;
  }
}