package jelite.engine.galaxy;

import java.awt.*;

import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.collection.*;
import jelite.hulls.*;
import jelite.Parameters;

public class Item implements SelectionListener
{
  protected Renderable _renderable;
  protected Transform _transform;
  protected Sector _sector = null;
  private boolean _remove = false;           // Use this to remove but not destroy item
  private boolean _deceased = false;         // Use this to remove and destroy item
  private Item[] _items = new Item[ 1 ];
  private int _length = 0;
  private boolean _scenery = false;
  private String _name = "Anonymous";
  private Hashtable _collisions = new Hashtable(5);

  public final void addCollision( Item item, Collision c )
  {
    _collisions.put( item, c );
  }

  public final Object removeCollision( Item item )
  {
    return _collisions.remove( item );
  }

  public final Collision getCollision( Item item )
  {
    return (Collision)_collisions.get( item );
  }

  public Hashtable.HashtableValueIterator getCollisionsIterator()
  {
    return _collisions.hashtableValueIterator();
  }

  public int getNumberOfCollisions()
  {
    return _collisions.size();
  }

  public void explode()
  {
    _sector.addSound( Parameters.SOUND_EXPLOSION, 1000, false );
    setDeceased();
    _sector.addNewItem( new Nova( _transform, _renderable.getRadius() ) );
  }

  public final void setName( String name )
  {
    _name = name;
  }

  public final String getName()
  {
    return _name;
  }

  public final void setScenery( boolean scenery )
  {
    _scenery = scenery;
  }

  public boolean getScenery()
  {
    return _scenery;
  }

  public final void setSector( Sector s )
  {
    _sector = s;
  }

  public final Sector getSector()
  {
    return _sector;
  }

  public final void setRemove( boolean remove )
  {
    _remove = remove;
  }

  public final boolean getRemove()
  {
    return _remove;
  }

  public final void setDeceased()
  {
    _deceased = true;
    _remove = true;
  }

  public boolean getDeceased()
  {
    return _deceased;
  }

  public final void add( Item o )
  {
    if( _length == _items.length )
    {
      Item t[] = new Item[ _items.length * 2 ];
      for( int i = 0; i < _items.length; ++i )
      {
        t[ i ] = _items[ i ];
      }
      _items = t;
    }
    _items[ _length++ ] = o;
  }

  public Item( Renderable renderable, Transform transform )
  {
    _renderable = renderable;
    _transform = transform;
  }

  public void update( long ms, long period, Transform parentTransform )
  {
    Transform t;

    if( parentTransform == null )
    {
      t = _transform;
    }
    else
    {
      t = parentTransform.mul( _transform );
    }

    for( int i = 0; i < _length; ++i )
    {
      _items[ i ].update( ms, period, t );
    }
  }

  public void interact( Item item )
  {
  }

  public boolean checkCollision( Item item )
  {
    double dx = _transform.getDx() - item._transform.getDx() + ( ( _sector.getX() - item._sector.getX() ) * Sector.SIZE );
    double dy = _transform.getDy() - item._transform.getDy() + ( ( _sector.getY() - item._sector.getY() ) * Sector.SIZE );
    double dz = _transform.getDz() - item._transform.getDz() + ( ( _sector.getZ() - item._sector.getZ() ) * Sector.SIZE );

    double dd = ( dx * dx ) + ( dy * dy ) + ( dz * dz );
    double r = ( _renderable.getRadius() + item._renderable.getRadius() );
    double rr = r * r;

    return dd <= rr;
  }

  public void getRelativePosition( Item item, Vector3D vector )
  {
    double dx = item._transform.getDx() - _transform.getDx() + ( ( item._sector.getX() - _sector.getX() ) * Sector.SIZE );
    double dy = item._transform.getDy() - _transform.getDy() + ( ( item._sector.getY() - _sector.getY() ) * Sector.SIZE );
    double dz = item._transform.getDz() - _transform.getDz() + ( ( item._sector.getZ() - _sector.getZ() ) * Sector.SIZE );

    vector.set( 0, dx );
    vector.set( 1, dy );
    vector.set( 2, dz );
  }

  public void collision( Item item, long period )
  {
  }

  public static final int itemToSectorCoordinate( double v )
  {
    if( v > 0 )
    {
      return (int)( ( v + ( Sector.SIZE / 2 ) ) / Sector.SIZE);
    }
    else
    {
      return (int)( ( v - ( Sector.SIZE / 2 ) ) / Sector.SIZE);
    }
  }

  public final int getSectorX()
  {
    return itemToSectorCoordinate( _transform.getDx() );
  }

  public final int getSectorY()
  {
    return itemToSectorCoordinate( _transform.getDy() );
  }

  public final int getSectorZ()
  {
    return itemToSectorCoordinate( _transform.getDz() );
  }

  public final void trimToSector()
  {
    _transform.setDx( _transform.getDx() - ( Sector.SIZE * getSectorX() ) );
    _transform.setDy( _transform.getDy() - ( Sector.SIZE * getSectorY() ) );
    _transform.setDz( _transform.getDz() - ( Sector.SIZE * getSectorZ() ) );
  }

  public Transform getTransform()
  {
    return _transform;
  }

  public void setTransform( Transform t )
  {
    _transform = t;
  }

  public final Renderable getRenderable()
  {
    return _renderable;
  }

  public void addSound( int soundIdentifier, long period, boolean loop )
  {
    jelite.client.ClientSoundMap map = jelite.client.ClientSoundMap.getClientSoundMap();

    if( map != null )
    {
      map.play( soundIdentifier );
    }
  }

  public void selected( Selection selection ) {}

  public void deselected( Selection selection ) {}
}
