package jelite.engine.galaxy.pilot;

import liaquay.collection.ArrayList;

public class Action
{
  // When this action was created. May be useful for bordem algorithms.
  private long _birthdate;

  // This is a list of the active actions this action is carring out.
  ArrayList _subActions = new ArrayList(5);

  // This indicates if this action has been aborted.
  private boolean _aborted = false;

  // This indicates if this action has been completed.
  private boolean _completed = false;

  // When this action expires
  private long _longevity;

  /**
   * Create an action supplying its birthdate.
   */
  public Action( long birthdate, long longevity )
  {
    _birthdate = birthdate;
    _longevity = longevity;
  }

  /**
   * Carry out this action.
   */
  public void perform( long ms, long period )
  {

  }

  public long getBirthdate()
  {
    return _birthdate;
  }

  public void setAborted()
  {
    _aborted = true;
  }

  public void setCompleted()
  {
    _completed = true;
  }

  public boolean getAborted()
  {
    return _aborted;
  }

  public boolean getCompleted()
  {
    return _completed;
  }

  /**
   * Service this action.
   */
  public void service( long ms, long period )
  {
    // Check if we have got bored
    _aborted = _aborted || ( ms > ( _birthdate + _longevity ) );


    if( _aborted || _completed )
    {
      return;
    }

    // Now perform this action
    perform( ms, period );

    // First perform any sub actions
    for( int i = 0; i < _subActions.size(); ++i )
    {
      Action action = (Action)_subActions.get( i );

      if( action._aborted || action._completed )
      {
        _subActions.remove( action );
      }
      else
      {
        action.service( ms, period );
      }
    }
  }

  public ArrayList getSubActions()
  {
    return _subActions;
  }

  public void add( Action action )
  {
    _subActions.add( action );
  }

  public void clearSubActions()
  {
    _subActions.removeAll();
  }

  public boolean isPerforming( Class c )
  {
    for( int i = 0; i < _subActions.size(); ++i )
    {
      if( _subActions.get( i ).getClass().equals( c ) )
      {
        return true;
      }
    }
    return false;
  }
}
