package jelite.engine.galaxy.pilot;

import jelite.engine.galaxy.*;
import liaquay.math.vector.*;

public class FaceAwayFromItemAction extends CraftAction
{
  private double _engineSetting;

  // The item we are trying to avoid
  private Item _item;

  // The inverse of the self transform
  private Transform _inverseSelf = new Transform();

  // Relative position of item to self in world coordinates
  private Vector3D _relativePositionWorld = new Vector3D(0,0,0);

  // The relative position of the enemy craft
  private Vector3D _relativePosition = new Vector3D(0,0,0);

  /**
   * Evade an item
   */
  public FaceAwayFromItemAction( long birthdate,
                                  long longevity,
                                  Craft self,
                                  Item item,
                                  double engineSetting )
  {
    super( birthdate, longevity, self );
    _engineSetting = engineSetting;
    _item = item;
  }

  public void perform( long ms, long period )
  {
   // System.out.println( "Performing FaceAwayFromItemAction action" );
    Craft self = getSelf();

    if( self.getDeceased() || _item.getDeceased() )
    {
      setCompleted();
      return;
    }

    self.getTransform().inverse( _inverseSelf );

    self.getRelativePosition( _item, _relativePositionWorld );

    if( _relativePositionWorld.lengthSquared() > 4000*4000 )
    {
      setCompleted();
      return;
    }

    _relativePositionWorld.rot( _inverseSelf, _relativePosition );

    self.setThrusterPower( 0, _engineSetting / 4 );
    self.setThrusterPower( 1, 0.0 );

    double x = _relativePosition.get(0);
    double y = _relativePosition.get(1);

    if( (x < 5) && (x > -5) )
    {
      self.setThrusterPower( 2, 0.0 );
      self.setThrusterPower( 3, 0.0 );
    }
    else if( ( x > 0 ) == ( y > 0 ) )
    {
      self.setThrusterPower( 2, 0.0 );
      self.setThrusterPower( 3, 1.0 );
    }
    else
    {
      self.setThrusterPower( 2, 1.0 );
      self.setThrusterPower( 3, 0.0 );
    }

    double modx = x >= 0 ? x : -x;
    double mody = y >= 0 ? y : -y;

    if( modx < 40 )
    {
      if( mody > 40 )
      {
        if( y < 0 )
        {
          self.setThrusterPower( 6, 0.0 );
          self.setThrusterPower( 7, 1.0 );
        }
        else
        {
          self.setThrusterPower( 6, 1.0 );
          self.setThrusterPower( 7, 0.0 );
        }
      }
      else
      {
        self.setThrusterPower( 0, _engineSetting );
      }
    }
    else
    {
      self.setThrusterPower( 6, 0.0 );
      self.setThrusterPower( 7, 0.0 );
    }
  }
}

