package jelite.engine.galaxy.pilot.craft;

import jelite.engine.galaxy.*;
import liaquay.math.vector.*;
import jelite.engine.galaxy.pilot.*;

public class EngagementAction extends CraftAction
{
  // This is the craft this action is attacking
  private Craft _enemy;

  // The relative position of the enemy craft
  private Vector3D _relativePosition = new Vector3D(0,0,0);

  private long _lastWeaponDischage = 0;

  /**
   * Create an engage the enemy action supplying its birthdate.
   */
  public EngagementAction( long birthdate, Craft self, Craft enemy )
  {
    super( birthdate, 60000, self );
    _enemy = enemy;

  }

  // The inverse of the self transform
  private Transform _inverseSelf = new Transform();

  // Relative position of enemy to self in world coordinates
  private Vector3D _relativePositionWorld = new Vector3D(0,0,0);


  public void perform( long ms, long period )
  {
    //System.out.println( "Performing engagement action" );

    Craft self = getSelf();

    if( self.getDeceased() || _enemy.getDeceased() )
    {
      setCompleted();
      return;
    }

    self.getTransform().inverse( _inverseSelf );

    self.getRelativePosition( _enemy, _relativePositionWorld );

    _relativePositionWorld.rot( _inverseSelf, _relativePosition );

    double x = _relativePosition.get(0);
    double y = _relativePosition.get(1);
    double z = _relativePosition.get(2);

    double distance = Math.sqrt( (x*x) + (y*y) + (z*z) );
    double craftRadius = self.getRenderable().getRadius();
    double enemyRaduis = _enemy.getRenderable().getRadius();
    distance -= craftRadius + enemyRaduis;

    if( distance < 1500 )
    {
      if( !isPerforming( FaceAwayFromEnemyAction.class ) )
      {
        clearSubActions();

        add( new FaceAwayFromEnemyAction( ms, 8000, self, _enemy, 0.7, _relativePosition ) );
      }
    }

    if( distance > 100000 )
    {
      clearSubActions();
      setCompleted();
      return;
    }

    if( getSubActions().size() == 0 )
    {
      add( new FaceTowardsEnemyAction( ms, 8000, self, _enemy, 0.7, _relativePosition ) );
    }

    double modx = x >= 0 ? x : -x;
    double mody = y >= 0 ? y : -y;

    if( ( z > 0 ) &&
        ( ( modx ) < 200 ) &&
        ( ( mody ) < 200 ) &&
        ( ( ms - _lastWeaponDischage ) > 6000) )
    {
      self.fire( 0, 1 );
      _lastWeaponDischage = ms;
    }

    if( ((EliteCraft)self).getNumberOfMissiles() > 0 )
    {
      if( ( z > 0 ) &&
          ( ( modx ) < 900 ) &&
          ( ( mody ) < 900 ) &&
          ( ( ms - _lastWeaponDischage ) > 18000) )
      {
        _lastWeaponDischage = ms;
        ((EliteCraft)self).setLock( _enemy );
        ((EliteCraft)self).fireMissile();
      }
    }
  }
}
