package jelite.engine.galaxy.pilot.craft;

import jelite.engine.galaxy.*;
import jelite.engine.player.*;
import jelite.engine.galaxy.pilot.*;
import liaquay.math.vector.*;

public class TopLevelAction extends CraftAction
{
  private double _aggression = 0.4;

  /**
   * Create a top-level action supplying its birthdate.
   */
  public TopLevelAction( long birthdate, Craft self )
  {
    super( birthdate, 1000000000000L, self );
  }

  public void perform( long ms, long period )
  {
    //System.out.println( "Performing top-level action" );

    // Dont do this every cycle
    searchForNearest();

    if( ( _nearestItem != null ) && ( _distanceToNearestItem < 1000 ) )
    {
      if( !isPerforming( FaceAwayFromItemAction.class ) )
      {
        // Emergency evade
        clearSubActions();
        add( new FaceAwayFromItemAction( ms, 500, getSelf(), _nearestItem, 1.0 ) );
      }
    }
    else if( getSubActions().size() == 0 )
    {
      // There are no active sub-actions so we need to decide what to do!

      // See if the is anybody to attack
      Craft enemy = searchForEnemy();

      if( ( enemy != null ) && ( Math.random() < _aggression ) )
      {
        EngagementAction action = new EngagementAction( ms, getSelf(), enemy );

        add( action );
      }
      else
      {
        if( enemy == null )
        {
          getSelf().setDeceased();
        }

        CruiseAction action = new CruiseAction( ms, getSelf() );

        add( action );
      }
    }
  }

  private Craft searchForEnemy()
  {
    Sector center = getSelf().getSector();

    if( center == null )
    {
      return null;
    }

    Sector[][][] sectors = center.getNeighbours();
    sectors[ 1 ][ 1 ][ 1 ] = center;

    for( int i = 0; i < 3; ++ i )
    {
      for( int j = 0; j < 3; ++j )
      {
        for( int k = 0; k < 3; ++k )
        {
          Sector sector = sectors[ i ][ j ][ k ];

          if( sector != null )
          {
            Craft craft = searchSectorForEnemy( sector );

            if( craft != null )
            {
              return craft;
            }
          }
        }
      }
    }

    return null;
  }

  private Craft searchSectorForEnemy( Sector sector )
  {
    for( int i = 0; i < sector.size(); ++i )
    {
      Item item = sector.getItem( i );

      if( item instanceof Craft )
      {
        Craft craft = (Craft)item;

        Player player = craft.getPlayer();

        if( player instanceof HumanPlayer )
        {
          return craft;
        }
      }
    }

    return null;
  }



  //
  // Find the object nearest to self
  //

  private Item _nearestItem = null;
  private double _distanceToNearestItem = 0;

  private void searchForNearest()
  {
    _distanceToNearestItem = 0;
    _nearestItem = null;

    Sector center = getSelf().getSector();

    if( center == null )
    {
      return;
    }

    Sector[][][] sectors = center.getNeighbours();
    sectors[ 1 ][ 1 ][ 1 ] = center;

    for( int i = 0; i < 3; ++ i )
    {
      for( int j = 0; j < 3; ++j )
      {
        for( int k = 0; k < 3; ++k )
        {
          Sector sector = sectors[ i ][ j ][ k ];

          if( sector != null )
          {
            searchSectorForNearest( sector );
          }
        }
      }
    }
  }

  private void searchSectorForNearest( Sector sector )
  {
    Vector3D relativePositionWorld = new Vector3D(0,0,0);

    Item self = getSelf();

    for( int i = 0; i < sector.size(); ++i )
    {
      Item item = sector.getItem( i );

      if( item != self )
      {

        self.getRelativePosition( item, relativePositionWorld );

        double d = relativePositionWorld.length();
        double craftRadius = self.getRenderable().getRadius();
        double itemRaduis = item.getRenderable().getRadius();
        d -= craftRadius + itemRaduis;

        if( ( _nearestItem == null ) ||
            ( d < _distanceToNearestItem ) )
        {
          _nearestItem = item;
          _distanceToNearestItem = d;
        }
      }
    }
  }

}
