package jelite.engine.player;

import jelite.engine.galaxy.*;
import jelite.engine.galaxy.pilot.craft.*;
import jelite.hulls.*;
import liaquay.math.vector.*;

public class HumanPlayer extends Player
{
  private String _name;
  private String _password;
  private long _lastEngagement;

  public HumanPlayer( String name, String password, long ms  )
  {
    _name = name;
    _password = password;
    _lastEngagement = ms;
  }

  public final String getName()
  {
    return _name;
  }

  public final boolean comparePasswords( String password )
  {
    return _password.equals( password );
  }

  public String toString()
  {
    return "Player\n" +
           "name=" + _name + "\n" +
           "alive=" + isAlive() + "\n";
  }

  public void update( long ms, long period )
  {
    long sinceLastEngagement = ms - _lastEngagement;

    double t = ((double)sinceLastEngagement) / 600000.0;

    if( Math.random() < t )
    {
      createEnemy();
      _lastEngagement = ms;
    }
  }

  public void createEnemy()
  {
    Renderable[] ships = new Renderable[] {
    Renderables.COBRA,
    Renderables.ANACONDA,
    Renderables.ASP,
    Renderables.BOA,
    Renderables.BOULDER,
    Renderables.CAPSULE,
    Renderables.COBRAMK1,
    Renderables.CONSTRICTOR,
    Renderables.COUGAR,
    Renderables.FERDELANCE,
    Renderables.GECKO,
    Renderables.KRAIT,
    Renderables.MAMBA,
    Renderables.MORAY,
    Renderables.PYTHON,
    Renderables.SHUTTLE,
    Renderables.SIDEWINDER,
    Renderables.THARGLET,
    Renderables.THARGOID,
    Renderables.TRANSPORTER,
    Renderables.VIPER,
    Renderables.WORM };

    Sector sector = getCraft().getSector();

    if( sector.size() < 6 )
    {
      int shipIndex = (int)(Math.random() * (double)ships.length );
      if( shipIndex >= ships.length ) shipIndex = ships.length -1;

      Transform t = Transform.createRotateTranslate(
         Math.random(), Math.random(), Math.random(),
         getCraft().getTransform().getDx(),
         getCraft().getTransform().getDy() + 300,
         getCraft().getTransform().getDz() + 20000 );

      EliteCraft craft = new EliteCraft(
                               ships[ shipIndex ],
                               t );

      EnemyPilot player = new EnemyPilot( System.currentTimeMillis(), craft );
      craft.addCrewMember( player );
      sector.addNewItem( craft );
    }
  }
}
