package jelite.hulls;

import java.awt.*;
import java.io.*;
import java.net.*;
import liaquay.text.parser.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.graphics.parser.*;

public class LoadOnDemandHull implements Renderable
{
  private static URL _documentBase = null;

  private OpaqueSolid3D _solid = null;
  private String _solidName;

  public static void setDocumentBase( URL url )
  {
    _documentBase = url;
  }

  public LoadOnDemandHull( String solidName )
  {
    _solidName = solidName;
  }

  public OpaqueSolid3D getOpaqueSolid3D()
  {
    if( _solid == null )
    {
      loadSolid();
    }
    return _solid;
  }

  public double getRadius()
  {
    OpaqueSolid3D solid = getOpaqueSolid3D();
    return solid.getSolid3D().getRadius();
  }

  public void render( Graphics3D g, Transform transform, SelectionListener listener )
  {
    OpaqueSolid3D solid = getOpaqueSolid3D();
    g.add( solid, transform, listener );
  }

  private void loadSolid()
  {
    try
    {
      URL url = new URL( _documentBase, "ships/"+_solidName+".opaqueSolid3D" );

      InputStream inputStream = url.openStream();

      //InputStreamReader r = new InputStreamReader( inputStream, "ASCII" );
      // The encoding type seems to upset IE and Netscape so remove for now
      InputStreamReader r = new InputStreamReader( inputStream );

      CharReader reader = new CharReader( r );

      ParseOpaqueSolid3D parser = new ParseOpaqueSolid3D();

      parser.parseSequential( reader );

      if( parser.getMatched() )
      {
        _solid = parser.getOpaqueSolid3D();
      }

      inputStream.close();

    }
    catch( Exception e )
    {
      e.printStackTrace();
    }
  }
}
