package liaquay.collection;

public class Hashtable
{

  class HashtableEntry
  {
    private Object _key;
    private Object _value;

    public HashtableEntry( Object key, Object value )
    {
      _key = key;
      _value = value;
    }

    public Object getValue()
    {
      return _value;
    }

    public Object getKey()
    {
      return _key;
    }

    public void setValue( Object value )
    {
      _value = value;
    }
  }


  class HashtableIterator implements Iterator
  {
    private int _baseIndex = 0;
    private int _listIndex = 0;
    private Object _next;

    HashtableIterator()
    {
      findNext();
    }

    private void findNext()
    {
      _next = null;
      while( _baseIndex < _base.length )
      {
        ArrayList list = _base[ _baseIndex ];
        if( list ==  null )
        {
          _listIndex = 0;
        }
        else
        {
          while( _listIndex < list.size() )
          {
            _next = list.get( _listIndex );
            ++_listIndex;
            return;
          }
          _listIndex = 0;
        }
        ++_baseIndex;
      }
    }

    public Object next()
    {
      HashtableEntry e = (HashtableEntry)_next;
      findNext();
      return e;
    }

    public Object getKey()
    {
      HashtableEntry e = (HashtableEntry)_next;
      return e.getKey();
    }

    public boolean hasNext()
    {
      return _next != null;
    }
  }


  public class HashtableValueIterator extends HashtableIterator
  {


    public Object next()
    {
      HashtableEntry e = (HashtableEntry)super.next();
      return e.getValue();
    }

  }

  public class HashtableKeyIterator extends HashtableIterator
  {


    public Object next()
    {
      HashtableEntry e = (HashtableEntry)super.next();
      return e.getKey();
    }

  }



  // ******************************* Start of Hashtable code



  private ArrayList[] _base;
  private int _itemCount=0;
  private float _maxLoadRatio=0.75f;


  public Hashtable( int width )
  {
    _base = new ArrayList[ width ];
    for( int i = 0; i < _base.length; ++i )
    {
      _base[ i ] = new ArrayList( 1 );
    }
  }

  public Hashtable()
  {
    this( 32 );
  }

  private int getBaseIndex( Object key )
  {
    int i = key.hashCode();
    return getBaseIndex(i);
  }

  private int getBaseIndex( int hashCode )
  {
    int i = hashCode;
    if( i < 0 ) i = -i;
    return i % _base.length;
  }

  private float getLoad()
  {
    return (float)_itemCount/(float)_base.length;
  }

  private void rehash(int width)
  {
    // make a new hashtable with our spcified width
    Hashtable newOne = new Hashtable (width);
    HashtableIterator i = new HashtableIterator();
    // iterate through our current one and add all
    //  values to our new hashtable
    while (i.hasNext())
    {
      HashtableEntry e = (HashtableEntry)i.next();
      newOne.put(e.getKey(), e.getValue());
    }
    // make the attributes of the new hashtable our own
    _base = newOne._base;
  }

  public int getWidth()
  {
    return _base.length;
  }

  public boolean contains( Object key )
  {
    ArrayList list = _base[ getBaseIndex( key ) ];
    for( int j = 0; j < list.size(); ++j )
    {
      HashtableEntry e = (HashtableEntry)list.get( j );
      if( e.getKey().equals( key ) )
      {
        return true;
      }
    }
    return false;
  }

  public Object get( Object key )
  {
    ArrayList list = _base[ getBaseIndex( key ) ];
    for( int j = 0; j < list.size(); ++j )
    {
      HashtableEntry e = (HashtableEntry)list.get( j );
      if( e.getKey().equals( key ) )
      {
        return e.getValue();
      }
    }
    return null;
  }

  public Object get( int hashCode )
  {
    ArrayList list = _base[ getBaseIndex( hashCode ) ];
    for( int j = 0; j < list.size(); ++j )
    {
      HashtableEntry e = (HashtableEntry)list.get( j );
      if( e.getKey().hashCode() == hashCode )
      {
        return e.getValue();
      }
    }
    return null;
  }

  public Object remove( Object key )
  {
    ArrayList list = _base[ getBaseIndex( key ) ];
    for( int j = 0; j < list.size(); ++j )
    {
      HashtableEntry e = (HashtableEntry)list.get( j );
      if( e.getKey().equals( key ) )
      {
        list.remove( e );
        _itemCount--;
        return e.getValue();
      }
    }
    return null;
  }

  public void removeAll()
  {
    for( int i = 0; i < _base.length; ++i )
    {
      _base[ i ].removeAll();
    }
    _itemCount=0;
  }

  public Object put( Object key, Object value )
  {
    ArrayList list = _base[ getBaseIndex( key ) ];

    for( int j = 0; j < list.size(); ++j )
    {
      HashtableEntry e = (HashtableEntry)list.get( j );
      if( e.getKey().equals( key ) )
      {
        Object o = e.getValue();
        e.setValue( value );
        return o;
      }
    }

    HashtableEntry e = new HashtableEntry( key, value );

    list.add( e );

    _itemCount++;

    // rehashing checks
    if (getLoad()>_maxLoadRatio)
    {
      int width = _base.length;
      // need to think of how to calculate new width properly!!!!!
      width=width*2;
      rehash(width);
    }
    return null;
  }

  /**
  * Returns an iterator over values in the Hashtable
  */
  public Iterator iterator()
  {
    return new HashtableValueIterator();
  }

  /**
  * Returns an iterator over values in the Hashtable
  */
  public HashtableValueIterator hashtableValueIterator()
  {
    return new HashtableValueIterator();
  }

  /**
  * Returns an iterator over keys in the Hashtable
  */
  public Iterator keyIterator()
  {
    return new HashtableKeyIterator();
  }

  public int size()
  {
    return _itemCount;
  }
}