package liaquay.collection;

public class HuffIterator
{
  private ArrayList _nodes = new ArrayList();

  public HuffIterator( HuffTree tree )
  {
    HuffTreeNode node = tree.getRoot();

    while( node != null )
    {
      _nodes.add( node );

      if( node instanceof HuffTreeBinaryNode )
      {
        node = ((HuffTreeBinaryNode)node).getLeft();
      }
      else
      {
        node = null;
      }
    }
  }

  public boolean hasNext()
  {
    return _nodes.size() > 1;
  }

  public Object next()
  {
    HuffTreeLeafNode l = (HuffTreeLeafNode)_nodes.get( _nodes.size() - 1 );
    Object o = l.getValue();

    Object n1 = null;
    HuffTreeBinaryNode n2 = null;
    do
    {
      if( _nodes.size() == 1 )
      {
        return o;
      }
      n1 = _nodes.remove();
      n2 = (HuffTreeBinaryNode)_nodes.get( _nodes.size() - 1 );
    }
    while( n1 == n2.getRight() );

    HuffTreeNode n3 = n2.getRight();

    _nodes.add( n3 );

    while( n3 instanceof HuffTreeBinaryNode )
    {
      n3 = ((HuffTreeBinaryNode)n3).getLeft();
      _nodes.add( n3 );
    }
    return o;
  }

  public boolean[] getSymbol()
  {
    boolean[] bits = new boolean[ _nodes.size() - 1 ];

    HuffTreeBinaryNode n1 = (HuffTreeBinaryNode)_nodes.get( 0 );

    for( int i = 1; i < _nodes.size() - 1; ++i )
    {
      HuffTreeBinaryNode o = (HuffTreeBinaryNode)_nodes.get( i );

      if( o == n1.getLeft() )
      {
        bits[ i - 1 ] = false;
      }
      else
      {
        bits[ i - 1 ] = true;
      }
      n1 = o;
    }
    HuffTreeLeafNode l = (HuffTreeLeafNode)_nodes.get( _nodes.size() - 1 );

    if( l == n1.getLeft() )
    {
      bits[ bits.length - 1 ] = false;
    }
    else
    {
      bits[ bits.length - 1 ] = true;
    }

    return bits;
  }

  public Object getValue()
  {
    HuffTreeLeafNode l = (HuffTreeLeafNode)_nodes.get( _nodes.size() - 1 );
    return l.getValue();
  }

  public String getSymbolAsString()
  {
    StringBuffer sb = new StringBuffer();

    boolean[] bits = getSymbol();

    for( int i = 0; i < bits.length; ++i )
    {
      sb.append( bits[ i ] ? "1" : "0" );
    }

    return sb.toString();
  }

  public String toString()
  {
    HuffTreeLeafNode l = (HuffTreeLeafNode)_nodes.get( _nodes.size() - 1 );
    return getSymbolAsString() + " " + l.getValue();
  }
}
