package liaquay.collection;

import liaquay.stream.*;
import java.io.*;

public class HuffTree
{
  private HuffTreeBinaryNode _root;

  public HuffTree( HuffTreeBinaryNode root )
  {
    _root = root;
  }

  /**
   *
   */
  public HuffTree( Object[] values, double[] weights )
  {
    ArrayList nodes = new ArrayList(values.length);

    for( int i = 0; i < values.length; ++i )
    {
      nodes.add( new HuffWeight( new HuffTreeLeafNode( values[ i ] ), weights[ i ] ) );
    }

    while( nodes.size() > 1 )
    {
      int index1 = -1;
      int index2 = -1;
      double weight1 = 0;
      double weight2 = 0;

      for( int i = 0; i < nodes.size(); ++i )
      {
        HuffWeight node = (HuffWeight)nodes.get( i );

        if( index1 < 0 )
        {
          index1 = i;
          weight1 = node.getWeight();
        }
        else if( node.getWeight() < weight1 )
        {
          index1 = i;
          weight1 = node.getWeight();
        }
      }

      for( int i = 0; i < nodes.size(); ++i )
      {
        if( i != index1 )
        {
          HuffWeight node = (HuffWeight)nodes.get( i );

          if( index2 < 0 )
          {
            index2 = i;
            weight2 = node.getWeight();
          }
          else if( node.getWeight() < weight2 )
          {
            index2 = i;
            weight2 = node.getWeight();
          }
        }
      }

      HuffWeight huffWeight1 = (HuffWeight)nodes.get( index1 );
      HuffWeight huffWeight2 = (HuffWeight)nodes.get( index2 );

      HuffTreeNode binaryNode = null;

      if( index1 > index2 )
      {
        nodes.remove( huffWeight1 );
        nodes.remove( huffWeight2 );

        binaryNode = new HuffTreeBinaryNode(
          huffWeight2.getNode(),
          huffWeight1.getNode() );
      }
      else
      {
        nodes.remove( huffWeight2 );
        nodes.remove( huffWeight1 );

        binaryNode = new HuffTreeBinaryNode(
          huffWeight1.getNode(),
          huffWeight2.getNode() );
      }

      nodes.add( new HuffWeight( binaryNode, huffWeight1.getWeight() + huffWeight2.getWeight() ) );
    }

    _root = (HuffTreeBinaryNode)((HuffWeight)nodes.get( 0 )).getNode();
  }

  public HuffTreeBinaryNode getRoot()
  {
    return _root;
  }

  public Object readObject( BitInputStream in )
    throws IOException
  {
    HuffTreeBinaryNode n = _root;

    while( true )
    {
      if( in.eof() )
      {
        throw new UnexpectedEndOfStreamException();
      }
      Object o = null;
      if( in.read() )
      {
        o = n.getRight();
      }
      else
      {
        o = n.getLeft();
      }
      if( o instanceof HuffTreeLeafNode )
      {
        return ((HuffTreeLeafNode)o).getValue();
      }
      n = (HuffTreeBinaryNode)o;
    }
  }

  public Hashtable getSymbolMap()
  {
    Hashtable map = new Hashtable();

    for( HuffIterator i = new HuffIterator( this ); i.hasNext(); i.next() )
    {
      map.put( i.getValue(), i.getSymbol() );
    }

    return map;
  }

  private boolean compareNodes( HuffTreeNode n1, HuffTreeNode n2 )
  {
    if( ( n1 instanceof HuffTreeBinaryNode ) &&
        ( n1 instanceof HuffTreeBinaryNode ) )
    {
      return compareNodes( (HuffTreeBinaryNode)n1, (HuffTreeBinaryNode)n2 );
    }
    else if( ( n1 instanceof HuffTreeLeafNode ) &&
             ( n1 instanceof HuffTreeLeafNode ) )
    {
      HuffTreeLeafNode l1 = (HuffTreeLeafNode)n1;
      HuffTreeLeafNode l2 = (HuffTreeLeafNode)n2;

      return l1.getValue().equals( l2.getValue() );
    }
    else
    {
      return false;
    }
  }

  private boolean compareNodes( HuffTreeBinaryNode n1, HuffTreeBinaryNode n2 )
  {
    HuffTreeNode n1Left = n1.getLeft();
    HuffTreeNode n2Left = n2.getLeft();
    HuffTreeNode n1Right = n1.getRight();
    HuffTreeNode n2Right = n2.getRight();

    return compareNodes( n1Left, n2Left ) &&
           compareNodes( n1Right, n2Right );
  }

  public boolean equals( HuffTree tree )
  {
    HuffTreeBinaryNode n1 = tree.getRoot();
    HuffTreeBinaryNode n2 = getRoot();

    return compareNodes( n1, n2 );
  }

  public String toString()
  {
    StringBuffer sb = new StringBuffer();

    for( HuffIterator it = new HuffIterator( this ); it.hasNext(); it.next() )
    {
      sb.append( it.toString() );
      sb.append( "\n" );
    }

    return sb.toString();
  }
}
