package liaquay.collection;

import liaquay.math.set.*;

public class MappedSortable implements Sortable
{
  private Sortable _sortable;
  private IntegerBijection _map;

  public MappedSortable( Sortable sortable )
  {
    _sortable = sortable;
    _map = new IntegerBijection( sortable.size() );
  }

  /**
   * Compare elements at indexes i and j.
   * <p> If o1 < o2 returns positive result
   * <p> If o1 > o2 returns negative result
   * <p> If o1 = o2 returns zero
   */
  public int compare( int i, int j )
  {
    return _sortable.compare( _map.map( i ), _map.map( j ) );

  }

  /**
   * Swaps the elements at indexes i and j.
   */
  public void swap( int i, int j )
  {
    _map.swap( i, j );
  }

  /**
   * The size of the objectArray list
   */
  public int size()
  {
    return _sortable.size();
  }

  /**
   * Get the map that sorts the list
   */
  public IntegerBijection getMap()
  {
    return _map;
  }
}
