package liaquay.collection;

/**
 * An adaptor for sorting {@link ObjectArray}s.
 */
public class SortableObjectArray implements Sortable
{
  private ObjectArray _objectArray;
  private ObjectComparator _objectComparator;

  public SortableObjectArray( ObjectArray objectArray,
                              ObjectComparator objectComparator )
  {
    _objectArray = objectArray;
    _objectComparator = objectComparator;
  }

  /**
   * Set the comparator to be used with this sortable.
   */
  public void set( ObjectComparator objectComparator )
  {
    _objectComparator = objectComparator;
  }

  /**
   * Swaps the elements at indexes i and j.
   */
  public void swap( int i, int j )
  {
    Object t = _objectArray.get( i );
    Object u = _objectArray.get( j );
    _objectArray.set( i, u );
    _objectArray.set( j, t );
  }

  /**
   * The size of the sortable list
   */
  public int size()
  {
    return _objectArray.size();
  }

  /**
   * Compare elements at indexes i and j.
   * <p> If o1 < o2 returns negative result
   * <p> If o1 > o2 returns positive result
   * <p> If o1 = o2 returns zero
   */
  public int compare( int i, int j )
  {
    return _objectComparator.compare( _objectArray.get( i ), _objectArray.get( j ) );
  }
}
