package liaquay.collection.test;

import junit.framework.*;
import liaquay.collection.*;

/**
 * ArrayList tests.
 */
public class ArrayListTest extends TestCase
{
  public ArrayListTest(String name)
  {
    super(name);
  }

  public static Test suite()
  {
    /*
     * the dynamic way
     */
    return new TestSuite(ArrayListTest.class);
  }

  public void testInsert1()
  {
    ArrayList list = new ArrayList( 0 );

    list.insert( "a", 0 );

    assertEquals( 1, list.size() );
    assertEquals( "a", list.get( 0 ) );
  }

  public void testInsert2()
  {
    ArrayList list = new ArrayList( 1 );

    list.insert( "a", 0 );

    assertEquals( 1, list.size() );
    assertEquals( "a", list.get( 0 ) );
  }

  public void testInsert3()
  {
    ArrayList list = new ArrayList( 0 );

    list.insert( "a", 0 );
    list.insert( "b", 0 );

    assertEquals( 2, list.size() );
    assertEquals( "b", list.get( 0 ) );
    assertEquals( "a", list.get( 1 ) );
  }

  public void testInsert4()
  {
    ArrayList list = new ArrayList( 0 );

    list.insert( "a", 0 );
    list.insert( "b", 1 );

    assertEquals( 2, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "b", list.get( 1 ) );
  }

  public void testInsert5()
  {
    ArrayList list = new ArrayList( 0 );

    list.insert( "a", 0 );
    list.insert( "b", 1 );
    list.insert( "c", 1 );

    assertEquals( 3, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "c", list.get( 1 ) );
    assertEquals( "b", list.get( 2 ) );
  }

  public void testInsert6()
  {
    ArrayList list = new ArrayList( 3 );

    list.insert( "a", 0 );
    list.insert( "b", 1 );
    list.insert( "c", 1 );

    assertEquals( 3, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "c", list.get( 1 ) );
    assertEquals( "b", list.get( 2 ) );
  }

  static class TestObjectComparator implements ObjectComparator
  {
    public int compare( Object o1, Object o2 )
    {
      String s1 = (String)o1;
      String s2 = (String)o2;
      return s2.compareToIgnoreCase( s1 );
    }
  }

  private static ObjectComparator COMPARATOR = new TestObjectComparator();

  public void testSortedInsert1()
  {
    ArrayList list = new ArrayList( 0 );

    list.insert( "a", COMPARATOR );
  }

  public void testSortedInsert2()
  {
    ArrayList list = new ArrayList( 1 );

    list.insert( "a", COMPARATOR );
  }

  public void testSortedInsert3()
  {
    ArrayList list = new ArrayList( 1 );

    list.insert( "b", COMPARATOR );
    list.insert( "a", COMPARATOR );

    assertEquals( 2, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "b", list.get( 1 ) );
  }

  public void testSortedInsert4()
  {
    ArrayList list = new ArrayList( 2 );

    list.insert( "b", COMPARATOR );
    list.insert( "a", COMPARATOR );

    assertEquals( 2, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "b", list.get( 1 ) );
  }

  public void testSortedInsert5()
  {
    ArrayList list = new ArrayList( 1 );

    list.insert( "c", COMPARATOR );
    list.insert( "a", COMPARATOR );
    list.insert( "b", COMPARATOR );

    assertEquals( 3, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "b", list.get( 1 ) );
    assertEquals( "c", list.get( 2 ) );
  }

  public void testSortedInsert6()
  {
    ArrayList list = new ArrayList( 4 );

    list.insert( "c", COMPARATOR );
    list.insert( "a", COMPARATOR );
    list.insert( "b", COMPARATOR );
    list.insert( "d", COMPARATOR );

    assertEquals( 4, list.size() );
    assertEquals( "a", list.get( 0 ) );
    assertEquals( "b", list.get( 1 ) );
    assertEquals( "c", list.get( 2 ) );
    assertEquals( "d", list.get( 3 ) );
  }
}