package liaquay.collection.test;

import junit.framework.*;
import liaquay.collection.*;

public class BinarySearcherTest extends TestCase
{
  private ObjectLocator locator;
  private BinarySearcher searcher;

  public BinarySearcherTest(String name)
  {
    super(name);
  }

  public static Test suite()
  {
    /*
     * the dynamic way
     */
    return new TestSuite(BinarySearcherTest.class);
  }

  public static void main (String[] args)
  {
    junit.textui.TestRunner.run (suite());
  }

  protected void setUp()
  {
    // setup searcher

    locator = new ObjectLocator( new Integer(37),
                                               new ComparableComparator() );
    ArrayList average = makeSortedList(100);
    SearchableObjectArray soa= new SearchableObjectArray(average, locator);
    searcher = new BinarySearcher();
  }

  public void testSearch()
  {

    // test each of a list of twenty
    findAllInList( 20 );
    // and one more to test odd example
    findAllInList( 21 );
    // try some smaller numbers
    findAllInList( 0 );
    findAllInList( 1 );
    findAllInList( 2 );
    findAllInList( 3 );
    findAllInList( 4 );

    // now try to find some outside lists
    int result =find (new Integer(8), 1);
    assertEquals(-1, result);
    result =find (new Integer(8), 2);
    assertEquals(-1, result);

  }



  private ArrayList makeSortedList( int length )
  {
    ArrayList list = new ArrayList( length );

    for( int i = 0; i < length; ++i )
    {
      list.add( new Integer(i));
    }

    return list;
  }


  private int find ( Object toBeSearchedFor, int lengthOfList )
  {
    locator.setObject( toBeSearchedFor );
    ArrayList list = makeSortedList( lengthOfList );
    SearchableObjectArray soa= new SearchableObjectArray(list, locator);

    int result = searcher.search(soa);
    return result;
  }

  private void findAllInList( int length )
  {
    for (int i=0; i<length; i++)
    {
      int result = find( new Integer(i), length);
      assertEquals(i, result);
    }
  }
}
