package liaquay.collection.test;

import junit.framework.*;
import liaquay.collection.*;
import liaquay.stream.*;

/**
 * Hufftree tests.
 *
 */
public class HuffTest extends TestCase 
{
  public HuffTest(String name) 
  {
    super(name);
  }
  
  public static Test suite() 
  {
    /*
     * the dynamic way
     */
    return new TestSuite(HashtableTest.class);
  }
  
  public void testHuffTree()
  {
    Object[] objects = 
    {
      new Character( 'A' ), // 0
      new Character( 'B' ), // 1
      new Character( 'C' ), // 2
      new Character( 'D' ), // 3
      new Character( 'E' ), // 4
      new Character( 'F' ), // 5
      new Character( 'G' ), // 6
      new Character( 'H' )  // 7
    };

    double[] weights = 
    {
      9,
      2,
      3,
      4,
      5,
      6,
      7,
      8 
    };

    HuffTree tree = new HuffTree( objects, weights );

    HuffIterator i = new HuffIterator( tree );
    assertEquals( i.getSymbolAsString(), "00" );
    assertEquals( i.getValue(), objects[ 0 ] ); // A
    i.next();    
    assertEquals( i.getSymbolAsString(), "010" );
    assertEquals( i.getValue(), objects[ 3 ] ); // D
    i.next();    
    assertEquals( i.getSymbolAsString(), "011" );
    assertEquals( i.getValue(), objects[ 4 ] ); // E
    i.next();    
    assertEquals( i.getSymbolAsString(), "100" );
    assertEquals( i.getValue(), objects[ 5 ] ); // F
    i.next();    
    assertEquals( i.getSymbolAsString(), "1010" );
    assertEquals( i.getValue(), objects[ 1 ] ); // B
    i.next();    
    assertEquals( i.getSymbolAsString(), "1011" );
    assertEquals( i.getValue(), objects[ 2 ] ); // C
    i.next();    
    assertEquals( i.getSymbolAsString(), "110" );
    assertEquals( i.getValue(), objects[ 6 ] ); // G
    i.next();    
    assertEquals( i.getSymbolAsString(), "111" );
    assertEquals( i.getValue(), objects[ 7 ] ); // H

    try
    {
      BooleanArrayBitInputStream in = new BooleanArrayBitInputStream(
        new boolean[] { true, false, true, true } );    
       
      assertEquals( tree.readObject( in ), objects[ 2 ] );
    }
    catch( Exception e )
    {
      fail( "" + e );
    }
  }

  public static void main( String[] args ) throws Exception
  {
    Object[] objects = {
      new Character( 'A' ),
      new Character( 'B' ),
      new Character( 'C' ),
      new Character( 'D' ),
      new Character( 'E' ),
      new Character( 'F' ),
      new Character( 'G' ),
      new Character( 'H' ) };

    double[] weights = {
      9,
      2,
      3,
      4,
      5,
      6,
      7,
      8 };

    HuffTree tree = new HuffTree( objects, weights );

    for( HuffIterator i = new HuffIterator( tree ); i.hasNext(); )
    {
      System.out.println( i.getSymbolAsString() + " " + i.getValue() );
      i.next();
    }

    BooleanArrayBitInputStream in = new BooleanArrayBitInputStream(
      new boolean[] { true, false, true, true } );

    System.out.println( tree.readObject( in ) );
  }
}

