package liaquay.graphics;

import liaquay.collection.*;

import java.awt.*;
import java.awt.image.*;

public class ArrayImage
{
  private int _pixels[];
  private Image _image = null;
  private ArrayImageProducer _producer = null;
  private int _width;
  private int _height;

  public ArrayImage( Image image )
  {
    _width = image.getWidth(null);
    _height = image.getHeight(null);
    _pixels = new int[_width * _height];
    PixelGrabber pg = new PixelGrabber(image.getSource(), 0, 0, _width, _height, _pixels, 0, _width);
    pg.setColorModel( new DirectColorModel( 32, 0xFF0000, 0xFF00, 0xFF ) );

    try
    {
      pg.grabPixels();
    }
    catch (InterruptedException e)
    {
      throw new RuntimeException( "interrupted waiting for pixels!" );
    }
  }

  public ArrayImage( int width, int height, int[] pixels )
  {
    _width = width;
    _height = height;
    _pixels = pixels;
  }

  public ArrayImage( int width, int height )
  {
    _width = width;
    _height = height;
    _pixels = new int[ _width * _height ];
  }

  public final Image getImage()
  {
    if( _image == null )
    {
      ColorModel cm = new DirectColorModel( 32, 0xFF0000, 0xFF00, 0xFF );
      _producer = new ArrayImageProducer( _width, _height, cm, _pixels );
      _image = Toolkit.getDefaultToolkit().createImage( _producer );
    }
    _producer.update();
    return _image;
  }

  public void clear( int color )
  {
    Arrays.fill( _pixels, color );
  }

  public final void set( int x, int y, int color )
  {
    _pixels[ ( _width * y ) + x ] = color;
  }

  public final int get( int x, int y )
  {
    return _pixels[ ( _width * y ) + x ];
  }

  public final void set( int pos, int color )
  {
    _pixels[ pos ] = color;
  }

  public final int get( int pos )
  {
    return _pixels[ pos ];
  }

  public final int getWidth()
  {
    return _width;
  }

  public final int getHeight()
  {
    return _height;
  }

  public final int[] getPixels()
  {
    return _pixels;
  }

  public final ArrayImage getSubImage( int x, int y, int w, int h )
  {
    ArrayImage image = new ArrayImage( w, h );

    for( int i = 0; i < w; ++i )
    {
      int xx = i + x;

      if( ( xx >= 0 ) && ( xx < _width ) )
      {
        for( int j = 0; j < h; ++j )
        {
          int yy = j + y;

          if( ( yy >= 0 ) && ( yy < _height ) )
          {
            image.set( i, j, get( xx, yy ) );
          }
        }
      }
    }

    return image;

  }

  /**
   * Points numbered clockwise from top left.
   * Images up to a resolution of 65535 x 65535
   * <p>
   * All coordinates as fixed point integers ( i.e. x << 16 ).
   * This allows fractional start positions for the image patch.
   */
  public final void getSubImage( ArrayImage image,
                                 int fx1, int fy1,
                                 int fx2, int fy2,
                                 int fx3, int fy3,
                                 int fx4, int fy4 )
  {
    int width = image.getWidth();
    int height = image.getHeight();

    int dxl = ( fx4 - fx1 ) / height;
    int dyl = ( fy4 - fy1 ) / height;
    int dxr = ( fx3 - fx2 ) / height;
    int dyr = ( fy3 - fy2 ) / height;

    int fxl = fx1;
    int fyl = fy1;

    int fxr = fx2;
    int fyr = fy2;

    for( int r = 0; r < height; ++r )
    {
      int dx = ( fxr - fxl ) / width;
      int dy = ( fyr - fyl ) / width;

      int x = fxl;
      int y = fyl;

      for( int c = 0; c < width; ++c )
      {
        int rgb = get( x >> 16, y >> 16 );

        image.set( c, r, rgb );

        x += dx;
        y += dy;
      }

      fxl += dxl;
      fyl += dyl;
      fxr += dxr;
      fyr += dyr;
    }
  }

  /**
   * Points numbered clockwise from top left.
   * Images up to a resolution of 65535 x 65535
   * <p>
   * All coordinates as fixed point integers ( i.e. x << 16 ).
   * This allows fractional start positions for the image patch.
   */
  public final ArrayImage getSubImage( int width, int height,
                                       int x1, int y1,
                                       int x2, int y2,
                                       int x3, int y3,
                                       int x4, int y4 )
  {
    ArrayImage image = new ArrayImage( width, height );

    getSubImage( image,
                 x1, y1,
                 x2, y2,
                 x3, y3,
                 x4, y4 );

    return image;
  }
}
