package liaquay.graphics;

public class CheckedMaterial extends TiledMaterial
{
  private TexturedMaterial _material1;
  private TexturedMaterial _material2;

  public CheckedMaterial( TexturedMaterial material1,
                          TexturedMaterial material2,
                          double size )
  {
    super( size );
    _material1 = material1;
    _material2 = material2;
  }
  
  public Colour getTiledReflectiveColour( double x, double y )
  {
    boolean a = ( x > 0.5 );
    boolean b = ( y > 0.5 );
    
    double i = x * 2.0;
    double j = y * 2.0;
    
    if( a ) i -= 1.0;
    if( b ) j -= 1.0;
    
    if( ( a && b ) || ( !a && !b ) )
    {
      return _material2.getReflectiveColour( i, j );  
    }
    else
    {
      return _material1.getReflectiveColour( i, j );  
    }  
  }
  
  public Colour getTiledEmissiveColour( double x, double y )
  {
    boolean a = ( x > 0.5 );
    boolean b = ( y > 0.5 );
    
    double i = x * 2.0;
    double j = y * 2.0;
    
    if( a ) i -= 1.0;
    if( b ) j -= 1.0;
    
    if( ( a && b ) || ( !a && !b ) )
    {
      return _material2.getEmissiveColour( i, j );  
    }
    else
    {
      return _material1.getEmissiveColour( i, j );  
    }   
  }
  
  public float getTiledReflectivity( double x, double y )
  {
    boolean a = ( x > 0.5 );
    boolean b = ( y > 0.5 );
    
    double i = x * 2.0;
    double j = y * 2.0;
    
    if( a ) i -= 1.0;
    if( b ) j -= 1.0;
    
    if( ( a && b ) || ( !a && !b ) )
    {
      return _material2.getReflectivity( i, j );  
    }
    else
    {
      return _material1.getReflectivity( i, j );  
    }  
  }
}
