package liaquay.graphics;

import java.awt.*;
import java.awt.image.*;

public class ImageMaterial implements TexturedMaterial
{
  private BufferedImage _image;
  private double _size;
  private double _scale;
  private Colour _black = new Colour( 0f,0f,0f );
  private float _reflectivity;
  private boolean _emissive;
  
  public ImageMaterial( Image image, 
                        double size, 
                        float reflectivity, 
                        boolean emissive )
  {
    _size = size;
    _scale = 1 / size;
    _reflectivity = reflectivity;
    _emissive = emissive;
    int width = image.getWidth( null );
    int height = image.getHeight( null );    
    _image = new BufferedImage( width, height, BufferedImage.TYPE_3BYTE_BGR );
    _image.getGraphics().drawImage( image, 0, 0, null );
  }
  
  public final Colour getColour( double x, 
                                 double y )
  {
    x = x % _size;
    y = y % _size;
    
    if( x < 0 ) x = _size + x;
    if( y < 0 ) y = _size + y;    
 
 
    if( x < 0 ) x = -x;
    if( y < 0 ) y = -y;
    
    x *= _scale;
    y *= _scale;
    
    double width = (double)_image.getWidth(null);
    double height = (double)_image.getHeight(null);
    
    int xcoord = (int)( width * x );
    int ycoord = (int)( height * y );
    
    int rgb = _image.getRGB( xcoord, ycoord );
    
    int b = rgb & 0xff;
    int g = ( rgb >> 8 ) & 0xff;
    int r = ( rgb >> 16 ) & 0xff;
    
    return new Colour( (float)r / 255.0f, 
                       (float)g / 255.0f,
                       (float)b / 255.0f );
  }      

  public Colour getReflectiveColour( double x, double y )
  {
    if( _emissive )
    {
      return Colour.BLACK;
    }
    else
    {
      return getColour( x, y );
    }
  }
  
  public Colour getEmissiveColour( double x, double y )
  {
    if( _emissive )
    {
      return getColour( x, y );
    }
    else
    {
      return Colour.BLACK;
    }
  }
  
  public float getReflectivity( double x, double y )
  {
    return _reflectivity;
  }  
  
}
