
package liaquay.graphics;

import java.io.*;
import java.net.*;
import liaquay.math.vector.*;
import liaquay.collection.*;

// Imports a scene from a 3ds (3d Studio Max) Ressource

public class Importer3ds
{
  private int currentJunkId;
  private int nextJunkOffset;

  private String currentObjectName = null;
  private boolean endOfStream = false;
  private OpaqueSolid3DBuilder builder = null;

  private ArrayList _objects = new ArrayList();



  public OpaqueSolid3D importFromStream(InputStream inStream)
  {
    System.out.println(">> Importing scene from 3ds stream ...");

    BufferedInputStream in=new BufferedInputStream(inStream);
    try
    {
      readJunkHeader(in);
      if (currentJunkId!=0x4D4D) {System.out.println("Error: This is no valid 3ds file."); return null; }
      while (!endOfStream) readNextJunk(in);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    return builder.buildOpaqueSolid3D();
  }

  private String readString(InputStream in) throws IOException
  {
    String result=new String();
    byte inByte;
    while ((inByte=(byte)in.read())!=0) result+=(char)inByte;
    return result;
  }

  private int readInt(InputStream in) throws IOException
  {
    return in.read()|(in.read()<<8)|(in.read()<<16)|(in.read()<<24);
  }

  private int readShort(InputStream in) throws IOException
  {
    return (in.read()|(in.read()<<8));
  }

  private float readFloat(InputStream in) throws IOException
  {
    return Float.intBitsToFloat(readInt(in));
  }


  private void readJunkHeader(InputStream in) throws IOException
  {
    currentJunkId=readShort(in);
    nextJunkOffset=readInt(in);
    endOfStream=currentJunkId<0;
  }

  private void readNextJunk(InputStream in) throws IOException
  {
    readJunkHeader(in);

    if (currentJunkId==0x3D3D) return; // Mesh block
    if (currentJunkId==0x4000) // Object block
    {
      currentObjectName=readString(in);
      System.out.println(">> Importing object: "+currentObjectName);
      return;
    }
    if (currentJunkId==0x4100)  // Triangular polygon object
    {
      System.out.println( "New Object!!!!" );

      builder = new OpaqueSolid3DBuilder();
      //currentObject=new idx3d_Object();
      //scene.addObject(currentObjectName,currentObject);
      return;
    }
    if (currentJunkId==0x4110) // Vertex list
    {
      readVertexList(in);
      return;
    }
    if (currentJunkId==0x4120) // Point list
    {
      readPointList(in);
      return;
    }
    if (currentJunkId==0x4140) // Mapping coordinates
    {
      readMappingCoordinates(in);
      return;
    }

    skipJunk(in);
  }

  private void skipJunk(InputStream in) throws IOException, OutOfMemoryError
  {
    for (int i=0; (i<nextJunkOffset-6)&&(!endOfStream);i++)
      endOfStream=in.read()<0;
  }

  private void readVertexList(InputStream in) throws IOException
  {
    float x,y,z;
    int vertices=readShort(in);
    for (int i=0; i<vertices; i++)
    {
      x=readFloat(in);
      y=readFloat(in);
      z=readFloat(in);

      builder.addVertex( new Vector3D( x,y,z ) );
      //System.out.println( "New Vertex " + x + " " + y + " " + z );

      //currentObject.addVertex(x,-y,z);
    }
  }

  private void readPointList(InputStream in) throws IOException
  {
    int v1,v2,v3;
    int triangles=readShort(in);
    for (int i=0; i<triangles; i++)
    {
      Solid3DFaceBuilder faceBuilder = new Solid3DFaceBuilder();

      v1=readShort(in);
      v2=readShort(in);
      v3=readShort(in);
      readShort(in);

      faceBuilder.addVertex( v1 );
      faceBuilder.addVertex( v3 );
      faceBuilder.addVertex( v2 );

      builder.addFace( faceBuilder.buildSolid3DFace() );
      //System.out.println( "New triangle " + v1 + " " + v2 + " " + v3 );
      //currentObject.addTriangle(
      //  currentObject.vertex(v1),
      //  currentObject.vertex(v2),
      //  currentObject.vertex(v3));
    }
  }

  private void readMappingCoordinates(InputStream in) throws IOException
  {
    int vertices=readShort(in);
    for (int i=0; i<vertices; i++)
    {
      float u = readFloat(in);
      float v = readFloat(in);

      System.out.println( "Mapping " + u + " " + v );
      //currentObject.vertex(i).u=readFloat(in);
      //currentObject.vertex(i).v=readFloat(in);
    }
  }

  public static void main( String[] args ) throws IOException
  {
    InputStream in = new FileInputStream( args[ 0 ] );


    Importer3ds importer = new  Importer3ds();
    System.out.println( importer.importFromStream( in ).getSolid3D() );

  }
}