package liaquay.graphics;

import java.awt.*;

public class PlainMaterial implements TexturedMaterial
{
  /**
   * The colour this surface reflects.
   */
  private Colour _reflective;

  /**
   * The colour this surface emits.
   */
  private Colour _emissive;

  /**
   * How reflective the surface is 0.0 - 1.0
   */
  private float _reflectivity;

  public PlainMaterial( Colour reflective,
                        Colour emissive,
                        float reflectivity )
  {
    _reflective = reflective;
    _emissive = emissive;
    _reflectivity = reflectivity;
  }

  public Colour getReflectiveColour()
  {
    return _reflective;
  }

  public Colour getEmissiveColour()
  {
    return _emissive;
  }

  public float getReflectivity()
  {
    return _reflectivity;
  }

  public void setReflectiveColour( Colour colour )
  {
    _reflective = colour;
  }

  public void setEmissiveColour( Colour colour )
  {
    _emissive = colour;
  }

  public String toString()
  {
    return "(" +
           _reflective +
           " " +
           _emissive +
           " " +
           _reflectivity +
           ")";
  }

  public Color getColor( double cosAngleToNormal )
  {
    float f = 0.5f + ( (float)cosAngleToNormal * 0.5f );

    float red = (_reflective.getRed() * f) + _emissive.getRed();
    float green = (_reflective.getGreen() * f) + _emissive.getGreen() ;
    float blue = (_reflective.getBlue() * f) + _emissive.getBlue();

    // Prevent burn out...
    float max = red;
    if( green > max ) max = green;
    if( blue > max ) max = blue;
    if( max > 1.0 )
    {
      red /= max;
      blue /= max;
      green /= max;
    }

    return new Color( red, green, blue );
  }

  public Colour getReflectiveColour( double x, double y )
  {
    return _reflective;
  }

  public Colour getEmissiveColour( double x, double y )
  {
    return _emissive;
  }

  public float getReflectivity( double x, double y )
  {
    return _reflectivity;
  }
}
