package liaquay.graphics;

public abstract class TiledMaterial implements TexturedMaterial
{
  private double _size;

  public TiledMaterial( double size )
  {
    _size = size;
  }
  
  private final double tile( double s )
  {
    s = s % _size;
    
    if( s < 0 ) s = _size + s;
  
    return s;
  }

  public Colour getReflectiveColour( double x, double y )
  {
    x = tile( x );
    y = tile( y );
    return getTiledReflectiveColour( x, y );
  }

  public Colour getEmissiveColour( double x, double y )
  {
    x = tile( x );
    y = tile( y );
    return getTiledEmissiveColour( x, y );
  }
  
  public float getReflectivity( double x, double y )
  {
    x = tile( x );
    y = tile( y );
    return getTiledReflectivity( x, y );
  }
  
  public abstract Colour getTiledReflectiveColour( double x, double y );
  public abstract Colour getTiledEmissiveColour( double x, double y );
  public abstract float getTiledReflectivity( double x, double y );
}
