package liaquay.graphics.concave;

import java.awt.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.collection.*;

/**
 * This implementation is not thread safe.
 */
public class ConcaveGraphics3D implements Graphics3D
{
  //
  // Renderes for the different types of object.
  //
  private OpaqueSolid3DRenderer _opaqueSolid3DRenderer = new OpaqueSolid3DRenderer();
  private OpaqueSphere3DRenderer _opaqueSphere3DRenderer = new OpaqueSphere3DRenderer(_opaqueSolid3DRenderer);

  //
  // private variables
  //
  private Image _image;
  private Transform _inverseCamera = new Transform();
  private Selection _selection;
  private ZBufferArrayImage _screen;
  private Camera3D _camera = null;
  private double _doubleWidth;

  // Stuff to do with lighting
  private int[] _diffuse = null;
  private int[] _specular = null;

  /**
   *
   */
  public ConcaveGraphics3D( ZBufferArrayImage screen, Selection selection )
  {
    Light light = new Light( new Vector3D( 0,0,1 ),            // Direction
                             RGBColor.getColor( 128,128,128 ), // Diffuse
                             RGBColor.getColor( 128,128,128 ), // Specular
                             255,                              // Hightlight Sheen
                             4096 );                           // Hightlight Spread

    LightMap lightMap = new LightMap( new Light[] { light }, RGBColor.getColor( 128,128,128) );

    _diffuse = lightMap.diffuse;
    _specular = lightMap.specular;

    _screen = screen;
    _image = _screen.getImage();
    _selection = selection;

    _doubleWidth = (double)screen.getWidth();
  }

  /**
   * Prepares the renderer for new a new scene.
   */
  public void beginScene( Camera3D camera )
  {
    _camera = camera;

    camera.getTransform().inverse( _inverseCamera );
    _screen.clear(0xff000000);
  }

  /**
   * Adds an opaque solid to the scene.
   */
  public void add( OpaqueSolid3D solid,
                   Transform transform,
                   SelectionListener selectionListener )
  {
    render( _inverseCamera,
            transform,
            solid.getSolid3D().getCenter(),
            solid.getSolid3D().getRadius(),
            solid,
            _opaqueSolid3DRenderer,
            selectionListener );
  }

  public void add( TranslucentSolid3D solid,
                   Transform transform,
                   SelectionListener selectionListener )
  {
  }

  /**
   * Adds an opaque shpere to the scene.
   */
  public void add( OpaqueSphere3D sphere,
                   Transform transform,
                   SelectionListener selectionListener )
  {
    render( _inverseCamera,
            transform,
            sphere.getSphere3D().getCenter(),
            sphere.getSphere3D().getRadius(),
            sphere,
            _opaqueSphere3DRenderer,
            selectionListener );
  }

  public void add( TranslucentSphere3D sphere,
                   Transform transform,
                   SelectionListener selectionListener )
  {
  }

  public void add( OpaquePlane3D plane,
                   Transform transform,
                   SelectionListener selectionListener )
  {
  }

  /**
   * Compeletes the scene.
   */
  public void endScene()
  {
    _image = _screen.getImage();
  }

  /**
   * Get an image representing the rendered scene.
   */
  public Image getImage()
  {
    return _image;
  }

  public Selection getSelection()
  {
    return _selection;
  }

  /**
   * Deliver a pick event to the graphics engine.
   * This will result in the appropriate selection listener being activated.
   */
  public void pick( int x, int y )
  {

  }

  // Local variables I dont want to new
  private Vector3D _center = new Vector3D(0,0,0);
  private Matrix4x3 _combinedTransform = Matrix4x3.createIdentity();
  private Transform _t1 = new Transform();

  private final void render( Transform inverseCamera,
                             Transform transform,
                             Vector3D center,
                             double radius,
                             Object object,
                             Renderer renderer,
                             SelectionListener selectionListener )
  {
    int width = _screen.getWidth();
    double distanceToFocalPoint = _camera.getDistanceToFocalPoint();

    int height = _screen.getHeight();
    int heightDiv2 = height >> 1;
    int widthDiv2 = width >> 1;

    transform.mul( inverseCamera, _t1 );
    _combinedTransform.set( _t1 );
    center.mul( _combinedTransform, _center );
    double distance = _center.get( 2 );

    int screenX =  (int)( _center.perx( distanceToFocalPoint ) * _doubleWidth ) + widthDiv2;
    int screenY = -(int)( _center.pery( distanceToFocalPoint ) * _doubleWidth ) + heightDiv2;
    int screenR =  (int)( ( radius * distanceToFocalPoint * _doubleWidth ) / _center.get( 2 ) );

    if( distance > 0 )
    {
      if( ( ( screenX + screenR >= 0 ) && ( screenX - screenR < width ) ) &&
          ( ( screenY + screenR > 0 ) && ( screenY - screenR < height ) ) )
      {
        renderer.render( object,
                         _screen,
                         _combinedTransform,
                         _center,
                         distanceToFocalPoint,
                         screenX,
                         screenY,
                         screenR,
                         _diffuse,
                         _specular );
      }
    }
  }
}