package liaquay.graphics.concave;

import liaquay.math.vector.Vector3D;

public final class LightMap
{
  int[] diffuse=new int[65536];
  int[] specular=new int[65536];
  private float[] sphere=new float[65536];
  private Light[] light;
  private int ambient;

  public LightMap(Light[] light, int ambient)
  {
    this.light = light;
    this.ambient = ambient;
    buildSphereMap();
    rebuildLightMap();
  }

  private void buildSphereMap()
  {
    float fnx,fny,fnz;
    int pos;
    for (int ny=-128;ny<128;ny++)
    {
      fny=(float)ny/128;
      for (int nx=-128;nx<128;nx++)
      {
        pos=nx+128+((ny+128)<<8);
        fnx=(float)nx/128;
        fnz=(float)(1-Math.sqrt(fnx*fnx+fny*fny));
        sphere[pos]=(fnz>0)?fnz:0;
      }
    }
  }

  public void rebuildLightMap()
  {
    Vector3D l;
    float fnx,fny,angle,phongfact,sheen, spread;
    int diffuse,specular,cos,dr,dg,db,sr,sg,sb;
    for (int ny=-128;ny<128;ny++)
    {
      fny=(float)ny/128;
      for (int nx=-128;nx<128;nx++)
      {
        int pos=nx+128+((ny+128)<<8);
        fnx=(float)nx/128;
        sr=sg=sb=0;
        dr=RGBColor.getRed(ambient);
        dg=RGBColor.getGreen(ambient);
        db=RGBColor.getBlue(ambient);
        for (int i=0;i<light.length;i++)
        {
          l=light[i].v;
          diffuse=light[i].diffuse;
          specular=light[i].specular;
          sheen=(float)light[i].highlightSheen/255f;
          spread=(float)light[i].highlightSpread/4096;
          spread=(spread<0.01f)?0.01f:spread;
          cos=(int)(255*(light[i].v.dot( new Vector3D(fnx,fny,sphere[pos]))));
          cos=(cos>0)?cos:0;
          dr+=(RGBColor.getRed(diffuse)*cos)>>8;
          dg+=(RGBColor.getGreen(diffuse)*cos)>>8;
          db+=(RGBColor.getBlue(diffuse)*cos)>>8;
          phongfact=sheen*(float)Math.pow((float)cos/255f,1/spread);
          sr+=(int)((float)RGBColor.getRed(specular)*phongfact);
          sg+=(int)((float)RGBColor.getGreen(specular)*phongfact);
          sb+=(int)((float)RGBColor.getBlue(specular)*phongfact);
        }
        this.diffuse[pos]=RGBColor.getCropColor(dr,dg,db);
        this.specular[pos]=RGBColor.getCropColor(sr,sg,sb);
      }
    }
  }
}
