package liaquay.graphics.concave;

import java.awt.*;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class OpaqueSolid3DRenderer implements Renderer
{
  private Vector3D[] _vertices = new Vector3D[ 5000 ];
  private boolean[] _flags = new boolean[ 5000 ];
  private Vector3D _normal = new Vector3D(0,0,0);
  private ScreenTriangle _triangle;

//private ArrayImage texture = TextureFactory.WOOD( 128, 128, 0.1f );
//private ArrayImage texture = TextureFactory.SKY( 128, 128, 2f );
private ArrayImage texture = TextureFactory.MARBLE( 128, 128, 1f );

  public OpaqueSolid3DRenderer()
  {
    for( int i = 0; i < _vertices.length; ++i )
    {
      _vertices[ i ] = new Vector3D(0,0,0);
    }

    _triangle = new ScreenTriangle();
    _triangle.p1 = new ScreenVertex();
    _triangle.p2 = new ScreenVertex();
    _triangle.p3 = new ScreenVertex();
  }

  public static final int getScreenX( Vector3D q, int width, int height, double ppd )
  {
    return (int)( q.perx( ppd ) * (double)width ) + (width/2);
  }

  public static final int getScreenY( Vector3D q, int width, int height, double ppd )
  {
    return -(int)( q.pery( ppd ) * (double)width ) + (height/2);
  }

  public static final int getScreenZ( Vector3D q, int width, int height, double ppd )
  {
    return (int)( (1 / q.get( 2 )) * 6000000000d );
  }

  public final void render( Object o,
                            ZBufferArrayImage screen,
                            Matrix4x3 combinedTransform,
                            Vector3D center,
                            double ppd,
                            int screenX,
                            int screenY,
                            int screenR,
                            int[] diffuse,
                            int[] specular )
  {
    OpaqueSolid3D opaqueSolid = (OpaqueSolid3D)o;
    Solid3D solid = opaqueSolid.getSolid3D();
    Solid3DFace[] faces = solid.getFaces();
    Vector3D[] vertices = solid.getVertices();
    int width = screen.getWidth();
    int height = screen.getHeight();

    for( int i = 0; i < vertices.length; ++i )
    {
      _flags[ i ] = false;
    }

    for( int i = 0; i < faces.length; ++i )
    {
      Solid3DFace face = faces[ i ];

      solid.getFaceNormal( i ).rot( combinedTransform, _normal );

      int v[] = face.getVertices();

      vertices[ v[ 0 ] ].mul( combinedTransform, _vertices[ v[ 0 ] ] );
      _flags[ v[ 0 ] ] = true;

      double d = _normal.dot( _vertices[ v[ 0 ] ] );

      if( d < 0 )
      {
        for( int j = 0; j < v.length; ++j )
        {
          int vertexIndex = v [ j ];

          if( !_flags[ vertexIndex ] )
          {
            vertices[ vertexIndex ].mul( combinedTransform, _vertices[ vertexIndex ] );
            _flags[ vertexIndex ] = true;
          }
        }

        _triangle.material = opaqueSolid.getOuterMaterials( i ).getPlainMaterial();
        _triangle.n = _normal;

        Vector3D q = _vertices[ v [ 0 ] ];

        _triangle.p1.x = getScreenX( q, width, height, ppd );
        _triangle.p1.y = getScreenY( q, width, height, ppd );
        _triangle.p1.z = getScreenZ( q, width, height, ppd );
        _triangle.p1.tx = 0;
        _triangle.p1.ty = 0;
_triangle.p1.n = _normal;
_triangle.p2.n = _normal;
_triangle.p3.n = _normal;

        q = _vertices[ v [ 1 ] ];

        _triangle.p3.x = getScreenX( q, width, height, ppd );
        _triangle.p3.y = getScreenY( q, width, height, ppd );
        _triangle.p3.z = getScreenZ( q, width, height, ppd );
        _triangle.p3.tx = 127;
        _triangle.p3.ty = 0;

        _triangle.p2.tx = 127;
        _triangle.p2.ty = 127;


        for( int t = 2; t < v.length; ++t )
        {
          q = _vertices[ v [ t ] ];

          ScreenVertex temp = _triangle.p2;
          _triangle.p2 = _triangle.p3;
          _triangle.p3 = temp;

          _triangle.p3.x = getScreenX( q, width, height, ppd );
          _triangle.p3.y = getScreenY( q, width, height, ppd );
          _triangle.p3.z = getScreenZ( q, width, height, ppd );

          //renderTriangleFlat( screen, _triangle, diffuse, specular );
          //renderTriangleFlatTextured( screen, _triangle, diffuse, specular, texture );
          renderTrianglePhongTextured( screen, _triangle, diffuse, specular, texture );
        }
      }
    }
  }

  public final void renderTrianglePhongTextured( ZBufferArrayImage screen,
                                                 ScreenTriangle tri,
                                                 int[] diffuse,
                                                 int[] specular,
                                                 ArrayImage texture )
  {
    int tw = texture.getWidth();
    int th = texture.getHeight();

    int width = screen.getWidth();
    int height = screen.getHeight();
    int[] zBuffer = screen.getZBuffer();
    int[] pixels = screen.getPixels();

    ScreenVertex p1 = tri.p1;
    ScreenVertex p2 = tri.p2;
    ScreenVertex p3 = tri.p3;


    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }
    if (p2.y>p3.y) { ScreenVertex tempVertex=p2; p2=p3; p3=tempVertex; }
    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }

    if (p1.y>=height)
    {
      // Off the bottom of the display
      return;
    }

    if (p3.y<0)
    {
      // Off the top of the display
      return;
    }

    if (p1.y==p3.y)
    {
      // Zero height triangle
      return;
    }

    int reflectivity = (int)(tri.material.getReflectivity() * 255);

    int x1=p1.x<<8;
    int x2=p2.x<<8;
    int x3=p3.x<<8;
    int y1=p1.y;
    int y2=p2.y;
    int y3=p3.y;

    int x4=x1+(x3-x1)*(y2-y1)/(y3-y1);

    x1<<=8;
    x2<<=8;
    x3<<=8;
    x4<<=8;

    int z1=p1.z;
    int z2=p2.z;
    int z3=p3.z;
    int tx1=p1.tx<<16;
    int tx2=p2.tx<<16;
    int tx3=p3.tx<<16;
    int ty1=p1.ty<<16;
    int ty2=p2.ty<<16;
    int ty3=p3.ty<<16;
    int nx1=(int)(p1.n.get( 0 )*65536*127);
    int nx2=(int)(p2.n.get( 0 )*65536*127);
    int nx3=(int)(p3.n.get( 0 )*65536*127);
    int ny1=(int)(p1.n.get( 1 )*65536*127);
    int ny2=(int)(p2.n.get( 1 )*65536*127);
    int ny3=(int)(p3.n.get( 1 )*65536*127);

    int dx=(x4-x2)>>16;
    if (dx==0) return;

//System.out.println( "dx " + dx );
//System.out.println( "dx " + ((x4-x2+(1<<15))>>16));
    if (dx<0)
    {
      dx -=1;
    }
    else
    {
      dx +=1;
    }



    int temp=((y2-y1)<<8)/(y3-y1);

    int z4=z1+((z3-z1)>>8)*temp;
    int nx4=nx1+((nx3-nx1)>>8)*temp;
    int ny4=ny1+((ny3-ny1)>>8)*temp;
    int tx4=tx1+((tx3-tx1)>>8)*temp;
    int ty4=ty1+((ty3-ty1)>>8)*temp;

    int dz=(z4-z2)/dx;
    int dnx=(nx4-nx2)/dx;
    int dny=(ny4-ny2)/dx;

    int dtx=(tx4-tx2)/dx;
    int dty=(ty4-ty2)/dx;

    if (dx<0)
    {
      temp=x2; x2=x4; x4=temp;
      z2=z4;
      tx2=tx4;
      ty2=ty4;
      nx2=nx4;
      ny2=ny4;
    }

    if (y2>=0)
    {
      int dy=y2-y1;

      if (dy!=0)
      {
        int dxL=(x2-x1)/dy;
        int dxR=(x4-x1)/dy;
        int dzBase=(z2-z1)/dy;
        int dnxBase=(nx2-nx1)/dy;
        int dnyBase=(ny2-ny1)/dy;
        int dtxBase=(tx2-tx1)/dy;
        int dtyBase=(ty2-ty1)/dy;
        int xBase=x1;
        int xMax=x1;
        int zBase=z1;
        int txBase=tx1;
        int tyBase=ty1;
        int nxBase=nx1;
        int nyBase=ny1;

        if (y1<0)
        {
          xBase-=y1*dxL;
          xMax-=y1*dxR;
          zBase-=y1*dzBase;
          nxBase-=y1*dnxBase;
          nyBase-=y1*dnyBase;
          txBase-=y1*dtxBase;
          tyBase-=y1*dtyBase;
          y1=0;
        }


        y2=(y2<height)?y2:height;
        int offset=y1*width;

        for (int y=y1;y<y2;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;
          int tx=txBase;
          int ty=tyBase;
          int nx=nxBase;
          int ny=nyBase;

          if (xL<0)
          {
            z-=xL*dz;
            tx-=xL*dtx;
            ty-=xL*dty;
            nx-=xL*dnx;
            ny-=xL*dny;
            xL=0;
          }
          xR=(xR<width)?xR:width-1;
int dxp = dx >0?dx:-dx;
//if( xR-xL > dxp ) System.out.println( " xr-xl " + (xR-xL) + " dx " + dxp );

          if( xR == xL ) xR++;
          for (int x=xL;x<xR;x++)
          {
            int pos=x+offset;

            if (z >= zBuffer[pos])
            {
              zBuffer[ pos ]=z;

              // int bkgrd = screen.p[pos];
              int lutID=(127-(nx>>16))+((127-(ny>>16))<<8);

              int txp = tx>>16;
              int typ = ty>>16;
//              if( txp < 0 ) System.out.println( "txp " + txp + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( txp >= tw ) System.out.println( "txp " + txp + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( typ < 0 ) System.out.println( "typ " + typ + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( typ >= th ) System.out.println( "typ " + typ + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
              if( txp < 0 ) txp = 0;
              if( txp >= tw ) txp = tw-1;
              if( typ < 0 ) typ = 0;
              if( typ >= th ) typ = th-1;


              int reflectiveColor = texture.get( txp, typ );
              //c=idx3d_Color.transparency(bkgrd,reflectiveColor,transparency);

              int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
              int s = RGBColor.scale( specular[lutID], reflectivity);
              int currentColor = RGBColor.add( c, s );

              pixels[ pos ] = currentColor;

            }
            z+=dz;
            tx+=dtx;
            ty+=dty;
            nx+=dnx;
            ny+=dny;


          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
          txBase+=dtxBase;
          tyBase+=dtyBase;
          nxBase+=dnxBase;
          nyBase+=dnyBase;
        }
      }
    }

    if (y2<height)
    {
      int dy=y3-y2;
      if (dy!=0)
      {
        int dxL=(x3-x2)/dy;
        int dxR=(x3-x4)/dy;
        int dzBase=(z3-z2)/dy;
        int dnxBase=(nx3-nx2)/dy;
        int dnyBase=(ny3-ny2)/dy;

        int xBase=x2;
        int xMax=x4;
        int zBase=z2;
        int dtxBase=(tx3-tx2)/dy;
        int dtyBase=(ty3-ty2)/dy;
        int txBase=tx2;
        int tyBase=ty2;
        int nxBase=nx2;
        int nyBase=ny2;

        if (y2<0)
        {
          xBase-=y2*dxL;
          xMax-=y2*dxR;
          zBase-=y2*dzBase;
          txBase-=y2*dtxBase;
          tyBase-=y2*dtyBase;
          nxBase-=y2*dnxBase;
          nyBase-=y2*dnyBase;
          y2=0;
        }

        y3=(y3<height)?y3:height;
        int offset=y2*width;

        for (int y=y2;y<y3;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;
          int tx=txBase;
          int ty=tyBase;
          int nx=nxBase;
          int ny=nyBase;

          if (xL<0)
          {
            z-=xL*dz;
            tx-=xL*dtx;
            ty-=xL*dty;
            nx-=xL*dnx;
            ny-=xL*dny;
            xL=0;
          }
          xR=(xR<width)?xR:width-1;

int dxp = dx >0?dx:-dx;
//if( xR-xL > dxp ) System.out.println( " xr-xl " + (xR-xL) + " dx " + dxp );

          if( xR == xL ) xR++;
          for (int x=xL;x<xR;x++)
          {
            int pos=x+offset;

            if (z >= zBuffer[pos])
            {
              zBuffer[ pos ]=z;

              // int bkgrd = screen.p[pos];
//              int lutID=((nx>>16)&255)+(((ny>>16)&255)<<8);
              int lutID=(127-(nx>>16))+((127-(ny>>16))<<8);

              int txp = tx>>16;
              int typ = ty>>16;
//              if( txp < 0 ) System.out.println( "txp " + txp + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( txp >= tw ) System.out.println( "txp " + txp + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( typ < 0 ) System.out.println( "typ " + typ + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
//              if( typ >= th ) System.out.println( "typ " + typ + " dtx " + (dtx>>16) + " dty " + (dty>>16) );
              if( txp < 0 ) txp = 0;
              if( txp >= tw ) txp = tw-1;
              if( typ < 0 ) typ = 0;
              if( typ >= th ) typ = th-1;


              int reflectiveColor = texture.get( txp, typ );
               //c=idx3d_Color.transparency(bkgrd,reflectiveColor,transparency);

              int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
              int s = RGBColor.scale( specular[lutID], reflectivity);
              int currentColor = RGBColor.add( c, s );

              pixels[ pos ] = currentColor;

            }
            z+=dz;
            tx+=dtx;
            ty+=dty;
            nx+=dnx;
            ny+=dny;
          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
          txBase+=dtxBase;
          tyBase+=dtyBase;
          nxBase+=dnxBase;
          nyBase+=dnyBase;
        }
      }
    }
  }

  public final void renderTriangleFlatTextured( ZBufferArrayImage screen,
                                                ScreenTriangle tri,
                                                int[] diffuse,
                                                int[] specular,
                                                ArrayImage texture )
  {
    int tw = texture.getWidth() - 1;
    int th = texture.getHeight() - 1;

    int width = screen.getWidth();
    int height = screen.getHeight();
    int[] zBuffer = screen.getZBuffer();
    int[] pixels = screen.getPixels();

    ScreenVertex p1 = tri.p1;
    ScreenVertex p2 = tri.p2;
    ScreenVertex p3 = tri.p3;


    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }
    if (p2.y>p3.y) { ScreenVertex tempVertex=p2; p2=p3; p3=tempVertex; }
    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }

    if (p1.y>=height)
    {
      // Off the bottom of the display
      return;
    }

    if (p3.y<0)
    {
      // Off the top of the display
      return;
    }

    if (p1.y==p3.y)
    {
      // Zero height triangle
      return;
    }

    int lutID=(int)(tri.n.get(0)*127+127)+((int)(tri.n.get(1)*127+127)<<8);
    int reflectivity = (int)(tri.material.getReflectivity() * 255);

    int x1=p1.x<<8;
    int x2=p2.x<<8;
    int x3=p3.x<<8;
    int y1=p1.y;
    int y2=p2.y;
    int y3=p3.y;

    int x4=x1+(x3-x1)*(y2-y1)/(y3-y1);
    x1<<=8;
    x2<<=8;
    x3<<=8;
    x4<<=8;

    int z1=p1.z;
    int z2=p2.z;
    int z3=p3.z;
    int tx1=p1.tx<<16;
    int tx2=p2.tx<<16;
    int tx3=p3.tx<<16;
    int ty1=p1.ty<<16;
    int ty2=p2.ty<<16;
    int ty3=p3.ty<<16;

    int dx=(x4-x2)>>16;
    if (dx==0) return;

    int temp=256*(y2-y1)/(y3-y1);

    int z4=z1+((z3-z1)>>8)*temp;
    int tx4=tx1+((tx3-tx1)>>8)*temp;
    int ty4=ty1+((ty3-ty1)>>8)*temp;

    int dz=(z4-z2)/dx;
    int dtx=(tx4-tx2)/dx;
    int dty=(ty4-ty2)/dx;

    if (dx<0)
    {
      temp=x2; x2=x4; x4=temp;
      z2=z4;
      tx2=tx4;
      ty2=ty4;
    }

    if (y2>=0)
    {
      int dy=y2-y1;

      if (dy!=0)
      {
        int dxL=(x2-x1)/dy;
        int dxR=(x4-x1)/dy;
        int dzBase=(z2-z1)/dy;
        int dtxBase=(tx2-tx1)/dy;
        int dtyBase=(ty2-ty1)/dy;
        int xBase=x1;
        int xMax=x1;
        int zBase=z1;
        int txBase=tx1;
        int tyBase=ty1;

        if (y1<0)
        {
          xBase-=y1*dxL;
          xMax-=y1*dxR;
          zBase-=y1*dzBase;
          txBase-=y1*dtxBase;
          tyBase-=y1*dtyBase;
          y1=0;
        }

        y2=(y2<height)?y2:height;
        int offset=y1*width;

        for (int y=y1;y<y2;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;
          int tx=txBase;
          int ty=tyBase;

          if (xL<0)
          {
            z-=xL*dz;
            tx-=xL*dtx;
            ty-=xL*dty;
            xL=0;
          }
          xR=(xR<width)?xR:width;

          for (int x=xL;x<xR;x++)
          {
            int pos=x+offset;

            if (z >= zBuffer[pos])
            {
              zBuffer[ pos ]=z;

              // int bkgrd = screen.p[pos];

              int txp = tx>>16;
              if( txp < 0 ) txp = 0;
              if( txp >= tw ) txp = tw;
              int typ = ty>>16;
              if( typ < 0 ) typ = 0;
              if( typ >= th ) typ = th;


              int reflectiveColor = texture.get( txp, typ );
              //c=idx3d_Color.transparency(bkgrd,reflectiveColor,transparency);

              int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
              int s = RGBColor.scale( specular[lutID], reflectivity);
              int currentColor = RGBColor.add( c, s );

              pixels[ pos ] = currentColor;

            }
            z+=dz;
            tx+=dtx;
            ty+=dty;
          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
          txBase+=dtxBase;
          tyBase+=dtyBase;
        }
      }
    }

    if (y2<height)
    {
      int dy=y3-y2;
      if (dy!=0)
      {
        int dxL=(x3-x2)/dy;
        int dxR=(x3-x4)/dy;
        int dzBase=(z3-z2)/dy;

        int xBase=x2;
        int xMax=x4;
        int zBase=z2;
        int dtxBase=(tx3-tx2)/dy;
        int dtyBase=(ty3-ty2)/dy;
        int txBase=tx2;
        int tyBase=ty2;

        if (y2<0)
        {
          xBase-=y2*dxL;
          xMax-=y2*dxR;
          zBase-=y2*dzBase;
          txBase-=y2*dtxBase;
          tyBase-=y2*dtyBase;
          y2=0;
        }

        y3=(y3<height)?y3:height;
        int offset=y2*width;

        for (int y=y2;y<y3;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;
          int tx=txBase;
          int ty=tyBase;

          if (xL<0)
          {
            z-=xL*dz;
            tx-=xL*dtx;
            ty-=xL*dty;
            xL=0;
          }
          xR=(xR<width)?xR:width;

          for (int x=xL;x<xR;x++)
          {
            int pos=x+offset;

            if (z >= zBuffer[pos])
            {
              zBuffer[ pos ]=z;

              // int bkgrd = screen.p[pos];

              int txp = tx>>16;
              if( txp < 0 ) txp = 0;
              if( txp >= tw ) txp = tw;
              int typ = ty>>16;
              if( typ < 0 ) typ = 0;
              if( typ >= th ) typ = th;


              int reflectiveColor = texture.get( txp, typ );
              //c=idx3d_Color.transparency(bkgrd,reflectiveColor,transparency);

              int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
              int s = RGBColor.scale( specular[lutID], reflectivity);
              int currentColor = RGBColor.add( c, s );

              pixels[ pos ] = currentColor;

            }
            z+=dz;
            tx+=dtx;
            ty+=dty;
          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
          txBase+=dtxBase;
          tyBase+=dtyBase;
        }
      }
    }
  }

  public final void renderTriangleFlat( ZBufferArrayImage screen,
                                        ScreenTriangle tri,
                                        int[] diffuse,
                                        int[] specular )
  {
    int width = screen.getWidth();
    int height = screen.getHeight();
    int[] zBuffer = screen.getZBuffer();
    int[] pixels = screen.getPixels();

    ScreenVertex p1 = tri.p1;
    ScreenVertex p2 = tri.p2;
    ScreenVertex p3 = tri.p3;


    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }
    if (p2.y>p3.y) { ScreenVertex tempVertex=p2; p2=p3; p3=tempVertex; }
    if (p1.y>p2.y) { ScreenVertex tempVertex=p1; p1=p2; p2=tempVertex; }

    if (p1.y>=height)
    {
      // Off the bottom of the display
      return;
    }

    if (p3.y<0)
    {
      // Off the top of the display
      return;
    }

    if (p1.y==p3.y)
    {
      // Zero height triangle
      return;
    }

    int reflectiveColor = tri.material.getReflectiveColour().getRGB();
    int emmisiveColor = tri.material.getEmissiveColour().getRGB();
    int reflectivity = (int)(tri.material.getReflectivity() * 255);

    int lutID=(int)(tri.n.get(0)*127+127)+((int)(tri.n.get(1)*127+127)<<8);
    int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
    int s = RGBColor.scale( specular[lutID], reflectivity);
    int currentColor = RGBColor.add( c, s );
    currentColor = RGBColor.add( emmisiveColor, currentColor );

    int x1=p1.x<<8;
    int x2=p2.x<<8;
    int x3=p3.x<<8;
    int y1=p1.y;
    int y2=p2.y;
    int y3=p3.y;

    int x4=x1+(x3-x1)*(y2-y1)/(y3-y1);
    x1<<=8;
    x2<<=8;
    x3<<=8;
    x4<<=8;

    int z1=p1.z;
    int z2=p2.z;
    int z3=p3.z;

    int dx=(x4-x2)>>16;

//System.out.println( "dx " + dx );
//System.out.println( "dx " + ((x4-x2+(1<<15))>>16));
if( x4 > x2 )
{
//dx = ((x4-x2+(1<<16))>>16);

}
else
{
//dx = ((x4-x2-(1<<16))>>16);

}


    if (dx==0) return;

    int temp=256*(y2-y1)/(y3-y1);

    int z4=z1+((z3-z1)>>8)*temp;

    int dz=(z4-z2)/dx;

    if (dx<0)
    {
      temp=x2; x2=x4; x4=temp;
      z2=z4;
    }

    if (y2>=0)
    {
      int dy=y2-y1;


      if (dy!=0)
      {
        int dxL=(x2-x1)/dy;
        int dxR=(x4-x1)/dy;

        int dzBase=(z2-z1)/dy;

        int xBase=x1;
        int xMax=x1;
        int zBase=z1;

        if (y1<0)
        {
          xBase-=y1*dxL;
          xMax-=y1*dxR;
          zBase-=y1*dzBase;
          y1=0;
        }

        y2=(y2<height)?y2:height;
        int offset=y1*width;

        for (int y=y1;y<y2;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;

          if (xL<0)
          {
            z-=xL*dz;
            xL=0;
          }
          xR=(xR<width)?xR:width;

          for (int x=xL;x<=xR;x++)
          {
            int pos=x+offset;

            if (z >= zBuffer[pos])
            {
              zBuffer[ pos ]=z;
              // int bkgrd=screen.p[pos];
              // c=idx3d_Color.transparency(bkgrd,currentColor,transparency);
              int col = currentColor;

              pixels[pos]=col;
            }
            z+=dz;
          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
        }
      }
    }

    if (y2<height)
    {
      int dy=y3-y2;
      if (dy!=0)
      {
        int dxL=(x3-x2)/dy;
        int dxR=(x3-x4)/dy;


        int dzBase=(z3-z2)/dy;

        int xBase=x2;
        int xMax=x4;
        int zBase=z2;

        if (y2<0)
        {
          xBase-=y2*dxL;
          xMax-=y2*dxR;
          zBase-=y2*dzBase;
          y2=0;
        }

        y3=(y3<height)?y3:height;
        int offset=y2*width;

        for (int y=y2;y<y3;y++)
        {
          int xL=xBase>>16;
          int xR=xMax>>16;
          int z=zBase;

          if (xL<0)
          {
            z-=xL*dz;
            xL=0;
          }
          xR=(xR<width)?xR:width;

          for (int x=xL;x<=xR;x++)
          {
            int pos=x+offset;

            if (z>=zBuffer[pos])
            {
              zBuffer[pos]=z;

              // int bkgrd=screen.p[pos];
              // c=idx3d_Color.transparency(bkgrd,currentColor,transparency);
              int col = currentColor;

              pixels[pos]=col;
            }
            z+=dz;
          }

          offset+=width;
          xBase+=dxL;
          xMax+=dxR;
          zBase+=dzBase;
        }
      }
    }
  }


}
