package liaquay.graphics.concave;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class OpaqueSphere3DRenderer implements Renderer
{
  Sphere _sphere1;
  Sphere _sphere2;
  Sphere _sphere3;

  public OpaqueSphere3DRenderer( OpaqueSolid3DRenderer solidRenderer )
  {
    _sphere1 = new Sphere( solidRenderer, 4 );
    _sphere2 = new Sphere( solidRenderer, 8 );
    _sphere3 = new Sphere( solidRenderer, 20 );
  }


  public final void render( Object o,
                            ZBufferArrayImage screen,
                            Matrix4x3 combinedTransform,
                            Vector3D center,
                            double ppd,
                            int screenX,
                            int screenY,
                            int screenR,
                            int[] diffuse,
                            int[] specular )
  {
    OpaqueSphere3D opaqueSphere = (OpaqueSphere3D)o;
    Materials materials = opaqueSphere.getOuterMaterials();
    TexturedMaterial texture = materials.getTexturedMaterial();

    if( (texture instanceof ArrayImageMaterial) && ( screenR > 2 ) )
    {
      Sphere sphere = _sphere1;


      if( screenR > 70 )
      {
        sphere = _sphere3;
      }
      else if( screenR > 20 )
      {
        sphere = _sphere2;
      }

      sphere.render( o,
                     screen,
                     combinedTransform,
                     center,
                     ppd,
                     screenX,
                     screenY,
                     screenR,
                     diffuse,
                     specular );
    }
    else
    {
      OpaqueSphere3D sphere = (OpaqueSphere3D)o;

      int r = screenR;

      if( r < 2 ) r = 2;

      renderDisc( screen, screenX, screenY, r, diffuse, specular, sphere.getOuterMaterials().getPlainMaterial() );
    }
  }


  public final void renderDisc( ArrayImage screen,
                                int centerX,
                                int centerY,
                                int radius,
                                int[] diffuse,
                                int[] specular,
                                PlainMaterial material )
  {
    int color = material.getReflectiveColour().getRGB();

    int reflectiveColor = material.getReflectiveColour().getRGB();
    int emmisiveColor = material.getEmissiveColour().getRGB();
    int reflectivity = (int)(material.getReflectivity() * 255);
    int scale = ( 127 << 16 ) / radius;


    int width = screen.getWidth();
    int height = screen.getHeight();

    int radiusSquared = radius * radius;

    if( radius > 0 )
    {
      if( ( centerY >= 0 ) && ( centerY < height ) )
      {
        if( ( centerX >= 0 ) && ( centerX < width ) )
        {
          int lutID=127+(127<<8);
          int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
          int s = RGBColor.scale( specular[lutID], reflectivity);
          color = RGBColor.add( c, s );
          color = RGBColor.add( emmisiveColor, color );
          screen.set( centerX, centerY, color );
        }

        if( ( centerX < width ) && ( centerX + radius >= 0 ) )
        {
          int start = 1;
          int xl = 0;

          if( centerX < 0 )
          {
            start = -centerX;
            xl = start * scale;
          }

          for( int x = start; ( ( centerX+x ) < width ) && ( ( x*x ) < radiusSquared ); ++x )
          {
            int lutID=((xl>>16)+127)+(127<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX+x, centerY, color );

            xl += scale;
          }
        }

        if( ( centerX > 0 ) && ( centerX - radius < width ) )
        {
          int start = 1;
          int xl = 0;

          if( centerX >= width )
          {
            start = centerX - width +1;
            xl = start * scale;
          }

          for( int x = start; ( ( centerX-x ) >= 0 ) && ( ( x*x ) < radiusSquared ); ++x )
          {
            int lutID=(xl>>16)+127+(127<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX-x, centerY, color );

            xl += scale;
          }
        }
      }
    }

    if( ( centerY >= 0 ) && ( centerY - radius < height ) )
    {
      int start = 1;
      int yl = 0;

      if( centerY >= height )
      {
        start = centerY - height +1;
        yl = scale * start;
      }


      for( int y = start; ( centerY - y >= 0 ) && ( y < radius ); ++ y )
      {
        if( ( centerX >= 0 ) && ( centerX < width ) )
        {
          int lutID=127+(((yl>>16)+127)<<8);
          int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
          int s = RGBColor.scale( specular[lutID], reflectivity);
          color = RGBColor.add( c, s );
          color = RGBColor.add( emmisiveColor, color );

          screen.set( centerX, centerY-y, color );
        }

        int ySquared = y*y;

        if( ( centerX < width ) && ( centerX + radius >= 0 ) )
        {
          int xstart = 1;
          int xl = 0;

          if( centerX < 0 )
          {
            xstart = -centerX;
            xl = xstart * scale;
          }

          for( int x = xstart; ( ( centerX+x ) < width ) && ( ( x*x ) + ySquared < radiusSquared ); ++x )
          {
            int lutID=(xl>>16)+127+(((yl>>16)+127)<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX+x, centerY-y, color );

            xl += scale;
          }
        }

        if( ( centerX > 0 ) && ( centerX - radius < width ) )
        {
          int xstart = 1;
          int xl = 0;

          if( centerX >= width )
          {
            xstart = centerX - width +1;
            xl = xstart * scale;
          }

          for( int x = xstart; ( ( centerX-x ) >= 0 ) && ( ( x*x )  + ySquared < radiusSquared ); ++x )
          {
            int lutID=(xl>>16)+127+(((yl>>16)+127)<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX-x, centerY-y, color );

            xl += scale;
          }
        }
        yl += scale;
      }
    }

    if( ( centerY < height ) && ( centerY + radius >= 0 ) )
    {
      int start = 1;
      int yl = 0;

      if( centerY < 0 )
      {
        start = -centerY;
        yl = scale * start;
      }


      for( int y = start; ( centerY + y < height ) && ( y < radius ); ++ y )
      {
        if( ( centerX >= 0 ) && ( centerX < width ) )
        {
          int lutID=127+(((yl>>16)+127)<<8);
          int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
          int s = RGBColor.scale( specular[lutID], reflectivity);
          color = RGBColor.add( c, s );
          color = RGBColor.add( emmisiveColor, color );

          screen.set( centerX, centerY+y, color );
        }

        int ySquared = y*y;

        if( ( centerX < width ) && ( centerX + radius >= 0 ) )
        {
          int xstart = 1;
          int xl = 0;

          if( centerX < 0 )
          {
            xstart = -centerX;
            xl = xstart * scale;
          }

          for( int x = xstart; ( ( centerX+x ) < width ) && ( ( x*x ) + ySquared < radiusSquared ); ++x )
          {
            int lutID=(xl>>16)+127+(((yl>>16)+127)<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX+x, centerY+y, color );

            xl += scale;
          }
        }

        if( ( centerX > 0 ) && ( centerX - radius < width ) )
        {
          int xstart = 1;
          int xl = 0;

          if( centerX >= width )
          {
            xstart = centerX - width +1;
            xl = xstart * scale;
          }

          for( int x = xstart; ( ( centerX-x ) >= 0 ) && ( ( x*x )  + ySquared < radiusSquared ); ++x )
          {
            int lutID=(xl>>16)+127+(((yl>>16)+127)<<8);
            int c = RGBColor.multiply(reflectiveColor, diffuse[lutID] );
            int s = RGBColor.scale( specular[lutID], reflectivity);
            color = RGBColor.add( c, s );
            color = RGBColor.add( emmisiveColor, color );

            screen.set( centerX-x, centerY+y, color );

            xl += scale;
          }
        }
        yl += scale;
      }
    }
  }

}



