package liaquay.graphics.concave.test;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.graphics.material.*;
import liaquay.graphics.convex.*;
import liaquay.graphics.raytrace.*;
import liaquay.graphics.concave.*;

public class ConcaveGraphicsApplet extends Applet
{
  private Image rayImage = null;
  private Graphics3D graphics3D = null;
  private OpaqueSphere3D sphere1 = null;
  private Camera3D camera = null;
  private boolean _running = true;

  public void init()
  {
    setLayout( new BorderLayout() );

    Dimension size = getSize();

    int width = size.width;
    int height = size.height;

    Selection selection = new Selection();
    SelectionListener selectionListener1 = new SelectionAdaptor();

    graphics3D = new ConcaveGraphics3D( new ZBufferArrayImage( width, height ), selection );

    rayImage = graphics3D.getImage();

    final ImageCanvas canvas = new ImageCanvas(rayImage);

    add( canvas, BorderLayout.CENTER );


   PlainMaterial material1 = new PlainMaterial( new Colour( 1f,1f,1f ),
                                               new Colour( 0f,0f,0f ),
                                               1f );

   PlainMaterial material2 = new PlainMaterial( new Colour( 1f,0f,0f ),
                                               new Colour( 0f,0f,0f ),
                                               1f );

   TexturedMaterial texture1 = new ArrayImageMaterial( TextureFactory.WOOD( 512, 512, 1f ), 1.0d, 1.0f, false );

   TexturedMaterial texture2 = new ArrayImageMaterial( loadImage( "earth.jpg" ), 1.0d, 1.0f, false );


   Materials materialsStar = new Materials( material1 );

   Materials materialsPlasma = new Materials( material2 );

   Materials materialsPlanet1 = new Materials( material1, texture2 );


    sphere1 = new OpaqueSphere3D( new Sphere3D( 100, new Vector3D( 0,0,0 ) ), materialsPlanet1 );

    camera = new Camera3D( Transform.createIdentity(), 1.0 );

    _running = true;


    new Thread()
    {
      public void run()
      {
        Transform t = new Transform();

        double angle = 0;

        while( _running )
        {
          angle += 0.03;
          //t.rotateTranslate( angle, angle, angle, 0, 0,280 );
          t.rotateTranslate( 0, angle, 0, 0, 0,280 );
          graphics3D.beginScene( camera );
          graphics3D.add( sphere1, t, null );
          graphics3D.endScene();
          canvas.paintNow();

          try
          {
            Thread.sleep( 10 );
          }
          catch( InterruptedException e )
          {
          }
        }
      }
    }.start();
  }

  public void stop()
  {
    _running = false;
  }

  public void start()
  {
    _running = true;
  }

  private Image loadImage( String imageName )
  {
    System.out.println( imageName );
    System.out.println(  ""+getDocumentBase() );

    Image image = getImage( getDocumentBase(), imageName );
    try
    {
      MediaTracker mt = new MediaTracker(this);
      mt.addImage( image, 0 );
      mt.waitForID( 0 );
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    System.out.println( "Width " + image.getWidth(null));
    System.out.println( "Height " + image.getHeight(null));

    return image;
  }
}