package liaquay.graphics.concave.test;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.graphics.material.*;
import liaquay.graphics.convex.*;
import liaquay.graphics.raytrace.*;
import liaquay.graphics.concave.*;

public class ConcaveGraphicsFrame extends Frame
{
  private Image rayImage = null;
  private Graphics3D graphics3D = null;
  private OpaqueSphere3D sphere1 = null;
  private Camera3D camera = null;
  private boolean _running = true;

  public void init( OpaqueSolid3D s )
  {
    setLayout( new BorderLayout() );

    Dimension size = getSize();

    int width = size.width;
    int height = size.height;

    Selection selection = new Selection();
    SelectionListener selectionListener1 = new SelectionAdaptor();

    graphics3D = new ConcaveGraphics3D( new ZBufferArrayImage( width, height ), selection );

    rayImage = graphics3D.getImage();

    final ImageCanvas canvas = new ImageCanvas(rayImage);
    final OpaqueSolid3D solid = s;

    add( canvas, BorderLayout.CENTER );

    camera = new Camera3D( Transform.createIdentity(), 1.0 );

    _running = true;

    new Thread()
    {
      public void run()
      {
        Transform t = new Transform();

        double angle = 0;

        while( _running )
        {
          angle += 0.02;
          t.rotateTranslate( -Math.PI/2, angle, 0, 0,-100,580 );
          graphics3D.beginScene( camera );
          graphics3D.add( solid,  t, null );
          graphics3D.endScene();
          canvas.paintNow();

          try
          {
            Thread.sleep( 1 );
          }
          catch( Exception e )
          {
          }
        }
      }
    }.start();


  }

  public void stop()
  {
    _running = false;
  }

  public void start()
  {
    _running = true;
  }

  public static void main( String[] args ) throws IOException
  {
    InputStream in = new FileInputStream( args[ 0 ] );

    Importer3ds importer = new  Importer3ds();

    ConcaveGraphicsFrame frame = new ConcaveGraphicsFrame();
    frame.setSize( 450,450 );
    frame.init( importer.importFromStream( in ) );
    frame.setVisible( true );
  }
}