package liaquay.graphics.convex;

import java.awt.*;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class OpaqueSolid3DRenderer implements Renderer
{
  private int _polyX[] = new int[ 40 ];
  private int _polyY[] = new int[ 40 ];
  private Vector3D[] _vertices = new Vector3D[ 50 ];
  private boolean[] _flags = new boolean[ 50 ];
  private Vector3D _normal = new Vector3D(0,0,0);
  private Vector3D _lightSource = new Vector3D( 0, -1, -1 ).normalize();

  public OpaqueSolid3DRenderer()
  {
    for( int i = 0; i < _vertices.length; ++i )
    {
      _vertices[ i ] = new Vector3D(0,0,0);
    }
  }

  public void render( Object o,
                      Graphics g,
                      Matrix4x3 combinedTransform,
                      double ppd,
                      int width,
                      int height,
                      int screenX,
                      int screenY,
                      int screenR )
  {
    OpaqueSolid3D opaqueSolid = (OpaqueSolid3D)o;
    Solid3D solid = opaqueSolid.getSolid3D();
    Solid3DFace[] faces = solid.getFaces();
    Vector3D[] vertices = solid.getVertices();

    for( int i = 0; i < vertices.length; ++i )
    {
      _flags[ i ] = false;
    }

    for( int i = 0; i < faces.length; ++i )
    {
      Solid3DFace face = faces[ i ];

      solid.getFaceNormal( i ).rot( combinedTransform, _normal );

      int v[] = face.getVertices();

      vertices[ v[ 0 ] ].mul( combinedTransform, _vertices[ v[ 0 ] ] );
      _flags[ v[ 0 ] ] = true;

      double d = _normal.dot( _vertices[ v[ 0 ] ] );

      if( d < 0 )
      {
        for( int j = 0; j < v.length; ++j )
        {
          int vertexIndex = v [ j ];

          if( !_flags[ vertexIndex ] )
          {
            vertices[ vertexIndex ].mul( combinedTransform, _vertices[ vertexIndex ] );
            _flags[ vertexIndex ] = true;
          }

          Vector3D q = _vertices[ vertexIndex ];
          _polyX[ j ] =  (int)( q.perx( ppd ) * (double)width );
          _polyY[ j ] = -(int)( q.pery( ppd ) * (double)width );
        }

        double cosAngleToLightSource = _lightSource.dot( _normal );

        g.setColor( opaqueSolid.getOuterMaterials( i ).getPlainMaterial().getColor( cosAngleToLightSource ) );


        g.fillPolygon( _polyX, _polyY, v.length );
      }
    }
  }
}