package liaquay.graphics.material;

import liaquay.graphics.*;

public class MandlebrotMaterial implements TexturedMaterial
{
  private double _size;
  private double _scale;
  private Colour _black = new Colour( 0f,0f,0f );
  private float _reflectivity;
  private boolean _emissive;
  
  public MandlebrotMaterial( double size, 
                             float reflectivity, 
                             boolean emissive )
  {
    _size = size;
    _scale = 1 / size;
    _reflectivity = reflectivity;
    _emissive = emissive;
  }
  
  private final Colour getColour( double x, double y )
  {
    x += _size /2;
    y += _size /10;
    
    x = x % _size;
    y = y % _size;
    
    if( x < 0 ) x = _size + x;
    if( y < 0 ) y = _size + y;
 
    if( x < 0 ) x = -x;
    if( y < 0 ) y = -y;
    
    x *= _scale;
    y *= _scale;
    
    double i = (x-0.5) * 3;
    double j = (y-0.5) * 3;
    
    int n = 64 - iterate( i, j );
    
    int r = (n * 15) & 0xff;
    int g = (n * 45) & 0xff;
    int b = (n * 8) & 0xff;
    
    return new Colour( (float)r / 255.0f, 
                       (float)g / 255.0f,
                       (float)b / 255.0f );
  }
  
  public final Colour getEmissiveColour( double x, 
                                         double y )
  {
    if( !_emissive )
    {
      return Colour.BLACK;
    }
    else
    {
      return getColour( x, y );
    }
  }  
  
  public final Colour getReflectiveColour( double x, 
                                           double y )
  {
    if( _emissive )
    {
      return Colour.BLACK;
    }
    else
    {
      return getColour( x, y );
    }
  }
  
  public float getReflectivity( double x, double y )
  {
    return _reflectivity;
  }
  
  public int iterate( double e, double f )
  {
    double a = 0;
    double b = 0;
    
    for( int n = 0; n < 64; ++n )
    {
      double aa = a*a;
      double bb = b*b;
      
      if( aa + bb > 2 )
      {
        return n;
      }
      
      double a2 = aa - bb + e;
      double b2 = (2*a*b) + f;
      
      a = a2;
      b = b2;
    }
    
    return 64;
  }
}
