package liaquay.graphics.parser.test;

import java.io.*;
import junit.framework.*;
import liaquay.text.parser.*;
import liaquay.graphics.*;
import liaquay.graphics.parser.*;

/**
 * ParseColour tests.
 */
public class ParsePlainMaterialTest extends TestCase 
{
  public ParsePlainMaterialTest(String name) 
  {
    super(name);
  }
  
  protected void setUp() 
  {
  }
  
  public static Test suite() 
  {
    /*
     * the dynamic way
     */
    return new TestSuite(ParsePlainMaterialTest.class);
  }
  
  
  public void test1()
  {
    ParsePlainMaterial parser = new ParsePlainMaterial();

    try
    {
      parser.parseSequential( "( (0.1 0.2 0.3) (0.4 0.5 0.6) 0.7 )" );

      if( parser.getMatched() )
      {
        PlainMaterial plainMaterial = parser.getPlainMaterial();
        
      
        Colour c1 = plainMaterial.getReflectiveColour(0,0);
        
        if( c1.getRed() != 0.1f ) fail( "Red component incorrect " + c1.getRed() );
        if( c1.getGreen() != 0.2f ) fail( "Green component incorrect " + c1.getGreen() );
        if( c1.getBlue() != 0.3f ) fail( "Blue component incorrect " + c1.getBlue() );

        Colour c2 = plainMaterial.getEmissiveColour(0,0);
        
        if( c2.getRed() != 0.4f ) fail( "Red component incorrect " + c2.getRed() );
        if( c2.getGreen() != 0.5f ) fail( "Green component incorrect " + c2.getGreen() );
        if( c2.getBlue() != 0.6f ) fail( "Blue component incorrect " + c2.getBlue() );
        
        if( plainMaterial.getReflectivity(0,0) != 0.7f ) fail( "Material has incorrect reflectivity " + plainMaterial.getReflectivity(0,0) );
      }
      else
      {
        fail( "Could not parse input string\n" + parser.explain() );
      }      
    }
    catch( Exception e )
    {
      fail( "Could not parse input string\n" + parser.explain() );
    }  
  }
}
