package liaquay.graphics.raytrace;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class TraceablePlane implements Traceable
{
  private MappedPlane3D _plane;
  private Materials _materials;

  public TraceablePlane( MappedPlane3D plane,
                         Materials materials )
  {
    _plane = plane;
    _materials = materials;
  }

  public Colour trace( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth )
  {
    // Work out where the ray collides with the plane.
    Vector3D t1 = intersection.sub( _plane.getNormal().mul( _plane.getDistance() ) );

    double x = _plane.getMappedX( t1 );
    double y = _plane.getMappedY( t1 );

    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 _plane.getNormal(),
                                                 depth );

    Colour ambient = box.getAmbientColour();

    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      ambient,
      tm.getReflectivity( x, y ),
      -_plane.getNormal().dot( direction ) );

    return colour;
  }

  public Vector3D intersection( Vector3D start,
                                Vector3D direction )
  {
    return _plane.halfLineIntersection( start, direction );
  }
}