package liaquay.graphics.raytrace;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class TraceablePolygon implements Traceable
{
  private MappedPlane3D _plane;
  private Materials _materials;
  private Polygon2D _polygon;

  public TraceablePolygon( MappedPlane3D plane,
                           double points[][],
                           Materials materials )
  {
    _plane = plane;
    _materials = materials;
    _polygon = new Polygon2D( points );
  }

  public Colour trace( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth )
  {
    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 _plane.getNormal(),
                                                 depth );

    Colour ambient = box.getAmbientColour();

    double cosAngleToNormal = _plane.getNormal().dot( direction );
    if( cosAngleToNormal < 0 ) cosAngleToNormal = -cosAngleToNormal;

    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      ambient,
      tm.getReflectivity( x, y ),
      cosAngleToNormal );

    return colour;
  }

  public boolean contains( Vector3D point )
  {
    if( _plane.contains( point, 0.0001 ) )
    {
      double x = _plane.getMappedX( point );
      double y = _plane.getMappedY( point );

      return _polygon.contains( x, y );
    }
    else
    {
      return false;
    }
  }

  public Vector3D intersection( Vector3D start,
                                Vector3D direction )
  {
    if( !_plane.above( start ) )
    {
      return null;
    }

    if( _plane.getNormal().dot( direction ) > 0 )
    {
      return null;
    }

    Vector3D intersection = _plane.halfLineIntersection( start, direction );

    if( intersection == null )
    {
      return null;
    }

    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    if( _polygon.contains( x, y ) )
    {
      return intersection;
    }

    return null;
  }

  public Vector3D getNormal()
  {
    return _plane.getNormal();
  }
}