package liaquay.graphics.raytrace;

import java.awt.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;

public class TraceableSphere implements Traceable
{
  private Sphere3D _sphere;
  private Materials _materials;

  public TraceableSphere( Sphere3D sphere,
                          Materials materials )
  {
    _sphere = sphere;
    _materials = materials;
  }

  public Colour trace( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth )
  {
    // Work out the reflection...

    // The normal from the center of the sphere to the point
    // on intersection.
    Vector3D r = intersection.sub( _sphere.getCenter() );

    // Normalise
    Vector3D normal = r.mul( 1 / _sphere.getRadius() );


    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 normal,
                                                 depth );

    Colour ambient = box.getAmbientColour();

    // Work out some mapping from sphere to plane...
    double x = 0;
    double y = 0;

    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      ambient,
      tm.getReflectivity( x, y ),
      -normal.dot( direction ) );

    return colour;
  }

  public Vector3D intersection( Vector3D start,
                                Vector3D direction )
  {
    return _sphere.nearestIntersection( start, direction );
  }
}
