package liaquay.graphics.raytrace;

import liaquay.math.vector.*;
import liaquay.graphics.*;

public class TraceableTranslucentPolygon// implements Traceable
{
  private MappedPlane3D _plane;
  private Materials _materials;
  private Polygon2D _polygon;

  public TraceableTranslucentPolygon( MappedPlane3D plane,
                                      double points[][],
                                      Materials materials )
  {
    _plane = plane;
    _materials = materials;
    _polygon = new Polygon2D( points );
  }

  public Vector3D getNormal()
  {
    return _plane.getNormal();
  }

  public Colour trace( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth,
                       double refractiveIndex,
                       float translucency )
  {
    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 _plane.getNormal(),
                                                 depth );

    Colour refraction = box.getRefractedColour( intersection,
                                                direction,
                                                _plane.getNormal(),
                                                refractiveIndex,
                                                depth );

    Colour ambient = box.getAmbientColour();

    double cosAngleToNormal = _plane.getNormal().dot( direction );
    if( cosAngleToNormal < 0 ) cosAngleToNormal = -cosAngleToNormal;


    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      refraction,
      ambient,
      tm.getReflectivity( x, y ),
      cosAngleToNormal,
      tm.getReflectiveColour( x, y ), // Refractive
      translucency );

    return colour;
  }


  public Colour trace1( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth )
  {
    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 _plane.getNormal(),
                                                 depth );

    Colour ambient = box.getAmbientColour();

    double cosAngleToNormal = _plane.getNormal().dot( direction );
    if( cosAngleToNormal < 0 ) cosAngleToNormal = -cosAngleToNormal;

    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      ambient,
      tm.getReflectivity( x, y ),
      cosAngleToNormal );

    return colour;
  }






  public boolean contains( Vector3D point, double tol )
  {
    if( _plane.contains( point, tol ) )
    {
      double x = _plane.getMappedX( point );
      double y = _plane.getMappedY( point );

      return _polygon.contains( x, y );
    }
    else
    {
      return false;
    }
  }

  public boolean entering( Vector3D direction )
  {
    return getNormal().dot( direction ) < 0;
  }

  public boolean exiting( Vector3D direction )
  {
    return getNormal().dot( direction ) > 0;
  }

  public Vector3D halfLineIntersection( Vector3D start,
                                        Vector3D direction,
                                        double minDistance )
  {
    Vector3D intersection = _plane.halfLineIntersection( start, direction, minDistance );

    if( intersection == null )
    {
      return null;
    }

    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    if( _polygon.contains( x, y ) )
    {
      return intersection;
    }

    return null;
  }

  public Vector3D intersection( Vector3D start,
                                Vector3D direction )
  {
    Vector3D intersection = _plane.intersection( start, direction );

    if( intersection == null )
    {
      return null;
    }

    double x = _plane.getMappedX( intersection );
    double y = _plane.getMappedY( intersection );

    if( _polygon.contains( x, y ) )
    {
      return intersection;
    }

    return null;
  }
}