package liaquay.graphics.raytrace;

import java.awt.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;

public class TraceableTranslucentSphere implements Traceable
{
  private Sphere3D _sphere;
  private Materials _materials;
  private double _permeability;
  private float _translucency;

  public TraceableTranslucentSphere( Sphere3D sphere,
                                     Materials materials,
                                     double permeability,
                                     float translucency )
  {
    _sphere = sphere;
    _materials = materials;
    _permeability = permeability;
    _translucency = translucency;
  }

  public Colour trace( Vector3D intersection,
                       Vector3D direction,
                       RayBox box,
                       int depth )
  {
    // Work out the reflection...

    // The normal from the center of the sphere to the point
    // on intersection.
    Vector3D r = intersection.sub( _sphere.getCenter() );

    // Normalise
    Vector3D normal = r.mul( 1 / _sphere.getRadius() );


    Colour reflection = box.getReflectiveColour( intersection,
                                                 direction,
                                                 normal,
                                                 depth );

    Colour refraction = box.getRefractedColour( intersection,
                                                direction,
                                                normal,
                                                _permeability,
                                                depth );

    Colour ambient = box.getAmbientColour();

    // Dot the normal with the direction of the line.
    // As spheres are convex if the value is negative the
    // ray is exiting the sphere.
    double t1 = direction.dot( normal );


    // Work out some mapping from sphere to plane...
    double x = 0;
    double y = 0;

    TexturedMaterial tm = _materials.getTexturedMaterial();

    Colour colour = RayBox.compose(
      tm.getReflectiveColour( x, y ),
      tm.getEmissiveColour( x, y ),
      reflection,
      refraction,
      ambient,
      tm.getReflectivity( x, y ),
      -t1,
      tm.getReflectiveColour( x, y ), // Refractive
      _translucency );

    return colour;
  }

  public Vector3D intersection( Vector3D start,
                                Vector3D direction )
  {
    return _sphere.nearestIntersection( start, direction );
  }
}
