package liaquay.graphics.raytrace.test;

import liaquay.graphics.*;
import liaquay.math.vector.*;

public class Octohedra
{
  private static Vector3D[] vertices =
    {
      new Vector3D(-10.0, -20.0,  20.0),
      new Vector3D(-20.0,  20.0,  10.0),
      new Vector3D( 20.0,  10.0,  20.0),
      new Vector3D(-20.0, -10.0, -20.0),
      new Vector3D( 10.0,  20.0, -20.0),
      new Vector3D( 20.0, -20.0, -10.0),
    };

  private static Solid3DFace[] faces =
    {
      new Solid3DFace( new int[] { 2, 1, 0 } ),
      new Solid3DFace( new int[] { 1, 3, 0 } ),
      new Solid3DFace( new int[] { 2, 4, 1 } ),
      new Solid3DFace( new int[] { 0, 5, 2 } ),
      new Solid3DFace( new int[] { 1, 4, 3 } ),
      new Solid3DFace( new int[] { 2, 5, 4 } ),
      new Solid3DFace( new int[] { 0, 3, 5 } ),
      new Solid3DFace( new int[] { 3, 4, 5 } )
    };


  public static OpaqueSolid3D create()
  {
    PlainMaterial material1 = new PlainMaterial( new Colour( 1.00f, 0.75f, 0.75f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material2 = new PlainMaterial( new Colour( 0.75f, 1.00f, 0.75f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material3 = new PlainMaterial( new Colour( 0.75f, 0.75f, 1.00f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material4 = new PlainMaterial( new Colour( 1.00f, 0.75f, 1.00f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    Materials materials1 = new Materials( material1 );
    Materials materials2 = new Materials( material2 );
    Materials materials3 = new Materials( material3 );
    Materials materials4 = new Materials( material4 );

    return create( materials1, materials2, materials3, materials4 );
  }

  public static TranslucentSolid3D createTranslucent()
  {
    PlainMaterial material1 = new PlainMaterial( new Colour( 1.00f, 1.00f, 1.0f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    Materials materials1 = new Materials( material1 );
    Materials materials2 = new Materials( material1 );
    Materials materials3 = new Materials( material1 );
    Materials materials4 = new Materials( material1 );

    Materials[] materials =
    {
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,

      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,

      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,

      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
    };

    Solid3D solid = new Solid3D( vertices, faces );

    return new TranslucentSolid3D( solid, materials, 1.0, 0.7f );
  }

  public static OpaqueSolid3D create( Materials materials1,
                                      Materials materials2,
                                      Materials materials3,
                                      Materials materials4 )
  {

 /*
    PlainMaterial material1 = new PlainMaterial( new Colour( 1.00f, 0.75f, 0.75f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material2 = new PlainMaterial( new Colour( 0.75f, 1.00f, 0.75f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material3 = new PlainMaterial( new Colour( 0.75f, 0.75f, 1.00f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );

    PlainMaterial material4 = new PlainMaterial( new Colour( 1.00f, 0.75f, 1.00f ),
                                                 new Colour( 0f,0f,0f ),
                                                 0f );


 */
    Materials[] materials =
    {
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,
      materials1,

      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,
      materials2,

      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,
      materials3,

      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
      materials4,
    };

    Solid3D solid = new Solid3D( vertices, faces );

    return new OpaqueSolid3D( solid, materials );
  }
}