package liaquay.graphics.raytrace.test;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.graphics.material.*;
import liaquay.graphics.raytrace.*;

public class RayTraceApplet extends Applet
{
  private Image rayImage = null;
  private Graphics3D graphics3D = null;

  public void init()
  {
    setLayout( new BorderLayout() );

    Dimension size = getSize();

    int width = size.width;
    int height = size.height;

    rayImage = createImage( width, height );

    graphics3D = new RayTraceGraphics3D( rayImage, 8, 4 );

    add( new RayTraceCanvas(rayImage), BorderLayout.CENTER );

    graphics3D.beginScene( new Camera3D( Transform.createIdentity(), 1.0 ) );

    PlainMaterial material1 = new PlainMaterial( new Colour( 1f,1f,1f ),
                                                 new Colour( 0f,0f,0f ),
                                                 1f );

    PlainMaterial material2 = new PlainMaterial( new Colour( 1f,0f,1f ),
                                                 new Colour( 0f,0f,0f ),
                                                 1f );

    MandlebrotMaterial material3 = new MandlebrotMaterial( 200, 0f, false );

    CheckedMaterial checkedMaterial = new CheckedMaterial( material1,
                                                           material2,
                                                           6 );

    ImageMaterial imageMaterial = new ImageMaterial( readImage( "reptile-tile.jpg" ),
                                                     10,
                                                     1.0f,
                                                     false );

    Materials materials1 = new Materials( material1,
                                          imageMaterial );

    Materials materials2 = new Materials( material2 );

    Materials materials3 = new Materials( material1,
                                          checkedMaterial);

    Materials materials4 = new Materials( material1 );

    Materials materials5 = new Materials( material1,
                                          material3 );

    OpaqueSphere3D sphere1 = new OpaqueSphere3D( new Sphere3D( 15, new Vector3D( 0,18,120 ) ),
                                                 materials4 );


    TranslucentSphere3D sphere2 = new TranslucentSphere3D(
      new Sphere3D( 25, new Vector3D( 0,0,120 ) ),
      materials4,
      materials4,
      1.6,
      1.0f );



    OpaquePlane3D plane1 = new OpaquePlane3D( new MappedPlane3D( new Vector3D( 0,-20, 0 ),
                                                                 new Vector3D( 1,-20, 0 ),
                                                                 new Vector3D( 0,-20, 1 ),
                                                                 false ),
                                               materials1 );

    OpaquePlane3D plane2 = new OpaquePlane3D( new MappedPlane3D( new Vector3D( 0,-20, 0 ),
                                                                 new Vector3D( 1,-20, 0 ),
                                                                 new Vector3D( 0,-20, 1 ),
                                                                 false ),
                                               materials5 );


    OpaquePlane3D plane3 = new OpaquePlane3D( new MappedPlane3D( new Vector3D( 0, 0, 400 ),
                                                                 new Vector3D( 1, 0, 400 ),
                                                                 new Vector3D( 0, 1, 400 ),
                                                                 false ),
                                               materials5 );

    //graphics3D.add( sphere2, Transform.createIdentity(), null );
    graphics3D.add( plane3,  Transform.createIdentity(), null );
    //graphics3D.add( FourOctohedra.create( materials1, materials2, materials3, materials4 ),  Transform.createRotateTranslate(.4,.4,0,0,0,120), null );
    //graphics3D.add( FourOctohedra.createTranslucent(),  Transform.createRotateTranslate(.4,.4,0,0,0,120), null );
    graphics3D.add( Octohedra.createTranslucent(),  Transform.createRotateTranslate(.4,.4,0,0,0,120), null );

    new Thread()
    {
      public void run()
      {
        graphics3D.endScene();
      }
    }.start();
  }

  private Image readImage( String imageName )
  {
    Image image = getImage( getDocumentBase(), imageName );
    try
    {
      MediaTracker mt = new MediaTracker(this);
      mt.addImage( image, 0 );
      mt.waitForID( 0 );
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return image;
  }
}