package liaquay.graphics.wireframe;

import java.awt.*;
import liaquay.math.vector.*;
import liaquay.graphics.*;
import liaquay.collection.*;

public class ViewListEntry
{
  private double _distance = 0;
  private Vector3D _center = new Vector3D(0,0,0);
  private Matrix4x3 _combinedTransform = Matrix4x3.createIdentity();
  private double _radius = 0;
  private int _screenX = 0;
  private int _screenY = 0;
  private int _screenR = 0;
  private Object _object = null;
  private Renderer _renderer = null;
  private SelectionListener _selectionListener = null;

  private Transform _t1 = new Transform();
  
  public final void initialise( Transform inverseCamera,
                                Transform transform,
                                Vector3D center,
                                double radius,
                                double ppd,
                                Object object,
                                Renderer renderer,
                                double width,
                                double height,
                                SelectionListener selectionListener )
  {
    transform.mul( inverseCamera, _t1 );
    _combinedTransform.set( _t1 );
    center.mul( _combinedTransform, _center );
    _distance = _center.get( 2 );
    _radius = radius;
    _screenX =  (int)( _center.perx( ppd ) * (double)width );
    _screenY = -(int)( _center.pery( ppd ) * (double)width ) ;
    _screenR =  (int)( ( radius * ppd * (double)width ) / _center.get( 2 ) );
    _object = object;
    _renderer = renderer;
    _selectionListener = selectionListener;
  }

  public final int compareTo( ViewListEntry v )
  {
    if( _distance < v._distance ) return 1;
    else if( _distance > v._distance ) return -1;
    else return 0;
  }

  public final void render( Graphics g, double ppd, int width, int height )
  {
    _renderer.render( _object,
                      g,
                      _combinedTransform,
                      ppd,
                      width,
                      height,
                      _screenX,
                      _screenY,
                      _screenR );
  }

  public final double getDistance()
  {
    return _distance;
  }

  public final Vector3D getCenter()
  {
    return _center;
  }

  public final int getScreenCenterX()
  {
    return _screenX;
  }

  public final int getScreenCenterY()
  {
    return _screenY;
  }

  public final int getScreenRadius()
  {
    return _screenR;
  }

  public final SelectionListener getSelectionListener()
  {
    return _selectionListener;
  }

  public final boolean pick( int sx, int sy, double ppd )
  {
    double x = _screenX - sx;
    double y = _screenY - sy;
    double r = 8;

    if( _screenR > 8 )
    {
      r = _screenR + 8;
    }

    return ( x * x ) + ( y * y ) < ( r * r );
  }

  public boolean onScreen( int width,
                           int height )
  {
    if( _distance > 0 )
    {
      // Calculate screen coords and radius
      int x = _screenX;
      int y = _screenY;
      int xoff = width / 2;
      int yoff = height / 2;
      int r = _screenR;

      if( (
            ( ( x  > 0 ) && ( x - r <  xoff ) ) ||
            ( ( x <= 0 ) && ( x + r > -xoff ) )
          )
          &&
          (
            ( ( y  > 0 ) && ( y - r <  yoff ) ) ||
            ( ( y <= 0 ) && ( y + r > -yoff ) )
          )
        )
      {
        return true;
      }
      return false;
    }
    else
    {
      return false;
    }
  }
}