package liaquay.math.vector;



public class Matrix4x3
{
  double[][] _values;

  public Matrix4x3()
  {
    _values = new double[4][3];
  }

  public Matrix4x3( Transform t )
  {
    _values = new double[4][3];
    set( t );
  }

  public Matrix4x3( double values[][] )
  {
    _values = values;
  }

  public Matrix4x3( Matrix4x3 m )
  {
    _values = new double[4][3];

    for( int i = 0; i < 4; ++ i )
    {
      for( int j = 0; j < 3; ++j )
      {
        _values[ i ][ j ] = m._values[ i ][ j ];
      }
    }
  }

  public Matrix4x3(
    double r11, double r21, double r31, double r41,
    double r12, double r22, double r32, double r42,
    double r13, double r23, double r33, double r43 )
  {
    _values = new double[4][3];

    _values[ 0 ][ 0 ] = r11; _values[ 1 ][ 0 ] = r21; _values[ 2 ][ 0 ] = r31; _values[ 3 ][ 0 ] = r41;
    _values[ 0 ][ 1 ] = r12; _values[ 1 ][ 1 ] = r22; _values[ 2 ][ 1 ] = r32; _values[ 3 ][ 1 ] = r42;
    _values[ 0 ][ 2 ] = r13; _values[ 1 ][ 2 ] = r23; _values[ 2 ][ 2 ] = r33; _values[ 3 ][ 2 ] = r43;
  }


  public final Matrix4x3 mul( Matrix4x3 m )
  {
    return mul( m, null );
  }

  public final Matrix4x3 mul( Matrix4x3 m, Matrix4x3 result )
  {
    double v[][] = m.getValues();
    double newValues[][];

    if( result == null )
    {
      newValues = new double[4][3];
    }
    else
    {
      newValues = result.getValues();
    }

    double r00 = ( v[ 0 ][ 0 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 0 ][ 2 ] );

    double r10 = ( v[ 0 ][ 0 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 1 ][ 2 ] );

    double r20 = ( v[ 0 ][ 0 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 2 ][ 2 ] );

    double r30 = ( v[ 0 ][ 0 ] * _values[ 3 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 3 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 3 ][ 2 ] ) +
          ( v[ 3 ][ 0 ] );

    double r01 = ( v[ 0 ][ 1 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 0 ][ 2 ] );

    double r11 = ( v[ 0 ][ 1 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 1 ][ 2 ] );

    double r21 = ( v[ 0 ][ 1 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 2 ][ 2 ] );

    double r31 = ( v[ 0 ][ 1 ] * _values[ 3 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 3 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 3 ][ 2 ] ) +
          ( v[ 3 ][ 1 ] );

    double r02 = ( v[ 0 ][ 2 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 0 ][ 2 ] );

    double r12 = ( v[ 0 ][ 2 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 1 ][ 2 ] );

    double r22 = ( v[ 0 ][ 2 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 2 ][ 2 ] );

    double r32 = ( v[ 0 ][ 2 ] * _values[ 3 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 3 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 3 ][ 2 ] ) +
          ( v[ 3 ][ 2 ] );

    newValues[ 0 ][ 0 ] = r00; newValues[ 1 ][ 0 ] = r10; newValues[ 2 ][ 0 ] = r20; newValues[ 3 ][ 0 ] = r30;
    newValues[ 0 ][ 1 ] = r01; newValues[ 1 ][ 1 ] = r11; newValues[ 2 ][ 1 ] = r21; newValues[ 3 ][ 1 ] = r31;
    newValues[ 0 ][ 2 ] = r02; newValues[ 1 ][ 2 ] = r12; newValues[ 2 ][ 2 ] = r22; newValues[ 3 ][ 2 ] = r32;

    if( result == null )
    {
      return new Matrix4x3( newValues );
    }
    else
    {
      return result;
    }
  }

  public final Matrix4x3 inertiaMul( Matrix4x3 m )
  {
    return inertiaMul( m, null );
  }

  public final Matrix4x3 inertiaMul( Matrix4x3 m, Matrix4x3 result )
  {
    double v[][] = m.getValues();
    double newValues[][];

    if( result == null )
    {
      newValues = new double[4][3];
    }
    else
    {
      newValues = result.getValues();
    }

    double r00 = ( v[ 0 ][ 0 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 0 ][ 2 ] );

    double r10 = ( v[ 0 ][ 0 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 1 ][ 2 ] );

    double r20 = ( v[ 0 ][ 0 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 0 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 0 ] * _values[ 2 ][ 2 ] );

    double r01 = ( v[ 0 ][ 1 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 0 ][ 2 ] );

    double r11 = ( v[ 0 ][ 1 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 1 ][ 2 ] );

    double r21 = ( v[ 0 ][ 1 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 1 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 1 ] * _values[ 2 ][ 2 ] );

    double r02 = ( v[ 0 ][ 2 ] * _values[ 0 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 0 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 0 ][ 2 ] );

    double r12 = ( v[ 0 ][ 2 ] * _values[ 1 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 1 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 1 ][ 2 ] );

    double r22 = ( v[ 0 ][ 2 ] * _values[ 2 ][ 0 ] ) +
          ( v[ 1 ][ 2 ] * _values[ 2 ][ 1 ] ) +
          ( v[ 2 ][ 2 ] * _values[ 2 ][ 2 ] );


    newValues[ 0 ][ 0 ] = r00; newValues[ 1 ][ 0 ] = r10; newValues[ 2 ][ 0 ] = r20;
    newValues[ 0 ][ 1 ] = r01; newValues[ 1 ][ 1 ] = r11; newValues[ 2 ][ 1 ] = r21;
    newValues[ 0 ][ 2 ] = r02; newValues[ 1 ][ 2 ] = r12; newValues[ 2 ][ 2 ] = r22;

    newValues[ 3 ][ 0 ] = v[ 3 ][ 0 ] + _values[ 3 ][ 0 ];
    newValues[ 3 ][ 1 ] = v[ 3 ][ 1 ] + _values[ 3 ][ 1 ];
    newValues[ 3 ][ 2 ] = v[ 3 ][ 2 ] + _values[ 3 ][ 2 ];


    if( result == null )
    {
      return new Matrix4x3( newValues );
    }
    else
    {
      return result;
    }
  }

  public final double[][] getValues()
  {
    return _values;
  }

  public final double get( int i, int j )
  {
    return _values[ i ][ j ];
  }

  public final void set( int i, int j, double f )
  {
    _values[ i ][ j ] = f;
  }



  public Matrix4x3 inverseRotateTranslate()
  {
    return inverseRotateTranslate( null );
  }

  public Matrix4x3 inverseRotateTranslate( Matrix4x3 result )
  {
    double[][]v;

    if( result == null )
    {
      v = new double[ 4 ][ 3 ];
    }
    else
    {
      v = result.getValues();
    }

    double r00 = _values[ 0 ][ 0 ]; double r10 = _values[ 1 ][ 0 ]; double r20 = _values[ 2 ][ 0 ];
    double r01 = _values[ 0 ][ 1 ]; double r11 = _values[ 1 ][ 1 ]; double r21 = _values[ 2 ][ 1 ];
    double r02 = _values[ 0 ][ 2 ]; double r12 = _values[ 1 ][ 2 ]; double r22 = _values[ 2 ][ 2 ];

    double x = _values[ 3 ][ 0 ];
    double y = _values[ 3 ][ 1 ];
    double z = _values[ 3 ][ 2 ];


    v[ 0 ][ 0 ] = r00; v[ 1 ][ 0 ] = r01; v[ 2 ][ 0 ] = r02;
    v[ 0 ][ 1 ] = r10; v[ 1 ][ 1 ] = r11; v[ 2 ][ 1 ] = r12;
    v[ 0 ][ 2 ] = r20; v[ 1 ][ 2 ] = r21; v[ 2 ][ 2 ] = r22;

    v[ 3 ][ 0 ] = -( (r00*x) + (r01*y) + (r02*z) ); // X
    v[ 3 ][ 1 ] = -( (r10*x) + (r11*y) + (r12*z) ); // Y
    v[ 3 ][ 2 ] = -( (r20*x) + (r21*y) + (r22*z) ); // Z

    if( result == null )
    {
      return new Matrix4x3( v );
    }
    else
    {
      return result;
    }
  }

  public final double getDx()
  {
    return _values[ 3 ][ 0 ];
  }

  public final double getDy()
  {
    return _values[ 3 ][ 1 ];
  }

  public final double getDz()
  {
    return _values[ 3 ][ 2 ];
  }

  public final void setDx( double s )
  {
     _values[ 3 ][ 0 ] = s;
  }

  public final void setDy( double s )
  {
     _values[ 3 ][ 1 ] = s;
  }

  public final void setDz( double s )
  {
     _values[ 3 ][ 2 ] = s;
  }

  public final void rotateTranslate( double a, double b, double c, double x, double y, double z )
  {
    double sa = Math.sin( a );
    double ca = Math.cos( a );
    double sb = Math.sin( b );
    double cb = Math.cos( b );
    double sc = Math.sin( c );
    double cc = Math.cos( c );

    double[][]v = _values;

    v[ 0 ][ 0 ] = (cc*cb)-(sc*sa*sb); v[ 1 ][ 0 ] = -(sc*ca); v[ 2 ][ 0 ] = (cc*sb)+(sc*sa*cb); v[ 3 ][ 0 ] = x;
    v[ 0 ][ 1 ] = (sc*cb)+(cc*sa*sb); v[ 1 ][ 1 ] = (cc*ca);  v[ 2 ][ 1 ] = (sc*sb)-(cc*sa*cb); v[ 3 ][ 1 ] = y;
    v[ 0 ][ 2 ] = -(ca*sb);           v[ 1 ][ 2 ] = (sa);     v[ 2 ][ 2 ] = (ca*cb);            v[ 3 ][ 2 ] = z;
  }

  public final void set( Transform t )
  {
    double[][]v = _values;
    double x = t.getQx();
    double y = t.getQy();
    double z = t.getQz();
    double w = t.getQw();

    double xx = x*x; double yy = y*y; double zz = z*z;
    double xy = x*y; double xz = x*z; double yz = y*z;
    double wx = w*x; double wy = w*y; double wz = w*z;
    double ww = w*w;

    v[ 0 ][ 0 ] =   ww + xx - yy - zz;
    v[ 1 ][ 0 ] =     2 * ( xy - wz );
    v[ 2 ][ 0 ] =     2 * ( xz + wy );
    v[ 3 ][ 0 ] =           t.getDx();

    v[ 0 ][ 1 ] =     2 * ( xy + wz );
    v[ 1 ][ 1 ] =   ww - xx + yy - zz;
    v[ 2 ][ 1 ] =     2 * ( yz - wx );
    v[ 3 ][ 1 ] =           t.getDy();

    v[ 0 ][ 2 ] =     2 * ( xz - wy );
    v[ 1 ][ 2 ] =     2 * ( yz + wx );
    v[ 2 ][ 2 ] =   ww - xx - yy + zz;
    v[ 3 ][ 2 ] =           t.getDz();
  }

  public final void mop()
  {
    double[][]v = _values;
    double x = 0;
    double y = 0;
    double z = 0;
    double w = 0;
    if( v[0][0] + v[1][1] + v[2][2] + 1 > 0 )
    {
      double s = Math.sqrt( v[0][0] + v[1][1] + v[2][2] + 1 );
      x = ( v[1][2] - v[2][1] ) / ( 2*s );
      y = ( v[2][0] - v[0][2] ) / ( 2*s );
      z = ( v[0][1] - v[1][0] ) / ( 2*s );
      w = 0.5f * s;
    }
    else
    {
      System.out.println( "Baaaaad" );
      return;
    }

    double xx = x*x; double yy = y*y; double zz = z*z;
    double xy = x*y; double xz = x*z; double yz = y*z;
    double wx = w*x; double wy = w*y; double wz = w*z;

    v[ 0 ][ 0 ] = 1 - 2 * ( yy + zz );
    v[ 1 ][ 0 ] =     2 * ( xy - wz );
    v[ 2 ][ 0 ] =     2 * ( xz + wy );

    v[ 0 ][ 1 ] =     2 * ( xy + wz );
    v[ 1 ][ 1 ] = 1 - 2 * ( xx + zz );
    v[ 2 ][ 1 ] =     2 * ( yz - wx );

    v[ 0 ][ 2 ] =     2 * ( xz - wy );
    v[ 1 ][ 2 ] =     2 * ( yz + wx );
    v[ 2 ][ 2 ] = 1 - 2 * ( xx + yy );
  }


  public String toString()
  {
    return
      ""   + _values[ 0 ][ 0 ] + ", " + _values[ 1 ][ 0 ] + ", " + _values[ 2 ][ 0 ] + ", " + _values[ 3 ][ 0 ] +
      "\n" + _values[ 0 ][ 1 ] + ", " + _values[ 1 ][ 1 ] + ", " + _values[ 2 ][ 1 ] + ", " + _values[ 3 ][ 1 ] +
      "\n" + _values[ 0 ][ 2 ] + ", " + _values[ 1 ][ 2 ] + ", " + _values[ 2 ][ 2 ] + ", " + _values[ 3 ][ 2 ];
  }

  public static Matrix4x3 createIdentity()
  {
    Matrix4x3 m = new Matrix4x3( 1, 0, 0, 0,
                                 0, 1, 0, 0,
                                 0, 0, 1, 0 );

    return m;
  }

  public static Matrix4x3 createRotateTranslate( double a, double b, double c, double x, double y, double z )
  {
    Matrix4x3 m = new Matrix4x3();
    m.rotateTranslate( a,b,c,x,y,z );
    return m;
  }
}