package liaquay.math.vector;

public class Polygon2D
{
  private double _points[][];
  private double _centerX;
  private double _centerY;
  private double _radiusSquared;

  public Polygon2D( double points[][] )
  {
    _points = points;

    calculateCenter();
    calculateRadiusSquared();
  }

  private void calculateCenter()
  {
    double x = 0.0;
    double y = 0.0;

    for( int i = 0; i < _points.length; ++i )
    {
      x += _points[ i ][ 0 ];
      y += _points[ i ][ 1 ];
    }

    _centerX = x;
    _centerY = y;
  }

  private void calculateRadiusSquared()
  {
    double rr = 0.0;

    for( int i = 0; i < _points.length; ++i )
    {
      double x = _points[ i ][ 0 ] - _centerX;
      double y = _points[ i ][ 1 ] - _centerY;
      double r = ( x*x ) + ( y*y );
      if( r > rr )
      {
        rr = r;
      }
    }

    _radiusSquared = rr;
  }

  public boolean contains( double xt, double yt)
  {
     double xc = xt - _centerX;
     double yc = yt - _centerY;

     if( ( xc * xc ) + ( yc * yc ) > _radiusSquared )
     {
       return false;
     }

    // Points on the boundary count as in.
    // (If we assume all polygons are described anticlockwise)

    if( _points.length < 3 ) // Is this really necessary?
    {
      return false;
    }

    double x1 = _points[ _points.length-1 ][ 0 ] - xt;
    double y1 = _points[ _points.length-1 ][ 1 ] - yt;
    double x0 = 0;
    double y0 = 0;
    int windingNumber = 0;

    for( int i = 0; i < _points.length; i++ )
    {
      x0 = x1;
      y0 = y1;
      x1 = _points[ i ][ 0 ] - xt;
      y1 = _points[ i ][ 1 ] - yt;

      // Now consider a ray on the +ve x axis
      int contribution = 0;
      if( y0 <= 0 && y1 > 0 )
      {
        // Potential +ve contribution - include boundaries.
        if( x0 >= 0 && x1 >= 0 )
        {
          contribution = 1;
        }
        else if( x0 < 0 && x1 < 0 )
        {
          contribution = 0;
        }
        else
        {
          // Calculate crossing point
          // xcross = x1 - (x1-x0) * y1 / ( y1 - y0 ) >= 0
          if( (y1 - y0)*x1 >= (x1 - x0) * y1 )
          {
            contribution = 1;
          }
        }
      }
      else if( y1 <= 0 && y0 > 0 )
      {
        // Potential -ve contribution - exclude boundaries.
        if( x0 > 0 && x1 > 0 )
        {
          contribution = -1;
        }
        else if( x0 <= 0 && x1 <= 0 )
        {
          contribution = 0;
        }
        else
        {
          // Calculate crossing point
          // xcross = x0 - (x0-x1) * y0 / ( y0 - y1 ) > 0
          if( (y0 - y1)*x0 > (x0 - x1) * y0 )
          {
            contribution = -1;
          }
        }

      }
      windingNumber += contribution;
    }
    return windingNumber != 0;
  }
}