package liaquay.stream;

import java.io.*;

public class SeekableBitFromByteInputStream implements SeekableBitInputStream, BitInputStream
{
  private SeekableByteInputStream _in;
  private long _index = 0;
  private byte _currentByte;
  private long _length;

  public SeekableBitFromByteInputStream( SeekableByteInputStream in,
                                         long length ) throws IOException
  {
    _in = in;
    _in.seek( _index );
    if( length > 0 )
    {
      _currentByte = _in.read();
    }
    _length = length;
  }

  public boolean read() throws IOException
  {
    boolean bit = ( _currentByte & ( 1 << ( 7 - ( _index & 7 ) ) ) ) != 0;

    ++_index;

    if( !eof() && (( _index & 7 ) == 0) )
    {
      _currentByte = _in.read();
    }

    return bit;
  }

  public boolean eof() throws IOException
  {
    return _index >= _length;
  }

  public void close() throws IOException
  {
    _in.close();
  }

  public void seek( long index ) throws IOException
  {
    if( ( index & ~7 ) != ( _index & ~7 ) )
    {
      _in.seek( index >> 3 );
      _currentByte = _in.read();
    }
    _index = index;
  }

  public long getCurrentIndex() throws IOException
  {
    return _index;
  }

}
