package liaquay.stream;

import java.io.*;

public class StreamBitOutputStream implements BitOutputStream
{
  private OutputStream _out;
  private long _index = 0;
  private int _currentByte = 0;

  public StreamBitOutputStream( OutputStream out )
  {
    _out = out;
  }

  public void write( boolean bit ) throws IOException
  {
    if( bit )
    {
      _currentByte |= 1 << ( 7 - ( _index & 7 ) );
    }

    if( ( _index & 7 ) == 7 )
    {
      _out.write( _currentByte );
      _currentByte = 0;
    }

    ++_index;
  }

  public void write( boolean[] bits ) throws IOException
  {
    for( int i = 0; i < bits.length; ++i )
    {
      write( bits[ i ] );
    }
  }

  /**
   * Gets the length of the file in bits
   */
  public long length()
  {
    return _index;
  }

  /**
   * This does not close stream this is writing to.
   */
  public void close() throws IOException
  {
    if( ( _index & 7 ) != 0 )
    {
      _out.write( _currentByte );
    }
  }
}
