package liaquay.text;

import java.io.*;

public class IndentWriter extends Writer
{
  private Writer _writer;
  private int _indentStep = 2;
  private int _indent = 0;

  public IndentWriter( Writer writer )
  {
    _writer = writer;
  }
  
  public IndentWriter( Writer writer, int indentStep )
  {
    _indentStep = indentStep;
    _writer = writer;
  }
 
  public void flush() throws IOException
  {
    _writer.flush();
  }

  public void close() throws IOException
  {
    _writer.close();
  }
  
  public void write(char[] cbuf, int off, int len) throws IOException
  {
    for( int i = 0; i < len; ++i )
    {
       writeCharacter( cbuf[ i + off ] );
    }
  }

  public void increaseIndent()
  {
    _indent += _indentStep;
  }
  
  public void decreaseIndent()
  {
    _indent -= _indentStep;
  } 
  
  public int getIndent()
  {
    return _indent;
  }
  
  private void writeCharacter( char c ) throws IOException
  {
    _writer.write( c );
    if( c == '\n' )
    {
      for( int i = 0; i < _indent; ++i )
      {
        _writer.write( ' ' );
      }
    }
  }
}