package liaquay.text.parser;

import java.io.*;

public class CharReader
{
  private CharBuffer _charBuffer = new CharBuffer();
  private CharBuffer _rewindBuffer = new CharBuffer();
  private Reader _inputStream = null;
  private Region _region = null;
  private int _lineNumber = 0;
  private int _characterPosition = 0;

  public CharReader( Reader inputStream )
  {
    _inputStream = inputStream;
  }

  private int getNextChar() throws IOException
  {
    if( _rewindBuffer.size() > 0 )
    {
      return _rewindBuffer.remove();
    }
    else
    {
      return _inputStream.read();
    }
  }

  public Region getRegion()
  {
    return _region;
  }

  public int read() throws IOException
  {
    if( _region == null )
    {
      throw new IOException( "Read outside of region" );
    }
    else
    {
      int c = getNextChar();

      if( c != -1 )
      {
        _charBuffer.add( (char)c );
      }
 //System.out.println( "Char " + c);
      return c;
    }
  }

  public void rewind( int n ) throws IOException
  {
    // if( ) etc

    for( int i = 0; i < n; ++i )
    {
      _rewindBuffer.add( _charBuffer.remove() );
    }
  }

  public Region mark()
  {
    _region = new Region( this, _region, _charBuffer.size() );
    return _region;
  }

  public void rollback()
  {
    int i = _region.getStart();

    while( _charBuffer.size() > i )
    {
      _rewindBuffer.add( _charBuffer.remove() );
    }
    _region = _region.getParent();
  }

  public void commit()
  {
    _region = _region.getParent();
    if( _region == null )
    {
      _charBuffer.removeAll();
    }
  }

  public char getChar( int i )
  {
    return _charBuffer.get( i );
  }

  public char[] getChars( int start )
  {
    int length = _charBuffer.size() - start;
    char b[] = new char[ length ];
    for( int i = 0; i < length; ++i )
    {
      b[ i ] = _charBuffer.get( i + start );
    }
    return b;
  }
}
