package liaquay.text.parser;

import java.io.*;
import liaquay.text.IndentWriter;

public abstract class CharsParser
{
  private boolean _matched = false;
  private boolean _keepContents = true;
  private char[] _b = null;

  protected abstract boolean parseChars( CharReader r )
    throws IOException, ParserException;

  // private ??
  public boolean parse( CharReader reader )
    throws IOException, ParserException
  {
    Region region = reader.mark();
    _matched = parseChars( reader );
    if( _matched )
    {
      matched( region );
      reader.commit();
    }
    else
    {
      reader.rollback();
    }
    return _matched;
  }

  public boolean parseSequential( CharReader reader )
    throws IOException, ParserException
  {
    if( _keepContents )
    {
      parse( reader );
    }
    else
    {
      _matched = parseChars( reader );
    }

    if( !_matched )
    {
      if( reader.getRegion() == null )
      {
        throw new ParserException( "Could not parse this" );
      }
    }

    return _matched;
  }

  public void clear()
  {
    _matched = false;
  }

  public boolean getMatched()
  {
    return _matched;
  }

  protected void matched( Region region )
  {
    if( _keepContents )
    {
      _b = region.getChars();
    }
  }

  public char[] getMatchedChars()
  {
    return _b;
  }

  public boolean getKeepContents()
  {
    return _keepContents;
  }

  public void setKeepContents( boolean keepContents )
  {
    _keepContents = keepContents;
  }

  public String getName()
  {
    return this.getClass().getName();
  }

  public void explain( IndentWriter writer ) throws IOException
  {
    if( getMatched() )
    {
      writer.write( getName() + " matched ok\n" );
    }
    else
    {
      writer.write( getName() + " failed to match\n" );
    }
  }

  public String explain()
  {
    CharArrayWriter s = new CharArrayWriter();
    IndentWriter w = new IndentWriter( s );
    try
    {
      explain( w );
      w.close();
      return s.toString();
    }
    catch( IOException e )
    {
      throw new RuntimeException( "IOException writing to CharArrayWriter!" );
    }
  }

  public boolean parseSequential( String s ) throws ParserException
  {
    try
    {
      CharArrayReader inputStream = new CharArrayReader( s.toCharArray() );

      CharReader reader = new CharReader( inputStream );

      return parseSequential( reader );
    }
    catch( IOException e )
    {
      throw new RuntimeException( "IOException reading String!" );
    }
  }
}
