package liaquay.text.parser;

import java.io.*;
import liaquay.collection.ArrayList;
import liaquay.text.IndentWriter;

public abstract class CompoundParser extends CharsParser
{
  private ArrayList _parsers = new ArrayList();

  public void add( CharsParser parser )
  {
    _parsers.add( parser );
  }

  public void removeAll()
  {
    _parsers.removeAll();
  }

  public void clear()
  {
    for( int i = 0; i < _parsers.size(); ++i )
    {
      CharsParser parser = getParser( i );
      parser.clear();
    }

    super.clear();
  }

  public int getNumberOfParsers()
  {
    return _parsers.size();
  }

  public CharsParser getParser( int i )
  {
    return (CharsParser)_parsers.get( i );
  }

  public void explain( IndentWriter writer ) throws IOException
  {
    if( getMatched() )
    {
      writer.write( getName() + " matched ok\n" );
    }
    else
    {
      writer.write( getName() + " failed to match\n" );

      writer.increaseIndent();

      for( int i = 0; i < getNumberOfParsers(); ++i )
      {
        CharsParser parser = getParser( i );

        parser.explain( writer );
      }

      writer.decreaseIndent();
    }
  }
}
