package liaquay.text.parser;

public class MatchCharRegion implements MatchChar
{
  private static final int MAX_CHAR = 65535;
  private int flags[] = new int[ ( MAX_CHAR + 1 ) / 32 ];

  private boolean getFlag( char c )
  {
    int ch = ((int)c) & MAX_CHAR;
    int index = ch >> 5;
    int bit = ch & 31;

    return ( flags[ index ] & ( 1 << bit ) ) != 0;
  }

  private void setFlag( char c, boolean b )
  {
    int ch = ((int)c) & MAX_CHAR;
    int index = ch >> 5;
    int bit = ch & 31;

    if( b )
    {
      flags[ index ] |= 1 << bit;
    }
    else
    {
      flags[ index ] &= ~(1 << bit);
    }
  }

  public boolean match( char b )
  {
    return getFlag( b );
  }

  public MatchCharRegion()
  {
    for( int i = 0; i < flags.length; ++i )
    {
      flags[ i ] = 0;
    }
  }

  public void remove( char a, char b )
  {
    for( char i = a; i <= b; ++i )
    {
      setFlag( i, false );
    }
  }

  public void add( char a, char b )
  {
    for( char i = a; i <= b; ++i )
    {
      setFlag( i, true );
    }
  }

  public void remove( char a )
  {
    setFlag( a, false );
  }

  public void add( char a )
  {
    setFlag( a, true );
  }

  private int numberOfMatches()
  {
    int c = 0;
    for( int i = 0; i <= MAX_CHAR; ++i )
    {
      if( getFlag( (char)i ) ) ++c;
    }
    return c;
  }

  public char[] allMatches()
  {
    char[] bs = new char[ numberOfMatches() ];
    int c = 0;
    for( int i = 0; i <= MAX_CHAR; ++i )
    {
      if( getFlag( (char)i ) )
      {
        bs[ c++ ] = (char)i;
      }
    }
    return bs;
  }

  public void add( MatchChar bm )
  {
    char[] bs = bm.allMatches();

    for( int i = 0; i < bs.length; ++i )
    {
      setFlag( bs[ i ], true );
    }
  }

  public void remove( MatchChar bm )
  {
    char[] bs = bm.allMatches();

    for( int i = 0; i < bs.length; ++i )
    {
      setFlag( bs[ i ], false );
    }
  }
}


