package liaquay.xml;

import java.io.*;
import liaquay.collection.*;

class TagStack
{
  private ArrayList _tags = new ArrayList();

  public final void add( String o )
  {
    _tags.add( o );
  }

  public final String remove()
  {
    return (String)_tags.remove();
  }

  public int size()
  {
    return _tags.size();
  }
}

public class XMLFormatter
{
  private TagStack _tagStack = new TagStack();
  private int _indentCharacters = 2;
  private Writer _writer;

  public XMLFormatter( Writer writer, int indentCharacters )
  {
    _writer = writer;
    _indentCharacters = indentCharacters;
  }


  private void indent( StringBuffer sb )
  {
    for( int i = 0; i < ( _tagStack.size() * _indentCharacters ); ++i )
    {
      sb.append( " " );
    }
  }

  private void appendStartTag( StringBuffer sb, String tag, XMLAttributes attributes )
  {
    sb.append( "<" ).append( tag );
    if( attributes != null )
    {
      for( int i = 0;  i < attributes.size(); ++i )
      {
        sb.append( " " );
        sb.append( attributes.get( i ).toString() );
      }
    }
    sb.append( ">" );
  }

  private void appendEndTag( StringBuffer sb, String tag )
  {
    sb.append( "</" ).append( tag ).append( ">" );
  }

  public void startTag( String tag ) throws IOException
  {
    startTag( tag, null );
  }

  public void startTag( String tag, XMLAttributes attributes ) throws IOException
  {
    StringBuffer sb = new StringBuffer();

    indent( sb );
    appendStartTag( sb, tag, attributes );
    sb.append( "\n" );

    _tagStack.add( tag );

    _writer.write( sb.toString() );
  }

  public void endTag() throws IOException
  {
    StringBuffer sb = new StringBuffer();

    String tag = (String)_tagStack.remove();

    indent( sb );
    appendEndTag( sb, tag );
    sb.append( "\n" );

    _writer.write( sb.toString() );
  }

  public void singleLineTag( String tag, String value ) throws IOException
  {
    singleLineTag( tag, value, null );
  }

  public void singleLineTag( String tag, String value,  XMLAttributes attributes ) throws IOException
  {
    StringBuffer sb = new StringBuffer();

    indent( sb );
    appendStartTag( sb, tag, attributes );
    sb.append( value );
    appendEndTag( sb, tag );
    sb.append( "\n" );

    _writer.write( sb.toString() );
  }

  public void singleLineTag( String tag ) throws IOException
  {
    singleLineTag( tag, (XMLAttributes)null );
  }

  public void singleLineTag( String tag, XMLAttributes attributes ) throws IOException
  {
    StringBuffer sb = new StringBuffer();

    indent( sb );
    sb.append( "<" );
    sb.append( tag );
    if( attributes != null )
    {
      for( int i = 0;  i < attributes.size(); ++i )
      {
        sb.append( " " );
        sb.append( attributes.get( i ).toString() );
      }
    }
    sb.append( "/>" );
    sb.append( "\n" );

    _writer.write( sb.toString() );
  }

  /**
   * Does not close the underlying stream
   */
  public void close() throws IOException
  {
    //_writer.close();
  }

  public void flush() throws IOException
  {
    _writer.flush();
  }
}
